#include <libgoa.h>
#include <glib-object.h>
gchar*
    g_object_get_string_R (GObject * gobject, const gchar * prop_name)
{
    gchar* _STRING;

    GParamSpec *pspec;

    pspec =
        g_object_class_find_property (G_OBJECT_GET_CLASS (gobject),
                                      prop_name);

    if (!pspec)
     {
         g_log (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
                "%s: GObject of type '%s' has no property '%s'", G_STRFUNC,
                G_OBJECT_TYPE_NAME (gobject), prop_name);
         return NULL;
     }

    if (pspec->value_type != G_TYPE_STRING)
     {
         g_log (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
                "%s: Property '%s' of GObject type '%s' is not of type STRING",
                G_STRFUNC, prop_name, G_OBJECT_TYPE_NAME (gobject));
         return NULL;
     }

    g_object_get (gobject, prop_name, &_STRING, NULL);

    return _STRING;
}
gint
    g_object_get_int_R (GObject * gobject, const gchar * prop_name)
{
    gint _INT;

    GParamSpec *pspec;

    pspec =
        g_object_class_find_property (G_OBJECT_GET_CLASS (gobject),
                                      prop_name);

    if (!pspec)
     {
         g_log (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
                "%s: GObject of type '%s' has no property '%s'", G_STRFUNC,
                G_OBJECT_TYPE_NAME (gobject), prop_name);
         return 0;
     }

    if (pspec->value_type != G_TYPE_INT)
     {
         g_log (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
                "%s: Property '%s' of GObject type '%s' is not of type INT",
                G_STRFUNC, prop_name, G_OBJECT_TYPE_NAME (gobject));
         return 0;
     }

    g_object_get (gobject, prop_name, &_INT, NULL);

    return _INT;
}
gboolean
    g_object_get_boolean_R (GObject * gobject, const gchar * prop_name)
{
    gboolean _BOOLEAN;

    GParamSpec *pspec;

    pspec =
        g_object_class_find_property (G_OBJECT_GET_CLASS (gobject),
                                      prop_name);

    if (!pspec)
     {
         g_log (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
                "%s: GObject of type '%s' has no property '%s'", G_STRFUNC,
                G_OBJECT_TYPE_NAME (gobject), prop_name);
         return FALSE;
     }

    if (pspec->value_type != G_TYPE_BOOLEAN)
     {
         g_log (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
                "%s: Property '%s' of GObject type '%s' is not of type BOOLEAN",
                G_STRFUNC, prop_name, G_OBJECT_TYPE_NAME (gobject));
         return FALSE;
     }

    g_object_get (gobject, prop_name, &_BOOLEAN, NULL);

    return _BOOLEAN;
}
gdouble
    g_object_get_double_R (GObject * gobject, const gchar * prop_name)
{
    gdouble _DOUBLE;

    GParamSpec *pspec;

    pspec =
        g_object_class_find_property (G_OBJECT_GET_CLASS (gobject),
                                      prop_name);

    if (!pspec)
     {
         g_log (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
                "%s: GObject of type '%s' has no property '%s'", G_STRFUNC,
                G_OBJECT_TYPE_NAME (gobject), prop_name);
         return 0;
     }

    if (pspec->value_type != G_TYPE_DOUBLE)
     {
         g_log (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
                "%s: Property '%s' of GObject type '%s' is not of type DOUBLE",
                G_STRFUNC, prop_name, G_OBJECT_TYPE_NAME (gobject));
         return 0;
     }

    g_object_get (gobject, prop_name, &_DOUBLE, NULL);

    return _DOUBLE;
}
