#ifndef BMP_PLAYLIST_H
#define BMP_PLAYLIST_H

#include <glib-object.h>
#include <gtk/gtk.h>

#include <time.h>

#include <libhrel/types.h>
#include <libhrel/tuple.h>
#include <libhrel/relation.h>

#include <bmp/metadata.h>
#include <bmp/playlist.h>
#include <bmp/plugin_interfaces.h>

#define BMP_TYPE_PLAYLIST		  (bmp_playlist_get_type ())
#define BMP_PLAYLIST(obj)		  (G_TYPE_CHECK_INSTANCE_CAST ((obj), BMP_TYPE_PLAYLIST, BmpPlaylist))
#define BMP_PLAYLIST_CLASS(klass)	  (G_TYPE_CHECK_CLASS_CAST ((klass), BMP_TYPE_PLAYLIST, BmpPlaylistClass))
#define BMP_IS_PLAYLIST(obj)	  	  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BMP_TYPE_PLAYLIST))
#define BMP_IS_PLAYLIST_CLASS(klass)	  (G_TYPE_CHECK_CLASS_TYPE ((klass), BMP_TYPE_PLAYLIST))
#define BMP_PLAYLIST_GET_CLASS(obj)	  (G_TYPE_INSTANCE_GET_CLASS ((obj), BMP_TYPE_PLAYLIST, BmpPlaylistClass))

typedef struct _BmpPlaylist BmpPlaylist;
typedef struct _BmpPlaylistClass BmpPlaylistClass;
typedef struct _BmpPlaylistPrivate BmpPlaylistPrivate;

struct _BmpPlaylist {
	GObject parent;

        BmpPlaylistPrivate *private;
};

struct _BmpPlaylistClass {
	GObjectClass parent;

	void (*tracklist_list_sorted)    (BmpPlaylist *self, gint i, gpointer array, gint n_items);
	void (*tracklist_items_removed)  (BmpPlaylist *self, gint i, gpointer array, gint n_items);
	void (*tracklist_items_added)    (BmpPlaylist *self, gint i, gpointer array, gint n_items);
};

gint
vcolumns_get_n_items  (void);

gint
vcolumns_get_item_nth (gint n);

#define TUPLE_COLUMN 7 
 
typedef struct _BmpPlaybackHistory BmpPlaybackHistory;
struct _BmpPlaybackHistory {
    GQueue	  *history;
    gint	   mark;
};


BmpPlaybackHistory*
bmp_playback_history_new		(void);

void
bmp_playback_history_free		(BmpPlaybackHistory	    *history);

void
bmp_playback_history_set		(BmpPlaybackHistory	    *history, 
					 gint			     position);

gint
bmp_playback_history_prev		(BmpPlaybackHistory	    *history,
					 BmpPluginFlowInterface	    *flow_plugin);

gint
bmp_playback_history_next		(BmpPlaybackHistory	    *history,
					 BmpPluginFlowInterface	    *flow_plugin);


void
bmp_playback_history_rewind		(BmpPlaybackHistory *history);
 
/* END BMPx Playback History Stuff */
GType
bmp_playlist_get_type			(void);

BmpPlaylist*
bmp_playlist_new			(void);

void
bmp_playlist_setup			(BmpPlaylist *self);

GtkListStore*
bmp_playlist_get_playlist		(BmpPlaylist *self);

gint
bmp_playlist_get_row_by_guid		(BmpPlaylist *self, 
					 RowGUID     *guid);

BmpFileItem*
bmp_playlist_get_row_by_index		(BmpPlaylist  *self,
					 gint	       item);


gchar*
bmp_playlist_get_title_nth		(BmpPlaylist  *self,
					 gint	       index);
gchar*
bmp_playlist_get_current_title		(BmpPlaylist  *self);


gchar*
bmp_playlist_get_uri_nth		(BmpPlaylist	  *self,
					 gint		   index);
gchar*
bmp_playlist_get_current_uri		(BmpPlaylist	  *self);



gint
bmp_playlist_get_length			(BmpPlaylist	  *self);



gboolean
bmp_playlist_tracklist_remove		(BmpPlaylist	  *self,
					 gint		   item);

void
bmp_playlist_tracklist_sort		(BmpPlaylist      *self,
					 gint		   metadatum_id);
	
gint
bmp_playlist_insert_tracklist_items	(BmpPlaylist	  *self,
					 const gchar     **uri_list);

gint
bmp_playlist_insert_tracklist_items_position (BmpPlaylist     *self,
					      const gchar    **uri_list,
					      gint	       _idx);

void
bmp_playlist_tracklist_remove_selected	    (BmpPlaylist      *self,
					     GList	      *path);

void
bmp_playlist_tracklist_remove_unselected    (BmpPlaylist      *self);

void
bmp_playlist_tracklist_remove_rows_all	    (BmpPlaylist      *self);

void
bmp_playlist_tracklist_remove_rows	    (BmpPlaylist      *self,
					     GArray	      *rows,
					     gint	       n_rows);

void
bmp_playlist_tracklist_remove_predicated (BmpPlaylist	  *self, 
					  HTuple	  *tuple_cmp,
					  BmpMetadatumId   metadatum_id);

void
bmp_playlist_tracklist_keep_predicated	 (BmpPlaylist	  *self, 
					  HTuple	  *tuple_cmp,
					  BmpMetadatumId   metadatum_id);
void
bmp_playlist_load_playlist     (BmpPlaylist *self);

void
bmp_playlist_sort_current_list (BmpPlaylist *self);

void
bmp_playlist_sort_current_list_by_tag (BmpPlaylist *self, BmpMetadatumId metadatum_id);

#endif
