#ifndef _RTFS_H_
#define _RTFS_H_

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdint.h>
#include <glib.h>

#include <libhrel/types.h>
#include <libhrel/tuple.h>
#include <libhrel/relation.h>
#include <libhrel/relation_base.h>

#define RTFS_BLOCKSIZE 512 /* in bytes */ 
#define RTFS_INITIAL_SIZE 16 /* in blocks */ 

#define RTFS_BITMAP	  ".map"
#define RTFS_DISK	  ".dsk"
#define RTFS_DIRECTORY	  ".dir"

typedef struct _RTFS_DirEnt RTFS_DirEnt;
struct _RTFS_DirEnt {
    uintptr_t    n_blocks;
    uintptr_t   *chain;

    gchar      *name;
};

enum {
    P_BITMAP,
    P_DISK,
    P_DIRECTORY
};

typedef struct _RTFS RTFS;
struct _RTFS {
    HRelation	*relation;
    uintptr_t	 size;	    /* in blocks */
    FILE	*disk;
    GByteArray	*bitmap;
    GHashTable  *directory;

    gchar	  *name;

    gchar	  *path[3];
    struct stat	   stats[3];

    GHashTable	  *buffer;
    GHashTable	  *dirty_blocks;

    GMutex	  *lock_disk;
    GMutex	  *lock_bitmap;
    GMutex	  *lock_directory;

    gulong	   handler_id_insert;
    gulong	   handler_id_delete;
};

RTFS*
rtfs_open (HRelation	  *relation,
	   const gchar    *basepath,
	   const gchar	  *name,
	   gboolean	   preload);

gint
rtfs_sync	      (RTFS		*rtfs);

gint
rtfs_close	      (RTFS		*rtfs);

HTuple*
rtfs_read_entry	      (RTFS		*rtfs,
		       gpointer		 pkey); 

void
rtfs_read_filesystem  (RTFS		*rtfs);

#endif
