#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <glib/gi18n.h>

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>
#include <X11/Xlib.h>

#include "draw.h"

void
chroma_gdk_color_to_rgba (GdkColor *color, double *r, double *g, double *b, double *a)
{
  *r = color->red   / 65535.0;
  *g = color->green / 65535.0;
  *b = color->blue  / 65535.0;
  *a = 1;
}

void
chroma_draw_polygon (cairo_t *cr, GdkColor *color, double alpha, CCoords points[], gint n_points, gboolean fill)
{
	gint n;
	double r,g,b,a;

 	cairo_set_line_width (cr, 1.0);
	chroma_gdk_color_to_rgba (color, &r, &g, &b, &a);

	cairo_move_to (cr, points[0].x, points[0].y);

	for (n = 1; n < n_points; n++)
		cairo_line_to (cr, points[n].x, points[n].y);

	cairo_line_to (cr, points[0].x, points[0].y);
	cairo_set_source_rgba (cr, r, g, b, alpha);

	if (fill)
		cairo_fill (cr);
	else
		cairo_stroke (cr);

	cairo_fill (cr);
}

void
chroma_draw_rounded_rectangle_pattern (cairo_t *cr, 
					    double xc, 
					    double yc, 
					    double _w, 
					    double _h, 
					    cairo_pattern_t *cr_pattern, 
					    double line_width, 
					    gboolean fill, 
					    gdouble rounding_offset)
{
    cairo_set_line_width (cr, line_width);

    cairo_move_to (cr, xc, yc+(rounding_offset*2));

    cairo_curve_to (cr, 	xc, yc+rounding_offset,
				xc, yc+0.5,
				xc+rounding_offset, yc+0.5);

    cairo_line_to  (cr, xc+_w-(rounding_offset*2), yc+0.5);

    cairo_curve_to (cr, xc+_w-rounding_offset, yc+0.5,
			xc+_w, yc+0.5,
			xc+_w, yc+rounding_offset);

    cairo_line_to (cr,  xc+_w, yc+_h-(rounding_offset*2));

    cairo_curve_to (cr, xc+_w, yc+_h-rounding_offset,
		        xc+_w, yc+_h-0.5,
			xc+_w-rounding_offset, yc+_h-0.5);

    cairo_line_to (cr, xc+(rounding_offset*2), yc+_h-0.5);

    cairo_curve_to (cr, xc+rounding_offset, yc+_h-0.5,
			xc, yc+_h-0.5,
			xc, yc+_h-rounding_offset);


    cairo_close_path (cr);
    cairo_set_source (cr, cr_pattern); 

    if (fill)
      cairo_fill (cr);
    else
      cairo_stroke (cr);
}

void
chroma_draw_rounded_rectangle (cairo_t *cr, 
				    double xc, 
				    double yc, 
				    double _w, 
				    double _h, 
				    GdkColor *color, 
				    double alpha, 
				    double line_width, 
				    gboolean fill, 
				    gdouble rounding_offset)
{
    double r, g, b, a;

    cairo_set_line_width (cr, line_width);

    cairo_move_to (cr, xc, yc+(rounding_offset*2));

    cairo_curve_to (cr, 	xc, yc+rounding_offset,
				xc, yc+0.5,
				xc+rounding_offset, yc+0.5);

    cairo_line_to  (cr, xc+_w-(rounding_offset*2), yc+0.5);

    cairo_curve_to (cr, xc+_w-rounding_offset, yc+0.5,
			xc+_w, yc+0.5,
			xc+_w, yc+rounding_offset);

    cairo_line_to (cr,  xc+_w, yc+_h-(rounding_offset*2));

    cairo_curve_to (cr, xc+_w, yc+_h-rounding_offset,
		        xc+_w, yc+_h-0.5,
			xc+_w-rounding_offset, yc+_h-0.5);

    cairo_line_to (cr, xc+(rounding_offset*2), yc+_h-0.5);

    cairo_curve_to (cr, xc+rounding_offset, yc+_h-0.5,
			xc, yc+_h-0.5,
			xc, yc+_h-rounding_offset);


    cairo_close_path (cr);
    chroma_gdk_color_to_rgba (color, &r, &g, &b, &a);
    cairo_set_source_rgba (cr, r, g, b, alpha);

    if (fill)
      cairo_fill (cr);
    else
      cairo_stroke (cr);
}

void
chroma_draw_rounded_rectangle_aqua	   (cairo_t *cr, 
				    double xc, 
				    double yc, 
				    double _w, 
				    double _h, 
				    GdkColor *color, 
				    double alpha, 
				    double line_width, 
				    gboolean fill, 
				    gdouble rounding_offset)
{
    double r, g, b, a;

    chroma_draw_rounded_rectangle (cr, xc, yc, _w, _h, color, alpha, line_width, fill, rounding_offset);

    cairo_set_line_width (cr, line_width);
    cairo_move_to (cr, xc, yc+(_h/2.2));
    cairo_curve_to (cr, 	xc, yc+rounding_offset,
				xc, yc+0.5,
				xc+rounding_offset, yc+0.5);
    cairo_line_to  (cr, xc+_w-(rounding_offset*2), yc+0.5);
    cairo_curve_to (cr, xc+_w-rounding_offset, yc+0.5,
			xc+_w, yc+0.5,
			xc+_w, yc+rounding_offset);
    cairo_line_to (cr,  xc+_w, yc+(_h/2.2));

    cairo_close_path (cr);
    chroma_gdk_color_to_rgba (color, &r, &g, &b, &a);
    cairo_set_source_rgba (cr, r, g, b, alpha+0.06);

    if (fill)
      {
	cairo_fill (cr);
      }
    else
      {
	cairo_stroke (cr);
      }
}

void
chroma_draw_arrow   (cairo_t		*cr,
	        GdkRectangle	*rect,
	        GdkColor	*color,
	        GtkArrowType	 arrow_type)
{
    CCoords points[3];
    gint x,y,w,h;
    
    x = rect->x;
    y = rect->y;
    w = rect->width;
    h = rect->height;

    switch (arrow_type)
	{
	    case GTK_ARROW_UP:
	      {
	        points[0].x = x+(w/2);
	        points[0].y = y+5;

	        points[1].x = x+w-3;
	        points[1].y = y+(h/2)+2;

	        points[2].x = x+3;
	        points[2].y = y+(h/2)+2;

		chroma_draw_polygon (cr, color, 0.8, points, 3, TRUE);
	      }
	    break;

	    case GTK_ARROW_DOWN:
	      {
	        points[0].x = x+(w/2);
	        points[0].y = (y+h)-4;

	        points[1].x = x+w-3;
	        points[1].y = (y+h/2)-1;

	        points[2].x = x+3;
	        points[2].y = (y+h/2)-1;

		chroma_draw_polygon (cr, color, 0.8, points, 3, TRUE);
	      }
	    break;

	    default: break;
       }
}


