/*  libhrel - Holyrel library

    Copyright (C) 2005 Chong Kai Xiong <descender@phreaker.net>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA  02111-1307  USA.
*/

#include <config.h>

#include "attr_map.h"

HAttribute *
h_attr_new (const gchar *name,
            GType        type)
{
  HAttribute *attr;

  g_return_val_if_fail (name != NULL, NULL);
  g_return_val_if_fail (type != G_TYPE_NONE, NULL);

  attr = g_new (HAttribute, 1);

  if (attr)
    {
      attr->name = g_strdup (name);
      attr->type = type;
    }

  return attr;
}

HAttribute *
h_attr_copy (HAttribute *attr)
{
  return h_attr_new (attr->name, attr->type);
}

void
h_attr_free (HAttribute *attr)
{
  g_free (attr->name);
  g_free (attr);
}

HAttrMap *
h_attr_map_new (void)
{
  return g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
                                (GDestroyNotify) h_attr_free);
}

HAttribute *
h_attr_map_get (HAttrMap    *map,
                const gchar *name)
{
  return (HAttribute *) g_hash_table_lookup (map, name);
}

void
h_attr_map_insert (HAttrMap    *map,
                   HAttribute  *attr)
{
  g_hash_table_insert (map, attr->name, attr);
}

typedef struct
{
  HAttrFunc func;
  gpointer  data;
}
IterAttrArgs;

static void
iter_attr (const gchar  *attr_name,
           HAttribute   *attr,
           IterAttrArgs *args)
{
  args->func (attr, args->data);
}

void
h_attr_map_foreach (HAttrMap  *map,
                    HAttrFunc  func,
                    gpointer   user_data)
{
  IterAttrArgs args;

  args.func = func;
  args.data = user_data;

  g_hash_table_foreach (map, (GHFunc) iter_attr, &args);
}

void
h_attr_map_free (HAttrMap *map)
{
  g_hash_table_destroy (map);
}

guint
h_attr_map_get_size (HAttrMap *map)
{
  return g_hash_table_size (map);
}
