/*  libhrel - Holyrel library

    Copyright (C) 2005 Chong Kai Xiong <descender@phreaker.net>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA  02111-1307  USA.
*/

#include <config.h>
#include "avl_tree_index.h"

struct _HAvlTreeIndex
{
  GTree *tree;
};


HAvlTreeIndex *
h_avl_tree_index_new (GCompareDataFunc  compare_func,
                      gpointer          compare_func_data,
                      GDestroyNotify    key_destroy_func,
                      GDestroyNotify    value_destroy_func)
{
  HAvlTreeIndex *index;

  index = g_new (HAvlTreeIndex, 1);

  index->tree = g_tree_new_full (compare_func,
                                 compare_func_data,
                                 key_destroy_func,
                                 (GDestroyNotify) g_list_free);

  return index;
}

void
h_avl_tree_index_free (HAvlTreeIndex *index)
{
  g_return_if_fail (index != NULL);

  g_tree_destroy (index->tree);
  g_free (index);
}

void
h_avl_tree_insert (HAvlTreeIndex *index,
                   HTuple        *tuple,
                   gpointer       value)
{
  g_return_if_fail (index != NULL);
  g_return_if_fail (tuple != NULL);
}

gboolean
h_avl_tree_remove (HAvlTreeIndex *index,
                   HTuple        *tuple)
{
  g_return_val_if_fail (index != NULL, FALSE);
  g_return_val_if_fail (tuple != NULL, FALSE);

  return FALSE;
}

gpointer
h_avl_tree_lookup (HAvlTreeIndex *index,
                   HTuple        *tuple)
{
  g_return_val_if_fail (index != NULL, NULL);
  g_return_val_if_fail (tuple != NULL, NULL);

  return NULL;
}


void
h_avl_tree_foreach (HAvlTreeIndex  *index,
                    HIndexFunc      func,
                    gpointer        user_data)
{
  g_return_if_fail (index != NULL);
  g_return_if_fail (func  != NULL);
}
