/*  libhrel - Holyrel library

    Copyright (C) 2005 Chong Kai Xiong <descender@phreaker.net>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA  02111-1307  USA.
*/

#include <config.h>

#include "btree.h"

typedef struct _HBTreeNode HBTreeNode;

struct _HBTree
{
  GCompareDataFunc key_compare_func;
  gpointer         key_compare_data;
  GDestroyNotify   key_destroy_func;
  GDestroyNotify   value_destroy_func;
  GNode           *root;
};

struct _HBTreeNode
{
  gpointer key;
  gpointer value;
};

static HBTreeNode *
h_btree_node_new (gpointer key,
                  gpointer value)
{
  HBTreeNode *node;

  node = g_new (HBTreeNode, 1);

  node->key = key;
  node->value = value;

  return node;
}

static void
h_btree_node_free_1 (HBTreeNode *node,
                     HBTree     *tree)
{
  if (tree->key_destroy_func)
    (*tree->key_destroy_func) (node->key);

  if (tree->value_destroy_func)
    (*tree->value_destroy_func) (node->value);

  g_free (node);
}

static GNode *
h_btree_new_node (gpointer key,
                  gpointer value)
{
  return g_node_new (h_btree_node_new (key, value));
}

HBTree *
h_btree_new (GCompareDataFunc key_compare_func,
             gpointer         key_compare_data)
{
  return h_btree_new_full (key_compare_func, key_compare_data,
                           NULL, NULL);
}

HBTree *
h_btree_new_full (GCompareDataFunc key_compare_func,
                  gpointer         key_compare_data,
                  GDestroyNotify   key_destroy_func,
                  GDestroyNotify   value_destroy_func)
{
  HBTree *tree;

  g_return_val_if_fail (key_compare_func != NULL, NULL);

  tree = g_new (HBTree, 1);

  tree->key_compare_func = key_compare_func;
  tree->key_compare_data = key_compare_data;
  tree->key_destroy_func = key_destroy_func;
  tree->value_destroy_func = value_destroy_func;
  tree->root = NULL;

  return tree;
}

guint
h_btree_n_nodes (HBTree         *tree,
                 GTraverseFlags  flags)
{
  g_return_val_if_fail (tree != NULL, 0);

  return g_node_n_nodes (tree->root, flags);
}

guint
h_btree_height (HBTree *tree)
{
  g_return_val_if_fail (tree != NULL, 0);

  return g_node_max_height (tree->root);
}

void
h_btree_insert (HBTree   *tree,
                gpointer  key,
                gpointer  value)
{
  g_return_if_fail (tree != NULL);
}

gboolean
h_btree_delete (HBTree        *tree,
                gconstpointer  key)
{
  g_return_val_if_fail (tree != NULL, FALSE);

  return FALSE;
}

gpointer
h_btree_lookup (HBTree        *tree,
                gconstpointer  key)
{
  g_return_val_if_fail (tree != NULL, NULL);

  return NULL;
}

void
h_btree_foreach (HBTree        *tree,
                 GTraverseFunc  func,
                 gpointer       user_data)
{
  g_return_if_fail (tree != NULL);
  g_return_if_fail (func != NULL);
}

void
h_btree_destroy (HBTree *tree)
{
  g_return_if_fail (tree != NULL);
}
