/*  libhrel - Holyrel library

    Copyright (C) 2005 Chong Kai Xiong <descender@phreaker.net>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA  02111-1307  USA.
*/

#include <config.h>

#include "misc.h"

/**
 * h_va_list_to_slist:
 * @args: argument list
 *
 * Converts a #va_list into a #GSList.
 *
 * Return value: a #GSList
 **/
GSList *
h_va_list_to_slist (va_list args)
{
  GSList   *list = NULL;
  gpointer  arg;

  while (1)
    {
      arg = va_arg (args, gpointer);
      if (!arg)
        break;

      list = g_slist_append (list, arg);
    }

  return list;
}

/**
 * h_string_list_copy:
 * @list: list of strings to copy
 *
 * Returns a deep copy of @list.
 *
 * Return value: copy of @list
 **/
GSList *
h_string_list_copy (GSList *list)
{
  GSList *copy = NULL;
  GSList *iter;

  for (iter = list; iter; iter = iter->next)
    copy = g_slist_append (copy, g_strdup ((const gchar *) iter->data));

  return copy;
}

/**
 * h_string_list_free:
 * @list: string list to free
 *
 * Frees a #GSList of strings.
 **/
void
h_string_list_free (GSList *list)
{
  g_slist_foreach (list, (GFunc) g_free, NULL);
  g_slist_free (list);
}

/**
 * h_list_find_custom_with_data
 * @list: list to search
 * @data: data to look for
 * @func: comparator
 * @func_data: user data passed to @func
 *
 * A version of #g_list_find_custom() that takes a #GCompareDataFunc
 * as the comparator.
 *
 * Return value: the found #GList element, or %NULL if it is not found
 **/
GList *
h_list_find_custom_with_data (GList            *list,
                              gconstpointer     data,
                              GCompareDataFunc  func,
                              gpointer          func_data)
{
  GList *iter;

  for (iter = list; iter; iter = iter->next)
    {
      if ((*func) (iter->data, data, func_data) == 0)
        return iter;
    }

  return NULL;
}
