/*  libhrel - Holyrel library

    Copyright (C) 2005 Chong Kai Xiong <descender@phreaker.net>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA  02111-1307  USA.
*/

#include <config.h>

#include "stack.h"

struct _HStack
{
  GList *values;
};

/**
 * h_stack_new:
 *
 * Creates a new #HStack.
 *
 * Return value: the new #HStack
 **/
HStack *
h_stack_new (void)
{
  return g_new0 (HStack, 1);
}

/**
 * h_stack_free:
 * @stack: a #HStack
 *
 * Frees the memory allocated to @stack.
 **/
void
h_stack_free (HStack *stack)
{
  g_return_if_fail (stack != NULL);

  g_list_free (stack->values);
  g_free (stack);
}

/**
 * h_stack_push:
 * @stack: a #HStack
 * @data: data
 *
 * Pushes @data onto @stack.
 **/
void
h_stack_push (HStack    *stack,
            gpointer  data)
{
  g_return_if_fail (stack != NULL);

  stack->values = g_list_prepend (stack->values, data);
}

/**
 * h_stack_pop:
 * @stack: a #HStack
 *
 * Removes the top of @stack.
 **/
void
h_stack_pop (HStack *stack)
{
  GList *top;

  g_return_if_fail (stack != NULL);

  top = stack->values;
  stack->values = g_list_remove_link (stack->values, top);
  g_list_free_1 (top);
}

/**
 * h_stack_get_top:
 * @stack: a #HStack
 *
 * Returns the top of @stack.
 *
 * Return value: top data
 **/
gpointer
h_stack_get_top (HStack *stack)
{
  g_return_val_if_fail (stack != NULL, NULL);

  return stack->values->data;
}
