/*  BMPx - The Dumb Music Player
 *  Copyright (C) 2005 BMPx development team.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * The BMPx project hereby grant permission for non-gpl compatible GStreamer
 * plugins to be used and distributed together with GStreamer and BMPx. This
 * permission are above and beyond the permissions granted by the GPL license
 * BMPx is covered by.
 */

#include <config.h>

#include <stdio.h>
#ifndef __USE_BSD
# define __USE_BSD	/* Needed for mkdtemp () */
#endif

#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <strings.h>

#ifdef ENABLE_NLS
# include <libintl.h>
#endif

#include <bmp/util.h>
#include <bmp/uri.h>
#include <bmp/vfs.h>
#include <bmp/plugin.h>
#include <bmp/plugin_interfaces.h>

#include "libcontainer_folder.h"

#ifdef HAVE_FTS_H
#  include <fts.h>
#endif

static VFSHandle *handle = NULL;

gboolean
container_folder_can_process (BmpPluginContainerInterface *self, const gchar *uri)
{
    gchar *filename;

    filename = g_filename_from_uri (uri, NULL, NULL);

    if (!filename)
	return FALSE;

    if (g_file_test (filename, G_FILE_TEST_IS_DIR))
      {
	g_free (filename);
	return TRUE;
      }

    return FALSE;
}

gboolean
container_folder_handle_set (BmpPluginContainerInterface *self, VFSHandle *_handle)
{
    if (_handle == NULL)
	return FALSE;

    if (!_handle->valid)
	return FALSE;

    handle = _handle;

    return TRUE;
}

VFSHandle*
container_folder_handle_get (BmpPluginContainerInterface *self)
{
    return handle;
}

gboolean
container_folder_handle_remove (BmpPluginContainerInterface *self)
{
    handle = NULL;
    return TRUE;
}

gboolean
container_folder_handle_read (BmpPluginContainerInterface *self)
{
    GError *error = NULL;
    GList  *items = NULL;

    dir_foreach(handle->filename, (DirForeachFunc)collect_path, &items, &error);
    if (error)
      {
	    g_log (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,"%s: Error collecting path '%s': %s", G_STRLOC, handle->filename, error->message);
	    g_error_free (error);
	    error = NULL;

	    /* FIXME?: We proceed anyway and hope for the best */
      }

    for ( ; items; items = items->next)
	  {
	    gchar *uri = g_filename_to_uri (items->data, NULL, &error);

	    if (error)
	      {
		g_log (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
		    "%s: Error converting path '%s' to URI: %s", G_STRLOC, (gchar*)items->data, error->message);
		g_error_free (error);
		error = NULL;
	      }
	    else
	      {
#if 0
		GURI  *bmp_uri;
		gchar *uri_str; 
    
		bmp_uri = bmp_uri_new (uri);
		bmp_uri_unescape (bmp_uri);
		uri_str = bmp_uri_get_string (bmp_uri);	
		bmp_uri_delete (bmp_uri);
	
		handle->uri_list = g_list_append (handle->uri_list, uri_str);
		g_free (uri);
#endif
		handle->uri_list = g_list_append (handle->uri_list, uri);
	      }
	  }

	g_list_foreach (g_list_first(items), (GFunc) g_free, NULL);

	return TRUE;
}

/* Metadata IFace */
static gchar* _metadata[] = {
    "Folder Container Plugin",
    "Milosz Derezynski",
    "(C) GPL 2005",
    "http://bmpx.berlios.de",
    "internalerror@gmail.com",
};

static const gchar**
get_metadata (BmpPlugMetadataInterface *self)
{
    return (const gchar**)_metadata;
}

static void
metadata_init (gpointer         g_iface,
               gpointer         iface_data)
{
        BmpPlugMetadataInterface *klass = (BmpPlugMetadataInterface *)g_iface;

        klass->get_metadata = get_metadata;
}

static void
container_folder_interface_init (gpointer         g_iface,
                               gpointer         iface_data)
{
        BmpPluginContainerInterface *klass = (BmpPluginContainerInterface*)g_iface;

	klass->can_process     = container_folder_can_process;
	klass->handle_set      = container_folder_handle_set;
	klass->handle_get      = container_folder_handle_get;
	klass->handle_remove   = container_folder_handle_remove;
	klass->handle_read     = container_folder_handle_read;
	klass->handle_write    = NULL;
	klass->get_export_data = NULL;
}

GType plugin_container_folder_type = 0;

void
plugin_container_folder_register_type (GTypeModule *module)
{
	    static const GTypeInfo object_info = {
                        sizeof (BmpPluginContainerFolderClass),
                        NULL,   /* base_init */
                        NULL,   /* base_finalize */
                        NULL,   /* class_init */
                        NULL,   /* class_finalize */
                        NULL,   /* class_data */
                        sizeof (BmpPluginContainerFolder),
                        0,      /* n_preallocs */
                        NULL    /* instance_init */
                };

                plugin_container_folder_type = g_type_module_register_type (module,
					     G_TYPE_OBJECT,
					     "BmpPluginContainerFolderType",
					     &object_info, 0);

                static const GInterfaceInfo interface_container_folder_info = {
                        (GInterfaceInitFunc) container_folder_interface_init,    /* interface_init */
                        NULL,                                       /* interface_finalize */
                        NULL                                        /* interface_data */
                };

                static const GInterfaceInfo interface_metadata_info = {
                        (GInterfaceInitFunc) metadata_init,    /* interface_init */
                        NULL,                                       /* interface_finalize */
                        NULL                                        /* interface_data */
                };

                g_type_module_add_interface (module,
					     plugin_container_folder_type,
                                             BMP_TYPE_PLUG_METADATA,
                                             &interface_metadata_info);

                g_type_module_add_interface (module,
					     plugin_container_folder_type,
                                             BMP_TYPE_PLUGIN_CONTAINER,
                                             &interface_container_folder_info);


}
