/*  BMPx - The Dumb Music Player
 *  Copyright (C) 2005 BMPx development team.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * The BMPx project hereby grant permission for non-gpl compatible GStreamer
 * plugins to be used and distributed together with GStreamer and BMPx. This
 * permission are above and beyond the permissions granted by the GPL license
 * BMPx is covered by.
 */


#include <config.h>

#include <stdio.h>
#ifndef __USE_BSD
# define __USE_BSD	/* Needed for mkdtemp () */
#endif

#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <strings.h>

#ifdef ENABLE_NLS
# include <libintl.h>
#endif

#include <libxml/tree.h>
#include <libxml/parser.h>
#include <libxml/xmlreader.h>
#include <libxml/xpath.h>
#include <libxml/xpathInternals.h>

#include <revision.h>

#include <bmp/objects.h>
#include <bmp/metadata.h>
#include <bmp/util.h>
#include <bmp/uri.h>
#include <bmp/vfs.h>
#include <bmp/plugin.h>
#include <bmp/plugin_interfaces.h>

#include <xcs/xml.h>

#include <libhrel/types.h>
#include <libhrel/tuple.h>

#include "libcontainer_query.h"

#ifdef HAVE_FTS_H
#  include <fts.h>
#endif

static VFSHandle *handle = NULL;

gboolean
container_query_can_process (BmpPluginContainerInterface *self, const gchar *uri)
{
    if (!g_ascii_strncasecmp("query:",uri,6))
	return TRUE;

    return FALSE;
}


gboolean
container_query_handle_set (BmpPluginContainerInterface *self, VFSHandle *_handle)
{
    if (_handle == NULL) 
	return FALSE;
    
    if (!_handle->valid)
	return FALSE;

    handle = _handle;

    return TRUE;
}

VFSHandle*
container_query_handle_get (BmpPluginContainerInterface *self)
{
    return handle;
}

gboolean
container_query_handle_remove (BmpPluginContainerInterface *self)
{
    handle = NULL;

    return TRUE;
}

static void
append_to_uri_list (HTuple *tuple, GList **list)
{
    HTupleElem *elem;

    elem = h_tuple_get (tuple, bmp_metadata_get_id_static (BMP_DATUM_LOCATION));

    *list = g_list_append (*list, g_strdup (elem->value));
}

gboolean 
container_query_handle_read (BmpPluginContainerInterface *self)
{
    GList      *uri_list = NULL;
    BMPURI       *uri;
    Query      *query;
    HRelation  *relation;

    gchar     **f_query, 
	      **f_predicate;

    gchar  *q	   = NULL; 
    gint    n = 0;

    g_return_val_if_fail (handle != NULL, FALSE);
    g_return_val_if_fail (VFS_HANDLE_VALID(handle), FALSE);

    //FIMXE: There is no real error checking here, we just assume the URI is not malformed

    uri = bmp_uri_new (handle->uri);
    bmp_uri_unescape (uri);
    q = uri->query; 

    query = bmp_metadata_cache_query_initialize (_bmp_metadata_cache);

    f_query = g_strsplit (q, "&", -1);

    while (f_query[n])
      {
	  BmpMetadatumId tag_id;
	  GType		 type;
	  gint		 v_int;

	  f_predicate = g_strsplit (f_query[n], "=", -1);

	  tag_id = bmp_metadata_get_id_by_name (f_predicate[0]);
	  type = bmp_metadata_get_type (tag_id);

	  switch (type)
	    {
		case G_TYPE_INT:
		  {
		      v_int = strtol (f_predicate[1], NULL, 10);
		      bmp_metadata_cache_query_append_int (_bmp_metadata_cache, query, tag_id, TRUE, v_int); 
		  }
		  break;

		case G_TYPE_STRING:
		  {
		      bmp_metadata_cache_query_append_string (_bmp_metadata_cache, query, tag_id, TRUE, f_predicate[1]); 
		  }
		  break;

		default: /** ERROR */ break;
	    }

	  g_strfreev (f_predicate);
	  n++;
      }

      g_strfreev (f_query);

    relation = bmp_metadata_cache_query_run (_bmp_metadata_cache, query);
    g_object_ref (relation);

    relation = h_relation_sort (relation,
				(GCompareDataFunc) h_tuple_compare_elem, 
				(gpointer) bmp_metadata_get_id_static (BMP_DATUM_TRACK));
    g_object_ref (relation);

    relation = h_relation_sort (relation,
				(GCompareDataFunc) h_tuple_compare_elem, 
				(gpointer) bmp_metadata_get_id_static (BMP_DATUM_ALBUM));

    h_relation_for_each_tuple (relation, (HTupleFunc)append_to_uri_list, &uri_list);

    h_relation_unref (relation);

    bmp_uri_delete (uri);
    handle->uri_list = uri_list;

    return TRUE;
}

static void
container_query_interface_init (gpointer         g_iface,
                               gpointer         iface_data)
{
        BmpPluginContainerInterface *klass = (BmpPluginContainerInterface *)g_iface;

	klass->can_process     = container_query_can_process;
	klass->handle_set      = container_query_handle_set;
	klass->handle_get      = container_query_handle_get;
	klass->handle_remove   = container_query_handle_remove;
	klass->handle_read     = container_query_handle_read;
	klass->handle_write    = NULL; 
	klass->get_export_data = NULL; 
}

/* Metadata IFace */
static const gchar**
get_metadata (BmpPlugMetadataInterface *self)
{
  static gchar* _metadata[] = {
    "Query Container (virtual)",
    "Milosz Derezynski",
    "(C) GPL 2005",
    "http://bmpx.berlios.de",
    "internalerror@gmail.com",
  };

  return (const gchar**)_metadata;
}

static void
metadata_init (gpointer         g_iface,
               gpointer         iface_data)
{
        BmpPlugMetadataInterface *klass = (BmpPlugMetadataInterface *)g_iface;

        klass->get_metadata = get_metadata;
}

GType plugin_container_query_type = 0;

void 
plugin_container_query_register_type (GTypeModule *module)
{
	    static const GTypeInfo object_info = {
                        sizeof (BmpPluginContainerQueryClass),
                        NULL,   /* base_init */
                        NULL,   /* base_finalize */
                        NULL,   /* class_init */
                        NULL,   /* class_finalize */
                        NULL,   /* class_data */
                        sizeof (BmpPluginContainerQuery),
                        0,      /* n_preallocs */
                        NULL    /* instance_init */
                };

                plugin_container_query_type = g_type_module_register_type (module,
					     G_TYPE_OBJECT,
					     "BmpPluginContainerQueryType",
					     &object_info, 0);

                static const GInterfaceInfo interface_container_query_info = {
                        (GInterfaceInitFunc) container_query_interface_init,    /* interface_init */
                        NULL,                                       /* interface_finalize */
                        NULL                                        /* interface_data */
                };

                static const GInterfaceInfo interface_metadata_info = {
                        (GInterfaceInitFunc) metadata_init,    /* interface_init */
                        NULL,                                       /* interface_finalize */
                        NULL                                        /* interface_data */
                };

                g_type_module_add_interface (module,
					     plugin_container_query_type,
                                             BMP_TYPE_PLUG_METADATA,
                                             &interface_metadata_info);


                g_type_module_add_interface (module,
					     plugin_container_query_type,
                                             BMP_TYPE_PLUGIN_CONTAINER,
                                             &interface_container_query_info);
}
