/*  BMPx - The Dumb Music Player
 *  Copyright (C) 2005 BMPx development team.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * The BMPx project hereby grant permission for non-gpl compatible GStreamer
 * plugins to be used and distributed together with GStreamer and BMPx. This
 * permission are above and beyond the permissions granted by the GPL license
 * BMPx is covered by.
 */


#include <config.h>

#include <stdio.h>
#ifndef __USE_BSD
# define __USE_BSD	/* Needed for mkdtemp () */
#endif

#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <strings.h>

#ifdef ENABLE_NLS
# include <libintl.h>
#endif

#include <libxml/tree.h>
#include <libxml/parser.h>
#include <libxml/xmlreader.h>
#include <libxml/xpath.h>
#include <libxml/xpathInternals.h>

#include <revision.h>

#include <bmp/objects.h>
#include <bmp/metadata.h>
#include <bmp/util.h>
#include <bmp/uri.h>
#include <bmp/vfs.h>
#include <bmp/plugin.h>
#include <bmp/plugin_interfaces.h>

#include <xcs/xml.h>

#include <libhrel/types.h>
#include <libhrel/tuple.h>

#include "libcontainer_xspf.h"

#ifdef HAVE_FTS_H
#  include <fts.h>
#endif

static VFSHandle *handle = NULL;

gboolean
container_xspf_can_process (BmpPluginContainerInterface *self, const gchar *uri)
{
    BMPURI *bmp_uri;
    gboolean rv;

    bmp_uri = bmp_uri_new (uri);
    rv = FALSE;
    
    if (str_has_suffix_nocase (bmp_uri->path, "xspf")) rv = TRUE;
    bmp_uri_delete (bmp_uri);

    return rv;
}


gboolean
container_xspf_handle_set (BmpPluginContainerInterface *self, VFSHandle *_handle)
{
    if (_handle == NULL) 
	return FALSE;
    
    if (!_handle->valid)
	return FALSE;

    handle = _handle;

    return TRUE;
}

VFSHandle*
container_xspf_handle_get (BmpPluginContainerInterface *self)
{
    return handle;
}

gboolean
container_xspf_handle_remove (BmpPluginContainerInterface *self)
{
    handle = NULL;

    return TRUE;
}

gboolean 
container_xspf_handle_write (BmpPluginContainerInterface *self)
{

#define UC(x) ((unsigned char*)x)

#define XSPF_ROOT_NODE_NAME "playlist"
#define XSPF_XMLNS "http://xspf.org/ns/0/"

    GList *iter_list = NULL;
    GError *error = NULL;
    xmlDocPtr doc;
    xmlNodePtr node_playlist,
	       node_tracklist,
	       node_track,
	       node_location,
	       node;
    int size;

    g_return_val_if_fail (handle != NULL, FALSE);
    g_return_val_if_fail (VFS_HANDLE_VALID(handle), FALSE);

    if (handle->buffer) 
      g_free (handle->buffer);

    handle->buffer = NULL;

    doc = 
      xmlNewDoc(UC("1.0")); 
    node_playlist = 
      xmlNewNode (NULL, (UC(XSPF_ROOT_NODE_NAME)));

    xmlSetProp (node_playlist, UC("version"), UC("1"));
    xmlSetProp (node_playlist, UC("xmlns"), UC("http://xspf.org/ns/0/"));
    xmlDocSetRootElement (doc, node_playlist);

    node =
      xmlNewNode (NULL, UC("creator"));
    xmlAddChild (node, xmlNewText(BAD_CAST "BMPx " RV_SNAPSHOT_VERSION));
    xmlAddChild (node_playlist, node);

    node_tracklist =
      xmlNewNode (NULL, UC("trackList"));
    xmlAddChild (node_playlist, node_tracklist);

    iter_list = handle->uri_list;

    for ( ; iter_list ; iter_list = iter_list->next) 
      {
	HTuple	    *tuple;
	HTupleElem  *elem;
	BMPURI	    *uri;
	gchar	    *uri_str;

	uri = bmp_uri_new ((gchar *)iter_list->data);
	bmp_uri_escape (uri);
	uri_str = bmp_uri_get_string (uri);

	node_track =
	  xmlNewNode (NULL, UC("track"));
	node_location =
	  xmlNewNode (NULL, UC("location"));
	xmlAddChild (node_location, xmlNewText(BAD_CAST uri_str) );
	xmlAddChild (node_track, node_location); 
	xmlAddChild (node_tracklist, node_track);
	g_free (uri_str);
	bmp_uri_delete (uri);

#if 0
	tuple = bmp_metadata_cache_get_metadata (_bmp_metadata_cache, iter_list->data);

	if (tuple)
	  {
	    elem = h_tuple_get (tuple, bmp_metadata_get_id_static (BMP_DATUM_ARTIST));
	    node = 
	      xmlNewNode (NULL, UC("creator")); 
	    xmlAddChild (node, xmlNewText(UC(elem->value)));
	    xmlAddChild (node_track, node);

	    elem = h_tuple_get (tuple, bmp_metadata_get_id_static (BMP_DATUM_ALBUM));
	    node = 
	      xmlNewNode (NULL, UC("album"));
	    xmlAddChild (node, xmlNewText(UC(elem->value)));
	    xmlAddChild (node_track, node);

	    elem = h_tuple_get (tuple, bmp_metadata_get_id_static (BMP_DATUM_TITLE));
	    node = 
	      xmlNewNode (NULL, UC("title"));
	    xmlAddChild (node, xmlNewText(UC(elem->value)));
	    xmlAddChild (node_track, node);
	  }
#endif
      }

    xmlKeepBlanksDefault(0);
    {
      xmlChar *buf;
      xmlDocDumpFormatMemoryEnc (doc, &buf, &size, "UTF-8", 1);
      handle->buffer = (gchar*)buf;
    }

    if (!g_file_set_contents (handle->filename, handle->buffer, strlen(handle->buffer), &error)) 
      { 
	g_log (G_LOG_DOMAIN, G_LOG_LEVEL_CRITICAL, "%s: Unable to save XPSF '%s': %s", G_STRLOC, handle->filename, error->message);
	g_error_free (error);
	error = NULL;
	return FALSE;
      }

    return TRUE;
}

gboolean 
container_xspf_handle_read (BmpPluginContainerInterface *self)
{
    GList		*uri_list = NULL;
    xmlXPathObjectPtr	 xpathObj;
    xmlNodeSetPtr	 nv;
    xmlDocPtr		 doc;
    gint		 n;

    g_return_val_if_fail (handle != NULL, FALSE);
    g_return_val_if_fail (VFS_HANDLE_VALID(handle), FALSE);

    handle->uri_list = NULL;

    if ((!handle->buffer) || (!strlen(handle->buffer)))    
      {
	return TRUE;
      }

    doc = xmlParseDoc (BAD_CAST handle->buffer);
    if (!doc)
      {
	g_log (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING, "%s: No document", G_STRLOC);
	return FALSE;
      }

    xpathObj = xml_execute_xpath_expression(doc, BAD_CAST "//xspf:location", BAD_CAST "xspf=http://xspf.org/ns/0/"); 
    if (!xpathObj) 
      {
	g_log (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING, "%s: XPath expression yields no result", G_STRLOC);
        return FALSE;
      }

    nv = xpathObj->nodesetval;
    if (!nv)
      {
	g_log (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING, "%s: XPath expression result contains no nodes", G_STRLOC);
	return FALSE;
      }

    for (n = 0; n < nv->nodeNr; n++)
      {
	BMPURI	*uri;
	gchar	*uri_tmp;

	uri_tmp = (gchar*)XML_GET_CONTENT(nv->nodeTab[n]->children);
	uri = bmp_uri_new (uri_tmp);
	bmp_uri_unescape (uri);

	uri_list = g_list_append (uri_list, bmp_uri_get_string (uri));
	bmp_uri_delete (uri);
      }

    xmlXPathFreeObject (xpathObj);
    handle->uri_list = uri_list;

    return TRUE;
}

const gchar**
container_xspf_get_export_data (BmpPluginContainerInterface *self)
{
	static gchar* _export_data[] = {
		  "XSPF Playlist", // Description
		  "xspf"	   // Default suffix
	};

	return (const gchar**)_export_data;
}

static void
container_xspf_interface_init (gpointer         g_iface,
                               gpointer         iface_data)
{
        BmpPluginContainerInterface *klass = (BmpPluginContainerInterface *)g_iface;

	klass->can_process     = container_xspf_can_process;
	klass->handle_set      = container_xspf_handle_set;
	klass->handle_get      = container_xspf_handle_get;
	klass->handle_remove   = container_xspf_handle_remove;
	klass->handle_read     = container_xspf_handle_read;
	klass->handle_write    = container_xspf_handle_write;
	klass->get_export_data = container_xspf_get_export_data;
}

/* Metadata IFace */
static const gchar**
get_metadata (BmpPlugMetadataInterface *self)
{
  static gchar* _metadata[] = {
    "XSPF Playlist Plugin",
    "Milosz Derezynski",
    "(C) GPL 2005",
    "http://bmpx.berlios.de",
    "internalerror@gmail.com",
  };

  return (const gchar**)_metadata;
}

static void
metadata_init (gpointer         g_iface,
               gpointer         iface_data)
{
        BmpPlugMetadataInterface *klass = (BmpPlugMetadataInterface *)g_iface;

        klass->get_metadata = get_metadata;
}

GType plugin_container_xspf_type = 0;

void 
plugin_container_xspf_register_type (GTypeModule *module)
{
	    static const GTypeInfo object_info = {
                        sizeof (BmpPluginContainerXSPFClass),
                        NULL,   /* base_init */
                        NULL,   /* base_finalize */
                        NULL,   /* class_init */
                        NULL,   /* class_finalize */
                        NULL,   /* class_data */
                        sizeof (BmpPluginContainerXSPF),
                        0,      /* n_preallocs */
                        NULL    /* instance_init */
                };

                plugin_container_xspf_type = g_type_module_register_type (module,
					     G_TYPE_OBJECT,
					     "BmpPluginContainerXSPFType",
					     &object_info, 0);

                static const GInterfaceInfo interface_container_xspf_info = {
                        (GInterfaceInitFunc) container_xspf_interface_init,    /* interface_init */
                        NULL,                                       /* interface_finalize */
                        NULL                                        /* interface_data */
                };

                static const GInterfaceInfo interface_metadata_info = {
                        (GInterfaceInitFunc) metadata_init,    /* interface_init */
                        NULL,                                       /* interface_finalize */
                        NULL                                        /* interface_data */
                };

                g_type_module_add_interface (module,
					     plugin_container_xspf_type,
                                             BMP_TYPE_PLUG_METADATA,
                                             &interface_metadata_info);


                g_type_module_add_interface (module,
					     plugin_container_xspf_type,
                                             BMP_TYPE_PLUGIN_CONTAINER,
                                             &interface_container_xspf_info);
}
