/*  BMPx - The Dumb Music Player
 *  Copyright (C) 2005 BMPx development team.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * The BMPx project hereby grant permission for non-gpl compatible GStreamer
 * plugins to be used and distributed together with GStreamer and BMPx. This
 * permission are above and beyond the permissions granted by the GPL license
 * BMPx is covered by.
 */

#include <config.h>

#include <stdio.h>
#ifndef __USE_BSD
# define __USE_BSD	/* Needed for mkdtemp () */
#endif

#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <strings.h>

#ifdef ENABLE_NLS
# include <libintl.h>
#endif

#include <bmp/uri.h>
#include <bmp/vfs.h>
#include <bmp/plugin.h>
#include <bmp/plugin_interfaces.h>

#include "libtransport_cdda.h"

#ifdef HAVE_FTS_H
#  include <fts.h>
#endif

static VFSHandle *handle = NULL;

gboolean
transport_cdda_can_process (BmpPluginTransportInterface *self, const gchar *uri)
{
    if (!g_ascii_strncasecmp("cdda",uri,4))
	return TRUE;

    return FALSE;
}

gboolean
transport_cdda_handle_set (BmpPluginTransportInterface *self, VFSHandle *_handle)
{
    if (_handle == NULL) 
	return FALSE;
    
    if (!_handle->valid)
	return FALSE;

    handle = _handle;

    return TRUE;
}

VFSHandle*
transport_cdda_handle_get (BmpPluginTransportInterface *self)
{
    return handle;
}

gboolean
transport_cdda_handle_remove (BmpPluginTransportInterface *self)
{
    handle = NULL;

    return TRUE;
}

gboolean
transport_cdda_handle_read (BmpPluginTransportInterface *self)
{
    handle->uri_list = g_list_append (handle->uri_list, g_strdup(handle->uri));
    handle->need_container = FALSE;
    return TRUE;
}

static void
transport_cdda_interface_init (gpointer         g_iface,
                               gpointer         iface_data)
{
        BmpPluginTransportInterface *klass = (BmpPluginTransportInterface *)g_iface;

	klass->can_process    = transport_cdda_can_process;
	klass->handle_set     = transport_cdda_handle_set;
	klass->handle_get     = transport_cdda_handle_get;
	klass->handle_remove  = transport_cdda_handle_remove;
	klass->handle_read    = transport_cdda_handle_read;
}

GType plugin_transport_cdda_type = 0;

void 
plugin_transport_cdda_register_type (GTypeModule *module)
{
	    static const GTypeInfo object_info = {
                        sizeof (BmpPluginTransportCDDAClass),
                        NULL,   /* base_init */
                        NULL,   /* base_finalize */
                        NULL,   /* class_init */
                        NULL,   /* class_finalize */
                        NULL,   /* class_data */
                        sizeof (BmpPluginTransportCDDA),
                        0,      /* n_preallocs */
                        NULL    /* instance_init */
                };

                static const GInterfaceInfo interface_transport_cdda_info = {
                        (GInterfaceInitFunc) transport_cdda_interface_init,    /* interface_init */
                        NULL,                                       /* interface_finalize */
                        NULL                                        /* interface_data */
                };

                plugin_transport_cdda_type = g_type_module_register_type (module,
					     G_TYPE_OBJECT,
					     "BmpPluginTransportCDDAType",
					     &object_info, 0);

                g_type_module_add_interface (module,
					     plugin_transport_cdda_type,
                                             BMP_TYPE_PLUGIN_TRANSPORT,
                                             &interface_transport_cdda_info);
}
