/*  BMPx - The Dumb Music Player
 *  Copyright (C) 2005 BMPx development team.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * The BMPx project hereby grant permission for non-gpl compatible GStreamer
 * plugins to be used and distributed together with GStreamer and BMPx. This
 * permission are above and beyond the permissions granted by the GPL license
 * BMPx is covered by.
 */


#include <config.h>

#include <stdio.h>
#ifndef __USE_BSD
# define __USE_BSD	/* Needed for mkdtemp () */
#endif

#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <strings.h>

#ifdef ENABLE_NLS
# include <libintl.h>
#endif

#include <bmp/uri.h>
#include <bmp/vfs.h>
#include <bmp/plugin.h>
#include <bmp/plugin_interfaces.h>

#include "libtransport_file.h"

#ifdef HAVE_FTS_H
#  include <fts.h>
#endif

static VFSHandle *handle = NULL;

gboolean
transport_file_can_process (BmpPluginTransportInterface *self, const gchar *uri)
{
    gchar *filename;

    filename = g_filename_from_uri (uri, NULL, NULL);
    
    if (!filename)
	return FALSE;

    if (g_file_test (filename, G_FILE_TEST_EXISTS))
      {
	g_free (filename);
	return TRUE;
      }
    
    g_free (filename);
    return FALSE;
}

gboolean
transport_file_handle_set (BmpPluginTransportInterface *self, VFSHandle *_handle)
{
    GError *error = NULL;

    if (_handle == NULL) 
	return FALSE;
    
    if (!_handle->valid)
	return FALSE;

    handle = _handle;

    handle->filename = g_filename_from_uri (handle->uri, NULL, &error);
    if (error)
      {
	    g_log (G_LOG_DOMAIN, G_LOG_LEVEL_CRITICAL,"%s: Unable to convert URI '%s' to filename: %s", G_STRLOC, handle->uri, error->message);
	    g_error_free (error);
	    error = NULL;
	    handle->valid = FALSE;
	    return FALSE;
      }

    return TRUE;
}

VFSHandle*
transport_file_handle_get (BmpPluginTransportInterface *self)
{
    return handle;
}

gboolean
transport_file_handle_remove (BmpPluginTransportInterface *self)
{
    handle = NULL;

    return TRUE;
}

gboolean
transport_file_handle_read (BmpPluginTransportInterface *self)
{
    if (g_file_test (handle->filename, G_FILE_TEST_IS_DIR))
	return TRUE;

    g_file_get_contents (handle->filename, &(handle->buffer), NULL, NULL);
    return TRUE;
}

static void
transport_file_interface_init (gpointer         g_iface,
                               gpointer         iface_data)
{
        BmpPluginTransportInterface *klass = (BmpPluginTransportInterface *)g_iface;

	klass->can_process    = transport_file_can_process;
	klass->handle_set     = transport_file_handle_set;
	klass->handle_get     = transport_file_handle_get;
	klass->handle_remove  = transport_file_handle_remove;
	klass->handle_read    = transport_file_handle_read;
}

static void
transport_file_instance_init (GTypeInstance   *instance,
			   gpointer         g_class)
{
}

GType plugin_transport_file_type = 0;

void 
plugin_transport_file_register_type (GTypeModule *module)
{
	    static const GTypeInfo object_info = {
                        sizeof (BmpPluginTransportFileClass),
                        NULL,   /* base_init */
                        NULL,   /* base_finalize */
                        NULL,   /* class_init */
                        NULL,   /* class_finalize */
                        NULL,   /* class_data */
                        sizeof (BmpPluginTransportFile),
                        0,      /* n_preallocs */
                        transport_file_instance_init    /* instance_init */
                };

                static const GInterfaceInfo interface_transport_file_info = {
                        (GInterfaceInitFunc) transport_file_interface_init,    /* interface_init */
                        NULL,                                       /* interface_finalize */
                        NULL                                        /* interface_data */
                };

                plugin_transport_file_type = g_type_module_register_type (module,
					     G_TYPE_OBJECT,
					     "BmpPluginTransportFileType",
					     &object_info, 0);

                g_type_module_add_interface (module,
					     plugin_transport_file_type,
                                             BMP_TYPE_PLUGIN_TRANSPORT,
                                             &interface_transport_file_info);
}
