#define ALSA_PCM_NEW_HW_PARAMS_API
#define ALSA_PCM_NEW_SW_PARAMS_API
#include <alsa/global.h>
#include <alsa/asoundlib.h>
#include <alsa/pcm_plugin.h>
#include <alsa/control.h>

#include <glib.h>

#include "alsa.h"

/* BOXED types for Card and Device */

//** Device
static BmpAlsaDevice*
alsa_device_copy (BmpAlsaDevice *item)
{
    BmpAlsaDevice *_copy;

    _copy = g_new0 (BmpAlsaDevice, 1);

    _copy->description = g_strdup (item->description);
    _copy->dev = g_strdup (item->dev);
    _copy->card_id = item->card_id;
    _copy->device_id = item->device_id;

    return _copy;
}

static void
alsa_device_free (BmpAlsaDevice *item)
{
    if (!item) return;

    g_free (item->description);
    g_free (item->dev);
    g_free (item);
}

GType
bmp_alsa_device_get_type (void)
{
    static GType our_type = 0;

    if (our_type == 0)
        our_type = g_boxed_type_register_static ("BmpAlsaDevice",
                                                 (GBoxedCopyFunc)
                                                 alsa_device_copy,
                                                 (GBoxedFreeFunc)
                                                 alsa_device_free);
    return our_type;
}

//** Card
static BmpAlsaCard*
alsa_card_copy (BmpAlsaCard *item)
{
    BmpAlsaCard *_copy;
    GList *iter;

    _copy = g_new0 (BmpAlsaCard, 1);

    _copy->description = g_strdup (item->description);
    _copy->card_id = item->card_id;
    _copy->devices = NULL;

    iter = item->devices;
    for ( ; iter ; iter = iter->next) 
      {
	  _copy->devices = g_list_append (_copy->devices, alsa_device_copy (iter->data));
      }

    return _copy;
}

static void
alsa_card_free (BmpAlsaCard *item)
{
    if (!item) return;

    g_free (item->description);
    g_free (item->dev);
    g_free (item);
}

GType
bmp_alsa_card_get_type (void)
{
    static GType our_type = 0;

    if (our_type == 0)
        our_type = g_boxed_type_register_static ("BmpAlsaCard",
                                                 (GBoxedCopyFunc)
                                                 alsa_card_copy,
                                                 (GBoxedFreeFunc)
                                                 alsa_card_free);
    return our_type;
}


GList*
bmp_alsa_get_cards (void)
{
   GList   *cards = NULL;
   gint	    card,
	    snd_err; 

   card = -1;

   snd_err = snd_card_next (&card);

   if (snd_err)
      {
	  g_critical ("%s: %s", G_STRLOC, snd_strerror (snd_err)); 
	  return NULL;
      }

   while (card > -1)
    {
      snd_pcm_info_t  *pcm_info;
      snd_ctl_t	      *ctl;
      gchar	       dev[64];
      gchar	      *name; 

      gint	       pcm_device;

      BmpAlsaCard     *bmp_alsa_card;

      bmp_alsa_card = g_new0 (BmpAlsaCard, 1); 

      snd_card_get_name (card, &name);
      sprintf (dev, "hw:%d", card);

      snd_err = snd_ctl_open (&ctl, dev, 0);
      if (snd_err)
	{
	    g_critical ("%s: %s", G_STRLOC, snd_strerror (snd_err)); 
	    break;
	}

      bmp_alsa_card->dev = g_strdup (dev);
      bmp_alsa_card->description = g_strdup (name);
      bmp_alsa_card->card_id = card;
      bmp_alsa_card->devices = NULL;

      pcm_device = -1;
      while (42)
	{
	    BmpAlsaDevice *bmp_alsa_device;

	    if (!ctl)
	      {
		  break;
	      }

	    snd_err = snd_ctl_pcm_next_device (ctl, &pcm_device);
	    if (snd_err)
	      {
		  g_critical ("%s: %s", G_STRLOC, snd_strerror (snd_err)); 
		  break;
	      }

	    if (pcm_device < 0) break; //FIXME: Do we really need this here still? 

	    snd_pcm_info_malloc (&pcm_info);
	    snd_pcm_info_set_device (pcm_info, pcm_device);
	    snd_pcm_info_set_subdevice (pcm_info, 0);
	    snd_pcm_info_set_stream (pcm_info, SND_PCM_STREAM_PLAYBACK);

	    snd_err = snd_ctl_pcm_info (ctl, pcm_info);
	    if (snd_err)
	      {
		  g_critical ("%s: %s", G_STRLOC, snd_strerror (snd_err)); 
		  break;
	      }

	    bmp_alsa_device = g_new0 (BmpAlsaDevice, 1); 

	    bmp_alsa_device->dev = g_strdup_printf ("hw:%d,%d", card, pcm_device); 
	    bmp_alsa_device->description = g_strconcat (name, ": ", snd_pcm_info_get_name (pcm_info), NULL);
	    bmp_alsa_device->card_id = card;
	    bmp_alsa_device->device_id = pcm_device;

	    bmp_alsa_card->devices = g_list_append (bmp_alsa_card->devices, bmp_alsa_device);

	    g_free (pcm_info);
	}

      cards = g_list_append (cards, bmp_alsa_card);

      if (ctl)
	{
	  snd_ctl_close (ctl);
	}

      snd_card_next (&card);
      if (card < 0 ) break;
    }

  return cards;
}
