/*  BMPx - The Dumb Music Player
 *  Copyright (C) 2005 BMPx development team.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id$
 *
 * The BMPx project hereby grant permission for non-gpl compatible GStreamer
 * plugins to be used and distributed together with GStreamer and BMPx. This
 * permission are above and beyond the permissions granted by the GPL license
 * BMPx is covered by.
 *
 * BMPx Bookmarks XBEL based backend code 
 *
 * (C) GPL 2005 M. Derezynski
 *
 * XBEL: http://pyxml.sourceforge.net/topics/xbel/docs/html/xbel.html
 *
 *
 * We use only the following structure:
 *
 * <xbel version="1.0">
 *    <folder>
 *	<title></title>
 *
 *	<bookmark href="">
 *	  <title></title>
 *	</bookmark>
 *
 *      ...
 *
 *   </folder>
 *
 *
 * based on xmlReader example:
 *
 * purpose: Demonstrate the use of xmlReaderForFile() to parse an XML file
 *          and dump the informations about the nodes found in the process.
 *          (Note that the XMLReader functions require libxml2 version later
 *          than 2.6.)
 * usage: reader1 <filename>
 * test: reader1 test2.xml > reader1.tmp ; diff reader1.tmp reader1.res ; rm reader1.tmp
 * author: Daniel Veillard
 * copy: see Copyright for the status of this software.
 */

#include <stdio.h>
#include <string.h>

#include <glib-object.h>
#include <xcs/xml.h>

#include "main.h"

static GtkListStore *bookmarks = NULL;

/* XBEL 1.0 standard entities */
#define XBEL_VERSION	    "1.0"
#define XBEL_DTD_NICK	    "xbel"
#define XBEL_DTD_SYSTEM	    "+//IDN python.org//DTD XML Bookmark " \
		"Exchange Language 1.0//EN//XML"

#define XBEL_DTD_URI	    "http://www.python.org/topics/xml/dtds/xbel-1.0.dtd"

#define XBEL_ROOT_ELEMENT	    "xbel"
#define XBEL_FOLDER_ELEMENT	    "folder"
#define XBEL_BOOKMARK_ELEMENT	    "bookmark"
#define XBEL_ALIAS_ELEMENT	    "alias"	
#define XBEL_SEPARATOR_ELEMENT	    "separator"
#define XBEL_TITLE_ELEMENT	    "title"
#define XBEL_DESC_ELEMENT	    "desc"
#define XBEL_INFO_ELEMENT	    "info"
#define XBEL_METADATA_ELEMENT	    "metadata"

#define XBEL_VERSION_ATTRIBUTE	    "version"
#define XBEL_FOLDED_ATTRIBUTE	    "folded"   
#define XBEL_OWNER_ATTRIBUTE	    "owner"
#define XBEL_ADDED_ATTRIBUTE	    "added"
#define XBEL_VISITED_ATTRIBUTE	    "visited"
#define XBEL_MODIFIED_ATTRIBUTE	    "modified"
#define XBEL_ID_ATTRIBUTE	    "id"
#define XBEL_HREF_ATTRIBUTE	    "href"
#define XBEL_REF_ATTRIBUTE	    "ref"	

#define XBEL_YES_VALUE		    "yes"
#define XBEL_NO_VALUE		    "no"

/* date and times inside XBEL entities should be expressed using ISO 8601
 * formats; we use the extended date and time format, relative to UTC
 */
#define ISO_8601_FORMAT "%Y-%m-%dT%H:%M:%SZ" /* strftime(3) format */
#define ISO_8601_LEN	21

void
bookmarks_load (void)
{
    GtkTreeIter		iter;
    xmlDocPtr		doc;
    xmlXPathObjectPtr	xpathobj;
    xmlNodeSetPtr	nv;
    xmlNodePtr		node;
    gchar	       *file;
    gint		n;

				       /* Name */     /* Comment */  /* URL */ 
    bookmarks = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);

    file = g_build_filename (BMP_PATH_USER_DIR, "bookmarks.xbel", NULL); 

    if (!g_file_test (file, G_FILE_TEST_EXISTS))
	{
	    g_free (file);
	    return;
	}

    doc = xmlParseFile (file);
    g_free (file);

    xpathobj = xml_execute_xpath_expression (doc, BAD_CAST ("/" XBEL_ROOT_ELEMENT "//" XBEL_BOOKMARK_ELEMENT), NULL);
    
    if (!xpathobj)  
	{
	    xmlFreeDoc (doc);
	    return;
	}

   if (!xpathobj->nodesetval)
	{
	    xmlFreeDoc (doc);
	    return;
	}

    nv = xpathobj->nodesetval;

    if (!nv->nodeNr) 
	{
	    xmlFreeDoc (doc);
	    return;
	}

    for (n = 0; n < nv->nodeNr; n++)
	{
	    enum {
		    STR_TITLE,
		    STR_HREF,
		    STR_DESC,

		    N_STRINGS
	    };

	    gchar	    *STR[N_STRINGS];
	    xmlNodePtr	     sibling;

	    STR[STR_HREF]    = NULL;
	    STR[STR_TITLE]   = NULL;
	    STR[STR_DESC]    = NULL;

	    node = nv->nodeTab[n];	    
	    sibling = node->children;

	    STR[STR_HREF] = (char *) xmlGetProp (node, BAD_CAST XBEL_HREF_ATTRIBUTE);
	    while (sibling)
		{
		    if (sibling->type == XML_ELEMENT_NODE)
			{
			   if ((!strcmp ((char *)sibling->name, (char *)XBEL_TITLE_ELEMENT)) && sibling->children)
			    {
				STR[STR_TITLE]	= (char *)XML_GET_CONTENT (sibling->children);
			    } 
			   else if ((!strcmp ((char *)sibling->name, (char *)XBEL_DESC_ELEMENT)) && sibling->children)
			    {
				STR[STR_DESC]	= (char *)XML_GET_CONTENT (sibling->children);
			    } 
			}

		    sibling = sibling->next;
		}

	    if (STR[STR_TITLE] && STR[STR_HREF] && STR[STR_DESC])
		{
		    gtk_list_store_append (bookmarks, &iter);
		    gtk_list_store_set (bookmarks, &iter, 0, STR[STR_TITLE], 1, STR[STR_DESC], 2, STR[STR_HREF], -1);
		}
	}

    xmlFreeDoc (doc);
}

GtkTreeModel*
bookmarks_get_treemodel (void)
{
    return GTK_TREE_MODEL(bookmarks);
}

static void
append_bookmark (xmlNodePtr	 NODE_PARENT,
		 GtkTreeModel	*model,
		 GtkTreeIter	*iter)
{
    enum {
	XC_TITLE,
	XC_HREF,
	XC_DESC,

	N_XC
    };

    enum {
	N_BOOKMARK,
	N_TITLE,
	N_DESC,
	N_TEXT,

	N_NODES
    };

    xmlNodePtr	 NODE[N_NODES];
    xmlChar	*XC[N_XC]; 
	
    gtk_tree_model_get (model, iter, 0, &XC[XC_TITLE], 1, &XC[XC_DESC], 2, &XC[XC_HREF], -1);
  
    /* Create <bookmark/> */
    NODE[N_BOOKMARK] =
      xmlNewNode (NULL, BAD_CAST XBEL_BOOKMARK_ELEMENT);

    /* Create <title/> */
    NODE[N_TITLE] = 
      xmlNewNode (NULL, BAD_CAST XBEL_TITLE_ELEMENT);
    /* Create <title/> text node/contents */
    NODE[N_TEXT] =
      xmlNewText (BAD_CAST XC[XC_TITLE]);
    /* Append <title/> to <bookmark/> */
    xmlAddChild (NODE[N_BOOKMARK], NODE[N_TITLE]); 
    /* Append #text to <title/> */
    xmlAddChild (NODE[N_TITLE], NODE[N_TEXT]);


    /* Create <desc/> */
    NODE[N_DESC] = 
      xmlNewNode (NULL, BAD_CAST XBEL_DESC_ELEMENT);
    /* Create <desc/> text node/contents */
    NODE[N_TEXT] =
      xmlNewText (BAD_CAST XC[XC_DESC]);
    /* Append <desc/> to <bookmark/> */
    xmlAddChild (NODE[N_BOOKMARK], NODE[N_DESC]); 
    /* Append #text to <desc/> */
    xmlAddChild (NODE[N_DESC], NODE[N_TEXT]);


    /* Set href attribute to <bookmark/> */
    xmlSetProp  (NODE[N_BOOKMARK], BAD_CAST XBEL_HREF_ATTRIBUTE, BAD_CAST XC[XC_HREF]);
    xmlAddChild (NODE_PARENT, NODE[N_BOOKMARK]);
}

void
bookmarks_save (void)
{
    xmlDocPtr	   doc;
    xmlNodePtr     NODE_PARENT; 

    GError        *error = NULL;

    GtkTreeModel  *bookmarks_model = GTK_TREE_MODEL(bookmarks);

    GtkTreeIter	   iter; 

    xmlChar	  *buffer;
    int		   buffer_size;

    gchar	  *bookmarks;

    gboolean	   proceed = gtk_tree_model_get_iter_first (bookmarks_model, &iter); 

    doc = xmlNewDoc (BAD_CAST "1.0");
    NODE_PARENT = xmlNewNode (NULL, BAD_CAST XBEL_ROOT_ELEMENT);
    xmlSetProp (NODE_PARENT, BAD_CAST "version", BAD_CAST "1.0");
    xmlDocSetRootElement (doc, NODE_PARENT);

    proceed = gtk_tree_model_get_iter_first (bookmarks_model, &iter);

    while (proceed)
      {
	  append_bookmark (NODE_PARENT, bookmarks_model, &iter); 
	  proceed = gtk_tree_model_iter_next (bookmarks_model, &iter);
      }

    xmlKeepBlanksDefault(0);
    xmlThrDefIndentTreeOutput(1);
    xmlDocDumpFormatMemory (doc, &buffer, &buffer_size, 1); 

    bookmarks = g_build_filename (BMP_PATH_USER_DIR, "bookmarks.xbel", NULL); 
    g_file_set_contents (bookmarks, (const gchar*)buffer, buffer_size, &error);

    if (error)
      {
	  g_log (G_LOG_DOMAIN, 
		 G_LOG_LEVEL_CRITICAL,
		 "%s: Unable to save bookmarks file '%s': %s", 
		 G_STRLOC, 
		 bookmarks, error->message);

	  g_error_free (error);
	  error = NULL;
      }

    xmlFreeDoc (doc);
    g_free (bookmarks);
    g_free (buffer);
} 
