/*  BMPx - The Dumb Music Player
 *  Copyright (C) 2005 BMPx development team.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id$
 *
 * The BMPx project hereby grant permission for non-gpl compatible GStreamer
 * plugins to be used and distributed together with GStreamer and BMPx. This
 * permission are above and beyond the permissions granted by the GPL license
 * BMPx is covered by.
 */

/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */

/* FIXME: */
#define PATH_MAX 4096

#include <glib.h>
#include <glib-object.h>
#include <fam.h>

#include <fam_monitor.h>

G_DEFINE_TYPE (FamMonitor, fam_monitor, G_TYPE_OBJECT)

typedef enum {
	FM_DELETED,
	FM_CREATED,
	FM_MOVED,

	FM_N_EVENTS
} BmpFamFileEventType;

#if 0
static gchar *event_names[] = {
	"deleted",
	"created",
	"moved"
};
#endif

typedef struct {
	gchar 		    *filename;
	BmpFamFileEventType  event;
} BmpFamFileEventInformation;

typedef enum {
	FAM_MONITOR_SIGNAL_1,
	FAM_MONITOR_N_SIGNALS
} FamMonitorSignals;

#if 0
static guint fam_monitor_signals[FAM_MONITOR_N_SIGNALS] = {0};
#endif

typedef enum {
	FAM_MONITOR_N_PROPERTIES
} FamMonitorProperties;

struct _FamMonitorPrivate {
        gboolean dispose_has_run;
	gboolean fam_running;
	GHashTable *watches;
};

static void
free_fam_watch (FamMonitorWatch *watch)
{
}

static void
fam_monitor_init (FamMonitor *this)
{
	FAMConnection *fc;

        this->private = g_new (FamMonitorPrivate, 1);
        this->private->dispose_has_run = FALSE;
	this->private->watches = g_hash_table_new_full(g_str_hash, g_str_equal,
                                                       (GDestroyNotify) g_free,
                                                       (GDestroyNotify) free_fam_watch);

	fc = g_new0(FAMConnection,1);

	if (FAMOpen(fc) == -1)
	  {
	    this->private->fam_running = FALSE;
	    FAMClose (fc);
	    g_free (fc);

	    g_message ("running: %s (FAM daemon not running: Not accepting watches!)", G_OBJECT_TYPE_NAME(this));
	  }
        else
          {
	    this->private->fam_running = TRUE;
	    FAMClose (fc);
	    g_free (fc);

	    g_message ("running: %s", G_OBJECT_TYPE_NAME(this));
	  }
}

static GObject *
fam_monitor_constructor (GType                  type,
                       guint                  n_construct_properties,
                       GObjectConstructParam *construct_properties)
{
        GObject *obj;

        {
                FamMonitorClass *klass;
                GObjectClass *parent_class;
                klass = FAM_MONITOR_CLASS (g_type_class_peek (FAM_TYPE_MONITOR));
                parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
                obj = parent_class->constructor (type,
                                                 n_construct_properties,
                                                 construct_properties);
        }

        return obj;
}

static void
fam_monitor_dispose (GObject *obj)
{
        FamMonitor *this = (FamMonitor *)obj;

        if (this->private->dispose_has_run) return;
        this->private->dispose_has_run = TRUE;

	g_message("stopped: %s", G_OBJECT_TYPE_NAME(obj));
}

static void
fam_monitor_finalize (GObject *obj)
{
        FamMonitor *this = (FamMonitor *)obj;

        g_free (this->private);
}

FamMonitor*
fam_monitor_new (void)
{
  FamMonitor *fam_monitor;

  fam_monitor = g_object_new (fam_monitor_get_type (), NULL);

  return fam_monitor;
}

static void
fam_monitor_class_init (FamMonitorClass *g_class)
{
        GObjectClass *gobject_class = G_OBJECT_CLASS (g_class);

        gobject_class->dispose = fam_monitor_dispose;
        gobject_class->finalize = fam_monitor_finalize;
        gobject_class->constructor = fam_monitor_constructor;
}

void
monitor_thread_func (FamMonitorWatch *fam_watch)
{
	FAMEvent *fe = g_new0(FAMEvent,1);
	gboolean exit_loop = FALSE;
	gboolean req_event = FALSE;

	while (42)
	  {

	   switch (FAMPending (fam_watch->fc))
             {

		case 1:
                  {

			BmpFamFileEventInformation *fi;

			FAMNextEvent(fam_watch->fc, fe);

			switch (fe->code)
                          {

					case FAMCreated:
                                          {
							gchar *path_fq;

							path_fq = g_build_filename (fam_watch->path_name, fe->filename, NULL);

							fi = g_new0(BmpFamFileEventInformation,1);
							fi->filename = path_fq; 
							fi->event = FM_CREATED;
							fam_watch->fl = g_list_append (fam_watch->fl, fi);
							req_event = TRUE;
					  } 
					break;

					case FAMDeleted:
                                          {
							gchar *path_fq;

							path_fq = g_build_filename (fam_watch->path_name, fe->filename, NULL);

							fi = g_new0(BmpFamFileEventInformation,1);
							fi->filename = path_fq; 
							fi->event = FM_DELETED;
							fam_watch->fl = g_list_append (fam_watch->fl, fi);
							req_event = TRUE;
					  }
					break;

					case FAMMoved:
                                          {
							gchar *path_fq;

							path_fq = g_build_filename (fam_watch->path_name, fe->filename, NULL);

							fi = g_new0(BmpFamFileEventInformation,1);
							fi->filename = path_fq; 
							fi->event = FM_MOVED;
							fam_watch->fl = g_list_append (fam_watch->fl, fi);
							req_event = TRUE;
					  } 
					break;

					case FAMStartExecuting: break;
					case FAMStopExecuting: break;
					case FAMAcknowledge: break;
					case FAMChanged: break;
					case FAMExists: break;
					case FAMEndExist: break;
			  }
	 	  }
		break;

		case 0:
		  {

			exit_loop = TRUE;
	   		g_usleep (50000);
		  }
		break;

		default: break;
	   }


	  if (exit_loop && req_event) break;
	  if (fam_watch->terminate) break;

	}

	g_free (fe);
	g_thread_exit (0);
}

static void
process_fam_events (BmpFamFileEventInformation *fi, FamMonitorWatch *watch)
{

	switch (fi->event)
	  {

		case FM_CREATED:
                  {

			if (watch->added_func)
				watch->added_func(fi->filename, watch->user_data);
		  } 
		break;

		case FM_DELETED: 
                  {

			if (watch->removed_func)
				watch->removed_func(fi->filename, watch->user_data);
		  }
		break;

		case FM_MOVED:
		case FM_N_EVENTS:
		break;
         }

    g_free (fi->filename);
}

static gboolean
process_message_queue (FamMonitorWatch *watch)
{
	GList *item = NULL;

	item = g_async_queue_try_pop (watch->queue);
	if (!item) return FALSE;

	g_list_foreach (item, (GFunc)process_fam_events, watch);

	if (g_async_queue_length(watch->queue) == 0)
		return FALSE;
	else
		return TRUE;
}

static void
monitor_thread (FamMonitorWatch *fam_watch)
{
	GThread *collector;
	GError *error = NULL;

	FAMOpen(fam_watch->fc);
	FAMMonitorDirectory(fam_watch->fc, fam_watch->path_name, fam_watch->fr, NULL);

		while (42)
                  {

			collector =
			    g_thread_create ((GThreadFunc)monitor_thread_func, fam_watch, TRUE, &error);

			if (error)
			  {
				g_message("%s: %s", G_STRLOC, error->message);
				g_error_free (error);
				error = NULL;
			  }

			g_thread_join (collector);
			g_async_queue_push (fam_watch->queue, g_list_copy(fam_watch->fl));
			g_idle_add ((GSourceFunc) process_message_queue, fam_watch);
			fam_watch->fl = NULL;

			if (fam_watch->terminate) break;
		  }

		FAMClose(fam_watch->fc);
		g_thread_exit (0);
}

void
fam_monitor_watch_new (FamMonitor	*this,
		       gchar		*monitor_name,
		       gchar		*path_name,
		       FamAddedFunc	added_func,
		       FamRemovedFunc	removed_func,
		       gpointer		user_data,
		       GDestroyNotify	destroy)
{
	FamMonitorWatch *fam_watch;
	GError *error = NULL;

	g_return_if_fail (FAM_IS_MONITOR(this));

	if (! this->private->fam_running)
	  {
	    g_message ("%s: Not accepting monitor '%s' for path '%s': FAM daemon not running.", __FILE__, monitor_name, path_name);
	    return;
	  }

	fam_watch = g_new0(FamMonitorWatch,1);
	fam_watch->added_func = added_func;
	fam_watch->removed_func = removed_func;
	fam_watch->fc = g_new0(FAMConnection,1);
	fam_watch->fr = g_new0(FAMRequest,1);
	fam_watch->fl = NULL;
	fam_watch->queue = g_async_queue_new();
	fam_watch->terminate = FALSE;
	fam_watch->monitor_name = g_strdup(monitor_name);
	fam_watch->path_name = g_strdup(path_name);
	fam_watch->user_data = user_data;
	fam_watch->destroy = destroy;

	g_hash_table_insert (this->private->watches, g_strdup(monitor_name), fam_watch);

        fam_watch->thread = g_thread_create ((GThreadFunc)monitor_thread, fam_watch, TRUE, &error);

	if (error)
	  {
		g_message("%s: %s", G_STRLOC, error->message);
		g_error_free (error);
		error = NULL;
	  } 
}

void
fam_monitor_watch_terminate (FamMonitor *this, gchar *monitor_name)
{
	FamMonitorWatch *watch;

	watch = g_hash_table_lookup (this->private->watches, monitor_name);

	if (!watch) return;

	watch->terminate = TRUE;
}
