/*  BMPx - The Dumb Music Player
 *  Copyright (C) 2005 BMPx development team.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id$
 *
 * The BMPx project hereby grant permission for non-gpl compatible GStreamer
 * plugins to be used and distributed together with GStreamer and BMPx. This
 * permission are above and beyond the permissions granted by the GPL license
 * BMPx is covered by.
 */

#ifndef _BMP_PLAY_H
#define _BMP_PLAY_H

#include <config.h>

#include <glib-object.h>
#include <libhrel/types.h>
#include <libhrel/tuple.h>

#define BMP_TYPE_PLAY		  (bmp_play_get_type ())
#define PLAY(obj)		  (G_TYPE_CHECK_INSTANCE_CAST ((obj), BMP_TYPE_PLAY, BmpPlay))
#define PLAY_CLASS(klass)	  (G_TYPE_CHECK_CLASS_CAST ((klass), BMP_TYPE_PLAY, BmpPlayClass))
#define BMP_IS_PLAY(obj)	  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BMP_TYPE_PLAY))
#define BMP_IS_PLAY_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), BMP_TYPE_PLAY))
#define PLAY_GET_CLASS(obj)	  (G_TYPE_INSTANCE_GET_CLASS ((obj), BMP_TYPE_PLAY, BmpPlayClass))

typedef struct _BmpPlay      BmpPlay;
typedef struct _BmpPlayClass BmpPlayClass;

struct _BmpPlay
{
  GObject parent;
};

struct _BmpPlayClass
{
  GObjectClass parent;

  void (*eos)		   (BmpPlay *self);
  void (*error)	           (BmpPlay *self);
  void (*buffer)	   (BmpPlay *self, gpointer data);
  void (*time_tick)        (BmpPlay *self, gint length, gint position);
  void (*title)		   (BmpPlay *self, gchar *text);
  void (*seek)		   (BmpPlay *self, gint position); 
};

typedef enum
{
  PLAYSTATUS_STOPPED = 1 << 1,
  PLAYSTATUS_PLAYING = 1 << 2,
  PLAYSTATUS_PAUSED  = 1 << 3,
  PLAYSTATUS_SEEKING = 1 << 4,
  PLAYSTATUS_WAITING = 1 << 5
} BmpPlayStatus;

typedef enum
{
  BMP_EQ_30HZ,
  BMP_EQ_60HZ,
  BMP_EQ_125HZ,
  BMP_EQ_250HZ,
  BMP_EQ_500HZ,
  BMP_EQ_1000HZ,
  BMP_EQ_2000HZ,
  BMP_EQ_4000HZ,
  BMP_EQ_8000HZ,
  BMP_EQ_16000HZ,

  BMP_N_EQ_PARAMS
} BmpEqEnum;

GType    bmp_play_get_type     (void);

BmpPlay *bmp_play_new          (void);

gint     bmp_play_get_length   (BmpPlay *self);

gint     bmp_play_get_status   (BmpPlay *self);

void     bmp_play_set_volume   (BmpPlay *self,
                                gint     volume);

gint     bmp_play_get_volume   (BmpPlay *self);

void	 bmp_play_reset_pipes  (BmpPlay *self);

gboolean bmp_play_is_audio_file (BmpPlay	*self,
				 const gchar	*uri);
 
#if 0
gchar     *bmp_play_get_suffixes      (BmpPlay     *self);
void       bmp_play_set_eq            (BmpPlay     *self,
                                       BmpEqEnum    eq,
                                       gint         value);
gboolean   bmp_play_stop_stream_fade  (BmpPlay     *self);

#endif 

GHashTable *bmp_play_metadata_get (BmpPlay     *self,
                                   const gchar *filename);

#endif /* PLAY_H */
