/*  BMPx - The Dumb Music Player
 *  Copyright (C) 2005 BMPx development team.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id$
 *
 * The BMPx project hereby grant permission for non-gpl compatible GStreamer
 * plugins to be used and distributed together with GStreamer and BMPx. This
 * permission are above and beyond the permissions granted by the GPL license
 * BMPx is covered by.
 */



/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */

#include <gtk/gtk.h>
#include <bmp/paramspecs.h>

#include "plugin_host.h"
#include "ui_util.h"

G_DEFINE_TYPE(BmpPluginHost, bmp_plugin_host, G_TYPE_OBJECT)

#include "main.h"

#if 0
typedef enum {
	BMP_PLUGIN_HOST_N_SIGNALS
} BmpPluginHostSignals;
static guint bmp_plugin_host_signals[BMP_PLUGIN_HOST_N_SIGNALS] = {};
#endif

typedef enum {
	BMP_PLUGIN_HOST_N_PROPERTIES
} BmpPluginHostProperties;

struct _BmpPluginHostPrivate
{
        gboolean dispose_has_run;

	GHashTable  *plugin_ui[BMP_N_PLUGIN_TYPES];
};

/* Value setters */
static void
value_changed_cb__cbox  (GtkComboBox		*cbox,
			 PluginHostPropertyData *pdata)  
{
    GObject *plugin;
    GValue   gvalue = {0,};
    gint     value;

    plugin = g_hash_table_lookup (plugins_table[pdata->type], pdata->name);
    value  = gtk_combo_box_get_active (cbox); 

    g_value_init (&gvalue, G_TYPE_INT);
    g_value_set_int (&gvalue, value);

    bmp_plug_property_set_prop (BMP_PLUG_PROPERTY(plugin), pdata->property, &gvalue);
}

static void
value_changed_cb__spinbutton  (GtkSpinButton		*spinbutton,
                                         PluginHostPropertyData *pdata)  
{
    GObject *plugin;
    GValue   gvalue = {0,};
    gint     value;

    plugin = g_hash_table_lookup (plugins_table[pdata->type], pdata->name);
    value  = gtk_spin_button_get_value_as_int (spinbutton); 

    g_value_init (&gvalue, G_TYPE_INT);
    g_value_set_int (&gvalue, value);

    bmp_plug_property_set_prop (BMP_PLUG_PROPERTY(plugin), pdata->property, &gvalue);
}

static void
value_changed_cb__entry  (GtkEntry		   *entry,
                                    PluginHostPropertyData *pdata)  
{
    GObject	  *plugin;
    GValue	   gvalue = {0,};
    const gchar   *value;

    plugin = g_hash_table_lookup (plugins_table[pdata->type], pdata->name);
    value  = gtk_entry_get_text (entry); 

    g_value_init (&gvalue, G_TYPE_STRING);
    g_value_set_string (&gvalue, value);

    bmp_plug_property_set_prop (BMP_PLUG_PROPERTY(plugin), pdata->property, &gvalue);
}

static void
value_changed_cb__checkbox  (GtkWidget	      *button,
				       PluginHostPropertyData *pdata)  
{
    GObject *plugin;
    GValue   gvalue = {0,};
    gboolean value; 

    plugin = g_hash_table_lookup (plugins_table[pdata->type], pdata->name);
    value  = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(button)); 

    g_value_init (&gvalue, G_TYPE_BOOLEAN);
    g_value_set_boolean (&gvalue, value);

    bmp_plug_property_set_prop (BMP_PLUG_PROPERTY(plugin), pdata->property, &gvalue);
}


/* Dialog respose */
static void
on_dialog_response_generic_cb (GtkDialog	    *dialog,
			       GtkResponseType	     response,
			       gpointer		     data)
{
      switch (response)
	{
	      case GTK_RESPONSE_CLOSE: gtk_widget_hide (GTK_WIDGET(dialog)); break;
	      case GTK_RESPONSE_DELETE_EVENT: gtk_widget_hide (GTK_WIDGET(dialog)); break;

	      default: break;
	}
}

void
bmp_plugin_host_create_gui_for_plugin (BmpPluginHost	  *self,
					        PluginType		   type,
						const gchar		  *name)
{
      GObject	    *plugin;
      GtkWidget	    *dialog;
      GList	    *props;
      const gchar  **metadata;
      gchar	    *title;

      plugin = g_hash_table_lookup (plugins_table[type], name);
      g_return_if_fail (plugin != NULL);
      g_return_if_fail (BMP_IS_PLUG_METADATA(plugin));
    
      metadata = bmp_plug_metadata_get (BMP_PLUG_METADATA(plugin)); 
      title = g_strconcat (metadata[METADATA_FIELD_NAME], " ", "Settings", " - BMP", NULL);

      dialog = gtk_dialog_new_with_buttons (title, 
					    NULL,
					    0,
					    GTK_STOCK_CLOSE,
					    GTK_RESPONSE_CLOSE,
					    NULL);
      g_free (title);
      gtk_container_set_border_width (GTK_CONTAINER(dialog), 8);
      bmp_window_set_icon_list (GTK_WIDGET(dialog), "player");

      g_object_connect (G_OBJECT(dialog),
			"signal::response",
			G_CALLBACK (on_dialog_response_generic_cb),
			NULL,
			NULL);

      g_object_connect (G_OBJECT(dialog),
			"signal::delete-event",
			G_CALLBACK (gtk_widget_hide),
			dialog,
			NULL);

      props = bmp_plug_property_get_props (BMP_PLUG_PROPERTY(plugin));

      for ( ; props ; props = props->next)
	{
	      GParamSpec *spec = G_PARAM_SPEC(props->data);

	      if (G_PARAM_SPEC_VALUE_TYPE(spec) == G_TYPE_STRING)
		{
		    PluginHostPropertyData  *pdata;
		    GtkWidget		    *table, 
					    *label, 
					    *entry;

		    table = gtk_table_new (2, 1, FALSE);
		    label = gtk_label_new (g_param_spec_get_blurb (spec));
		    g_object_set (label, "xalign", 0.0, NULL);

		    entry = gtk_entry_new ();
		    g_object_set (entry, "xalign", 0.0, NULL);

  		    gtk_table_attach (GTK_TABLE(table),
				      label,
				      0, 1,
				      0, 1,
				      GTK_EXPAND | GTK_FILL,  
				      GTK_FILL,  
				      0, 0);
		  
  		    gtk_table_attach (GTK_TABLE(table),
				      entry,
				      0, 1,
				      1, 2,
				      GTK_EXPAND | GTK_FILL,  
				      GTK_FILL,  
				      12, 8);

		    gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox),
					table,
					TRUE, TRUE, 0);

		    pdata = g_new0 (PluginHostPropertyData,1);
		    pdata->type = type;
		    pdata->name = g_strdup (name);
		    pdata->property = g_quark_from_static_string (g_param_spec_get_name (spec)); 

		    gtk_entry_set_text (GTK_ENTRY(entry), G_PARAM_SPEC_STRING(spec)->default_value);

	            g_object_connect (G_OBJECT(entry),
				      "signal::changed",
				      G_CALLBACK (value_changed_cb__entry),
				      pdata,
				      NULL);
		}

	      else

	      if (G_PARAM_SPEC_TYPE(spec) == BMP_TYPE_PARAM_LIST)
		{
		    PluginHostPropertyData  *pdata;
		    GtkWidget		    *table, 
					    *label, 
					    *cbox;
		    GList		    *iter;

		    table = gtk_table_new (2, 1, FALSE);
		    label = gtk_label_new (g_param_spec_get_blurb (spec));
		    g_object_set (label, "xalign", 0.0, NULL);

		    cbox = gtk_combo_box_new_text ();
		    
		    iter = BMP_PARAM_SPEC_LIST(spec)->list;
  
		    for ( ; iter ; iter = iter->next )
		      {
			gtk_combo_box_append_text (GTK_COMBO_BOX(cbox), (const gchar*)iter->data);
		      }
  
		    gtk_combo_box_set_active (GTK_COMBO_BOX(cbox), BMP_PARAM_SPEC_LIST(spec)->default_value);

  		    gtk_table_attach (GTK_TABLE(table),
				      label,
				      0, 1,
				      0, 1,
				      GTK_EXPAND | GTK_FILL,  
				      GTK_FILL,  
				      0, 0);
		  
  		    gtk_table_attach (GTK_TABLE(table),
				      cbox,
				      0, 1,
				      1, 2,
				      GTK_EXPAND | GTK_FILL,  
				      GTK_FILL,  
				      12, 8);

		    gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox),
					table,
					FALSE, TRUE, 0);

		    pdata = g_new0 (PluginHostPropertyData,1);
		    pdata->type = type;
		    pdata->name = g_strdup (name);
		    pdata->property = g_quark_from_static_string (g_param_spec_get_name (spec)); 

	            g_object_connect (G_OBJECT(cbox),
				      "signal::changed",
				      G_CALLBACK (value_changed_cb__cbox),
				      pdata,
				      NULL);
		}

	      else

	      if (G_PARAM_SPEC_VALUE_TYPE(spec) == G_TYPE_INT)
		{
		    PluginHostPropertyData  *pdata;
		    GtkWidget		    *table, 
					    *label, 
					    *spinbutton;
		    GtkObject		    *adjustment;

		    table = gtk_table_new (2, 1, FALSE);
		    label = gtk_label_new (g_param_spec_get_blurb (spec));
		    g_object_set (label, "xalign", 0.0, NULL);

		    adjustment = gtk_adjustment_new (G_PARAM_SPEC_INT(spec)->default_value,
						     G_PARAM_SPEC_INT(spec)->minimum,
						     G_PARAM_SPEC_INT(spec)->maximum, 
						     1, 1, 5);
		    spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT(adjustment), 1, 0);
		    g_object_set (spinbutton, "xalign", 0.0, NULL);

  		    gtk_table_attach (GTK_TABLE(table),
				      label,
				      0, 1,
				      0, 1,
				      GTK_EXPAND | GTK_FILL,  
				      GTK_FILL,  
				      0, 0);
		  
  		    gtk_table_attach (GTK_TABLE(table),
				      spinbutton,
				      0, 1,
				      1, 2,
				      GTK_EXPAND | GTK_FILL,  
				      GTK_FILL,  
				      12, 8);

		    gtk_widget_set_size_request (spinbutton, 100, -1);

		    gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox),
					table,
					FALSE, TRUE, 0);

		    pdata = g_new0 (PluginHostPropertyData,1);
		    pdata->type = type;
		    pdata->name = g_strdup (name);
		    pdata->property = g_quark_from_static_string (g_param_spec_get_name (spec)); 

	            g_object_connect (G_OBJECT(spinbutton),
				      "signal::value-changed",
				      G_CALLBACK (value_changed_cb__spinbutton),
				      pdata,
				      NULL);
		}

	      else

	      if (G_PARAM_SPEC_VALUE_TYPE(spec) == G_TYPE_BOOLEAN)
		{
		    PluginHostPropertyData  *pdata;
		    GtkWidget		    *checkbox; 

		    checkbox = gtk_check_button_new_with_label (g_param_spec_get_blurb (spec));
		    g_object_set (checkbox, "xalign", 0.0, NULL);

		    gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox),
					checkbox,
					FALSE, TRUE, 0);

		    pdata = g_new0 (PluginHostPropertyData,1);
		    pdata->type = type;
		    pdata->name = g_strdup (name);
		    pdata->property = g_quark_from_static_string (g_param_spec_get_name (spec)); 

	            g_object_connect (G_OBJECT(checkbox),
				      "signal::toggled",
				      G_CALLBACK (value_changed_cb__checkbox),
				      pdata,
				      NULL);
		}

      }

      g_hash_table_insert (self->private->plugin_ui[type], g_strdup (name), dialog); 
}

void
bmp_plugin_host_gui_show (BmpPluginHost	  *self,
				   PluginType			   type,
				   const gchar			  *name)
{
      GtkWidget *widget;

      widget = g_hash_table_lookup (self->private->plugin_ui[type], name); 
      g_return_if_fail (widget != NULL);

      gtk_widget_show_all (widget);
}
 
/* GObject */
static void
bmp_plugin_host_init (BmpPluginHost *self)
{
    gint n;

    self->private = g_new (BmpPluginHostPrivate, 1);
    self->private->dispose_has_run = FALSE;

    for (n = 0; n < BMP_N_PLUGIN_TYPES; n++)
      {
	self->private->plugin_ui[n] = g_hash_table_new (g_str_hash, g_str_equal);
      }
}

static void
bmp_plugin_host_set_property (GObject      *object,
				       guint         property_id,
				       const GValue *value,
				       GParamSpec   *pspec)
{
        switch (property_id)
	  {
	    default: break;
	  }
}

static void
bmp_plugin_host_get_property (GObject      *object,
				       guint         property_id,
				       GValue       *value,
				       GParamSpec   *pspec)
{
        switch (property_id)
	  {
	    default: break;
	  }
}

static GObject *
bmp_plugin_host_constructor (GType                  type,
                       guint                  n_construct_properties,
                       GObjectConstructParam *construct_properties)
{
        GObject *obj;

        {
                BmpPluginHostClass *klass;
                GObjectClass *parent_class;
                klass = BMP_PLUGIN_HOST_CLASS (g_type_class_peek (BMP_TYPE_PLUGIN_HOST));
                parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
                obj = parent_class->constructor (type,
                                                 n_construct_properties,
                                                 construct_properties);
        }

        return obj;
}

static void
bmp_plugin_host_dispose (GObject *obj)
{
        BmpPluginHost *self = (BmpPluginHost *)obj;

        if (self->private->dispose_has_run) return;

        self->private->dispose_has_run = TRUE;

}

static void
bmp_plugin_host_finalize (GObject *obj)
{
        BmpPluginHost *self = (BmpPluginHost *)obj;

        g_free (self->private);
}

BmpPluginHost*
bmp_plugin_host_new (void)
{
  BmpPluginHost *self;

  self = g_object_new (bmp_plugin_host_get_type (), NULL);

  return self;
}

static void
bmp_plugin_host_class_init (BmpPluginHostClass *g_class)
{
        GObjectClass *gobject_class = G_OBJECT_CLASS (g_class);

        gobject_class->set_property = bmp_plugin_host_set_property;
        gobject_class->get_property = bmp_plugin_host_get_property;
        gobject_class->dispose	    = bmp_plugin_host_dispose;
        gobject_class->finalize	    = bmp_plugin_host_finalize;
        gobject_class->constructor  = bmp_plugin_host_constructor;
}
