/*  BMPx - The Dumb Music Player
 *  Copyright (C) 2005 BMPx development team.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id$
 *
 * The BMPx project hereby grant permission for non-gpl compatible GStreamer
 * plugins to be used and distributed together with GStreamer and BMPx. This
 * permission are above and beyond the permissions granted by the GPL license
 * BMPx is covered by.
 */

/*
 * This file contains code originally taken from XMMS-scrobbler:
 *  
 * XMMS-scrobbler (C) GPL by
 *
 * Audun Hove <audun@nlc.no>
 * Pipian <pipian@pipian.com>
 *
 */

/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */

#include <config.h>
#include "main.h"

#include <glib.h>
#include <glib/gi18n.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <strings.h>

#ifdef ENABLE_NLS
# include <libintl.h>
#endif

#include <ne_basic.h>
#include <ne_compress.h>
#include <ne_socket.h>
#include <ne_utils.h>
#include <ne_props.h>
#include <ne_session.h>
#include <ne_request.h>
#include <ne_uri.h>

#include <bmp/util.h>
#include <bmp/uri.h>
#include <bmp/vfs.h>

#include <goa/libgoa.h>

#include "scrobbler.h"
#include "md5.h"

#include "system_control.h"

#define BMPX_SCROBBLER_URI_SCHEME   "http"
#define BMPX_SCROBBLER_URI_HOST     "post.audioscrobbler.com"
#define BMPX_SCROBBLER_URI_PORT	     80
#define BMPX_SCROBBLER_USERAGENT    "BMP2"

#define BMPX_SCROBBLER_URI_PATH     "/?hs=true&p=1.1&c=%s&v=%s&u=%s"

#define RESP_UPTODATE		    "UPTODATE"
#define RESP_UPDATE		    "UPDATE"
#define RESP_FAILED		    "FAILED"
#define RESP_BADUSER		    "BADUSER"

#define BMPX_SCROBBLER_CLIENT_ID "mpx"
#define BMPX_SCROBBLER_CLIENT_VERSION "1.0"

typedef enum {
    SCROBBLER_HANDSHAKE_UPTODATE,
    SCROBBLER_HANDSHAKE_UPDATE,
    SCROBBLER_HANDSHAKE_BADUSER,
    SCROBBLER_HANDSHAKE_FAILED,

    SCROBBLER_HANDSHAKE_NETWORK_ERROR,
    SCROBBLER_HANDSHAKE_STATE_UNDEFINED

} ScrobblerHandshakeStatus;

typedef struct _ScrobblerHandshakeInfo ScrobblerHandshakeInfo;
struct _ScrobblerHandshakeInfo {
	gboolean     uptodate;
	gchar	    *uri;
	gchar	    *md5;
	gint	     interval;
	gchar	    *info;	// Used for FAILED <reason>, and UPDATE url field
};

typedef struct _ScrobblerQueueItem ScrobblerQueueItem;
struct _ScrobblerQueueItem {
	gchar	    *md5_pre;
	gchar	    *md5_post;
};


static void     bmp_scrobbler_init       (BmpScrobbler      *self);
static void     bmp_scrobbler_class_init (BmpScrobblerClass *klass);
static gpointer bmp_scrobbler_parent_class = NULL;
static void     bmp_scrobbler_class_intern_init (gpointer klass)
{
  bmp_scrobbler_parent_class = g_type_class_peek_parent (klass);
  bmp_scrobbler_class_init ((BmpScrobblerClass*) klass);
}

GType
bmp_scrobbler_get_type (void)
{
  static GType g_define_type_id = 0; 
  if (G_UNLIKELY (g_define_type_id == 0)) 
    { 
      static const GTypeInfo g_define_type_info = { 
        sizeof (BmpScrobblerClass), 
        (GBaseInitFunc) NULL, 
        (GBaseFinalizeFunc) NULL, 
        (GClassInitFunc) bmp_scrobbler_class_intern_init, 
        (GClassFinalizeFunc) NULL, 
        NULL,   /* class_data */ 
        sizeof (BmpScrobbler), 
        0,      /* n_preallocs */ 
        (GInstanceInitFunc) bmp_scrobbler_init, 
      }; 
      g_define_type_id = g_type_register_static (G_TYPE_OBJECT, "BmpScrobbler", &g_define_type_info, 0); 
    } 
  return g_define_type_id; 
}

#if 0
typedef enum {
	BMP_SCROBBLER_N_SIGNALS
} BmpScrobblerSignals;
static guint bmp_scrobbler_signals[BMP_SCROBBLER_N_SIGNALS] = {};
#endif

typedef enum {
	BMP_SCROBBLER_N_PROPERTIES

} BmpScrobblerProperties;

enum {
	H_TRACK_CHANGE,	
	H_SET_STREAM_POS,
	H_SEEK_EVENT,
	H_LASTFM_ENABLE,
	H_LASTFM_ENABLE_GENERAL,

	N_HANDLERS
};

struct _BmpScrobblerPrivate
{
        gboolean dispose_has_run;

	ScrobblerHandshakeInfo *info;

	gboolean		handshaked;
	gboolean		enabled;
	gboolean		sent;

	gint			duration;
	gint			position_previous;

	gchar		       *md5_response;

	gulong			handler[N_HANDLERS];

	gint			message_domain_id;

	gboolean		green_light;
	guint		        green_light_src_id;

	GQueue		       *queue;
	guint			queue_src_id;
};

void
scrobbler_run_handshake (BmpScrobbler *this);

static char*
get_timestr (time_t t, int gmt);

G_LOCK_DEFINE(GREEN_LIGHT);
G_LOCK_DEFINE(QUEUE);

static void
set_green_light (BmpScrobbler *this)
{
    G_LOCK (GREEN_LIGHT);
    g_log (BMPX_SCROBBLER_LOG_DOMAIN, G_LOG_LEVEL_INFO, "%s: GREEN LIGHT", G_STRLOC);
    this->private->green_light = TRUE;
    G_UNLOCK (GREEN_LIGHT);
}

static void
scrobbler_restart_green_light_timeout (BmpScrobbler *this, gint seconds)
{
    GSource *source;

    G_LOCK (GREEN_LIGHT);
    if (this->private->green_light_src_id)
      {
	  source = g_main_context_find_source_by_id (NULL, this->private->green_light_src_id);
	  if (source)
	    {
	      g_source_destroy (source);
	    }
      }

    this->private->green_light = FALSE;
    G_UNLOCK (GREEN_LIGHT);

    g_log (BMPX_SCROBBLER_LOG_DOMAIN, G_LOG_LEVEL_INFO, "%s: GREEN LIGHT in %d seconds", G_STRLOC, seconds);
    g_timeout_add (seconds*1000, (GSourceFunc) set_green_light, this);
}
	
static void
scrobbler_track_change (BmpSystemControl    *system_control,
			BmpScrobbler	    *this)
{
    //Determine the length of the current song
    GError	*error;
    GHashTable	*metadata;
    GValue  	*value;
    gint	 index;

    error = NULL;
    metadata = NULL;

    bmp_system_control_get_current_track (system_control, &index, &error);
    if (!bmp_system_control_get_metadata_for_list_item (system_control, index, &metadata, &error)) return;

    value = g_hash_table_lookup (metadata, bmp_metadata_get_id_static (BMP_DATUM_TIME));
    this->private->duration = g_value_get_int (value);
    g_hash_table_destroy (metadata);

    this->private->sent = FALSE;
    this->private->position_previous = 0; 
}

/* Stolen from libtransport_http */
typedef struct _SCReadHandle SCReadHandle;
struct _SCReadHandle {
    GMutex  *handle_lock;
    char    *memory;
    size_t   size;
};
#define SC_READ_HANDLE(_data) ((SCReadHandle*)(_data))

static int
read_block (void *data, const char *buf, size_t len)
{
  SCReadHandle *rhandle = SC_READ_HANDLE(data);

  if (!len)
    {
        g_mutex_unlock (rhandle->handle_lock);
        return 0;
    }

  rhandle->memory = realloc (rhandle->memory, rhandle->size+len);
  memcpy  (rhandle->memory+rhandle->size, buf, len);
  rhandle->size += len;

  return 0;
}

static gboolean
process_queue (BmpScrobbler *this)
{
    ScrobblerQueueItem	*queue_item;
    SCReadHandle	*rhandle;
    gchar		*post_data;
    gchar	       **resp,
		       **line;
    ne_session	        *sess;
    ne_request		*reqs;
    BMPURI		*uri;

    G_LOCK (GREEN_LIGHT);
    if (!this->private->green_light)
      {
	g_log (BMPX_SCROBBLER_LOG_DOMAIN, G_LOG_LEVEL_INFO, "%s: GREEN LIGHT is FALSE: cannot send", G_STRLOC);
	G_UNLOCK (GREEN_LIGHT);
	return TRUE;
      }
    G_UNLOCK (GREEN_LIGHT);

    queue_item = g_queue_peek_head (this->private->queue);
    if (!queue_item)
      {
	g_log (BMPX_SCROBBLER_LOG_DOMAIN, G_LOG_LEVEL_INFO, "%s: Queue is empty", G_STRLOC);
	this->private->queue_src_id = 0;
	return FALSE;
      }

    queue_item = g_queue_pop_head (this->private->queue);
    post_data = g_strconcat (queue_item->md5_pre, this->private->md5_response, queue_item->md5_post, NULL);

    g_free (queue_item->md5_pre);
    g_free (queue_item->md5_post);
    g_free (queue_item);

    g_log (BMPX_SCROBBLER_LOG_DOMAIN, G_LOG_LEVEL_INFO, "%s: Submitting song with POST data '%s'", G_STRLOC, post_data);

    if (!(uri = bmp_uri_new (this->private->info->uri))) return TRUE;

    sess = ne_session_create (uri->scheme,
			      uri->hostname, 
			      80);
    ne_set_useragent (sess, BMPX_SCROBBLER_USERAGENT);
    ne_set_read_timeout (sess, 15); 
    reqs = ne_request_create (sess, "POST", uri->path);
    ne_add_request_header (reqs, "Content-Type", "application/x-www-form-urlencoded");
    ne_set_request_body_buffer (reqs, (const char *)post_data, (size_t)strlen(post_data));
    rhandle = g_new0 (SCReadHandle, 1);
    rhandle->memory = NULL;
    rhandle->size = 0;
    rhandle->handle_lock = g_mutex_new ();
    ne_add_response_body_reader (reqs, ne_accept_2xx, read_block, rhandle);
    g_mutex_lock (rhandle->handle_lock);
    if (ne_request_dispatch (reqs) != NE_OK)
	{
	    gchar *str;

	    g_mutex_unlock (rhandle->handle_lock);
	    g_free (rhandle);
	    ne_request_destroy (reqs);

	    str = g_strdup_printf ("%s: '%s'", _("A Network error occured while trying to connect to Last.FM"), ne_get_error (sess));
	    ne_session_destroy (sess);

	    bmp_system_control_message_dispatch (bmp_system_control,
					     this->private->message_domain_id, 
					     GTK_MESSAGE_ERROR,
					     str);

	    xcs_key_set_boolean (xcs, "lastfm", "enable", FALSE);
	    g_free (str);
	    return FALSE;
	}

    while (!g_mutex_trylock(rhandle->handle_lock))
      {
	while (g_main_context_pending(NULL)) g_main_context_iteration (NULL, TRUE);
      }

    g_mutex_unlock (rhandle->handle_lock);

    ne_request_destroy (reqs);
    ne_session_destroy (sess);

    rhandle->memory = realloc (rhandle->memory, rhandle->size + 1);
    rhandle->memory[rhandle->size+1] = 0;

    resp = g_strsplit (rhandle->memory, "\n", -1);

    if (!strcmp("BADAUTH",resp[0]))
	{
	    bmp_system_control_message_dispatch (bmp_system_control,
					     this->private->message_domain_id, 
					     GTK_MESSAGE_ERROR,
					    _("There was an authentication error during song submission.\n\nPlease check your Last.FM username/password.\n\nLast.FM support has to be disabled until proper credentials are provided.")); 
	    xcs_key_set_boolean (xcs, "lastfm", "enable", FALSE);
	}
    else if (!strncmp("FAILED ", resp[0], 7))
	{
	    g_log (BMPX_SCROBBLER_LOG_DOMAIN, G_LOG_LEVEL_INFO, "%s: Submitting song '%s' failed: %s", G_STRLOC, post_data, resp[0]);
	}
    else /* OK assumed */
	{
	    g_log (BMPX_SCROBBLER_LOG_DOMAIN, G_LOG_LEVEL_INFO, "%s: Submitting song '%s' succeeded.", G_STRLOC, post_data);
	}

    line = g_strsplit (resp[1], " ", -1);
    scrobbler_restart_green_light_timeout (this, atoi(line[1])); 

    g_strfreev(line);
    g_strfreev(resp);
    g_mutex_free (rhandle->handle_lock);
    g_free (rhandle);
    g_free (post_data);

    if (g_queue_get_length (this->private->queue) == 0)
      {
	this->private->queue_src_id = 0;
	return FALSE;
      }

    return TRUE;
}

static void
scrobbler_send_song_information (BmpScrobbler	    *this,
				 gint		     position)
{
#define SCROBBLER_SONG_POST_STRING_PRE  "u=%s&s="
#define SCROBBLER_SONG_POST_STRING_POST "&a[0]=%s&t[0]=%s&m[0]=&b[0]=%s&l[0]=%d&i[0]=%s"
   
    ScrobblerQueueItem	*queue_item; 
    gchar		*current_uri;
    GError		*error;
    GHashTable		*metadata;
    GValue		*value;
    gchar		*timestr;

    gchar		*artist, *artist_e,
			*title,  *title_e,
			*album,  *album_e;
    gint		 duration,
			 index;
    gchar		*user;

    if (this->private->sent) return;

    this->private->sent = TRUE;
    error = NULL;
    metadata = NULL;

    bmp_system_control_get_current_track (bmp_system_control, &index, &error);
    if (!bmp_system_control_get_metadata_for_list_item (bmp_system_control, index, &metadata, &error)) return;

    current_uri = g_value_dup_string (g_hash_table_lookup (metadata, bmp_metadata_get_id_static (BMP_DATUM_LOCATION)));

    if (g_uri_test (current_uri, BMP_URI_TEST_PROTOCOL, PROTOCOL_HTTP))
	{
	    g_free (current_uri);
	    return;
	}

    value = g_hash_table_lookup (metadata, bmp_metadata_get_id_static (BMP_DATUM_TIME));
    duration = g_value_get_int (value);

    if ((duration < 30) && !g_uri_test (current_uri, BMP_URI_TEST_PROTOCOL, PROTOCOL_CDDA))
	{
	    g_log (BMPX_SCROBBLER_LOG_DOMAIN, G_LOG_LEVEL_INFO, "%s: Not submitting '%s' to scrobbler: duration of less than 30 seconds",
						    G_STRLOC,
						    current_uri);
	    g_free (current_uri);
	    return;
	}

    value  = g_hash_table_lookup (metadata, bmp_metadata_get_id_static (BMP_DATUM_ARTIST));
    artist = g_value_dup_string (value);

    value  = g_hash_table_lookup (metadata, bmp_metadata_get_id_static (BMP_DATUM_ALBUM));
    album  = g_value_dup_string (value);

    value  = g_hash_table_lookup (metadata, bmp_metadata_get_id_static (BMP_DATUM_TITLE));
    title  = g_value_dup_string (value);

    if (artist) g_strstrip (artist);
    if (album)  g_strstrip (album);
    if (title)  g_strstrip (title);

    if (artist && !strlen(artist)) 
      {
	  g_free (artist);
	  artist = NULL;
      }

    if (title && !strlen(title)) 
      {
	  g_free (title);
	  title = NULL;
      }

    if (!duration || !artist || !title)
	{
	    g_log (BMPX_SCROBBLER_LOG_DOMAIN, G_LOG_LEVEL_INFO, "%s: Not submitting '%s' to scrobbler: missing metadata!",
						    G_STRLOC,
						    current_uri);
	    g_free (current_uri);
	    return;
	}


    timestr = get_timestr (time(NULL),1);
    user = (gchar*)xcs_key_fetch_string (xcs, "lastfm", "username");

    //Push to queue
    G_LOCK (QUEUE);

    g_log (BMPX_SCROBBLER_LOG_DOMAIN, G_LOG_LEVEL_INFO, "%s: Pushing '%s' onto the queue", G_STRLOC, current_uri);

    queue_item = g_new0 (ScrobblerQueueItem,1);

    queue_item->md5_pre = g_strdup_printf (SCROBBLER_SONG_POST_STRING_PRE, user);

    artist_e = ne_path_escape (artist);
    album_e  = ne_path_escape (album);
    title_e  = ne_path_escape (title);

    queue_item->md5_post = g_strdup_printf (SCROBBLER_SONG_POST_STRING_POST, 
					    artist_e, 
					    title_e,
					    album_e, 
					    duration,
					    timestr);

    if (artist) g_free (artist);
    if (title)  g_free (title);
    if (album)  g_free (album);

    if (artist_e) g_free (artist_e);
    if (title_e)  g_free (title_e);
    if (album_e)  g_free (album_e);

    g_queue_push_tail (this->private->queue, queue_item); 

    G_UNLOCK (QUEUE);

    if ((!this->private->queue_src_id) && xcs_key_fetch_boolean (xcs, "lastfm", "enable")) 
      {
	  g_log (BMPX_SCROBBLER_LOG_DOMAIN, G_LOG_LEVEL_INFO, "%s: Restarting queue processing", G_STRLOC);
	  this->private->queue_src_id = g_timeout_add (2000, (GSourceFunc) process_queue, this);
      }
}

static void
scrobbler_seek_event (BmpSystemControl	  *control,
		      gint		   position,
		      BmpScrobbler	  *this)
{
    gboolean playing = (g_object_get_int (bmp_play_engine, "playstatus") == PLAYSTATUS_PLAYING) ? TRUE : FALSE;

    if (playing && (!this->private->sent))
	{
	    if ((position >= 240) || (position >= (this->private->duration/2)))
		{
		    this->private->sent = TRUE;
		    return;
		}
	}
}

static void
scrobbler_set_stream_pos	(BmpSystemControl   *system_control,				
				 gint		     position,
				 BmpScrobbler	    *this) 
{
    gboolean playing = (g_object_get_int (bmp_play_engine, "playstatus") == PLAYSTATUS_PLAYING) ? TRUE : FALSE;

    if (playing && (!this->private->sent))
	{
	    if ((position >= 240) || (position >= (this->private->duration/2)))
		{
		    scrobbler_send_song_information (this, position);
		    return;
		}
	}
}

static void
scrobbler_lastfm_enable_changed (Xcs		    *config,
				 BmpScrobbler	    *this)
{
    gboolean enabled = xcs_key_fetch_boolean (xcs, "lastfm", "enable");
    if (enabled)
      {
	scrobbler_run_handshake (this);
      }
    else
      {
	this->private->handshaked = FALSE;
      }
}

/* GObject */
static void
bmp_scrobbler_init (BmpScrobbler *this)
{
    GError *error;
    gchar *filename, *buffer;
    gchar **lines;
    gsize length;
    gint i;

    this->private = g_new (BmpScrobblerPrivate, 1);
    this->private->dispose_has_run = FALSE;

    this->private->info = NULL;
    this->private->sent = FALSE;
    this->private->handshaked = FALSE;
    this->private->duration = 0;
    this->private->md5_response = NULL;
    this->private->green_light = FALSE;
    this->private->green_light_src_id = 0;
    this->private->queue = g_queue_new ();
    this->private->queue_src_id = 0;

    for (i = 0; i < N_HANDLERS; this->private->handler[i++] = 0);

    error = NULL;
    filename = g_build_filename (BMP_PATH_USER_DIR, "lastfm.queue", NULL);
    g_file_get_contents (filename,
			 &buffer,
			 &length,	  
			 &error);
    g_free (filename);

    if (error)
      {
	g_log (BMPX_SCROBBLER_LOG_DOMAIN, G_LOG_LEVEL_INFO, "%s: No queue to load: %s", G_STRLOC, error->message);
	g_error_free (error);
	error = NULL;
      }

    if ((!buffer) || (!strlen(buffer))) return;

    lines = g_strsplit (buffer, "\n", -1);
    i = 0;
    while ((lines[i]) && strlen(lines[i]))
      {
	  ScrobblerQueueItem  *queue_item;
	  //Push to queue
	  G_LOCK (QUEUE);

	  queue_item = g_new0 (ScrobblerQueueItem,1);

	  queue_item->md5_pre  = g_strdup (lines[i]); 
	  i++;
	  queue_item->md5_post = g_strdup (lines[i]); 

	  if (lines[i] && strlen(lines[i]))	  
	    {
		g_queue_push_tail (this->private->queue, queue_item); 
	    }
	  else
	    {
		g_free (queue_item);
		G_UNLOCK (QUEUE);
		break;
	    }
	  G_UNLOCK (QUEUE);
	  i++;
      }
  
  g_strfreev (lines);
  g_free (buffer);
}

static void
bmp_scrobbler_set_property (GObject      *object,
                        guint         property_id,
                        const GValue *value,
                        GParamSpec   *pspec)
{
        switch (property_id)
	    {
		default: break;
	    }
}

static void
bmp_scrobbler_get_property (GObject      *object,
                        guint         property_id,
                        GValue       *value,
                        GParamSpec   *pspec)
{
        switch (property_id)
	    {
		default: break;
	    }
}

static GObject *
bmp_scrobbler_constructor (GType                  type,
                       guint                  n_construct_properties,
                       GObjectConstructParam *construct_properties)
{
        GObject *obj;

        {
                /* Invoke parent constructor. */
                BmpScrobblerClass *klass;
                GObjectClass *parent_class;
                klass = BMP_SCROBBLER_CLASS (g_type_class_peek (BMP_TYPE_SCROBBLER));
                parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
                obj = parent_class->constructor (type,
                                                 n_construct_properties,
                                                 construct_properties);
        }


        return obj;
}

static void
append_to_string (ScrobblerQueueItem	  *item,
		  GString		  *queue_string)
{
  g_string_append (queue_string, item->md5_pre);
  g_string_append (queue_string, "\n"); 
  g_string_append (queue_string, item->md5_post);
  g_string_append (queue_string, "\n"); 
}

static void
bmp_scrobbler_dispose (GObject *obj)
{
    BmpScrobbler  *this = (BmpScrobbler*) obj;
    GSource	  *source;
    GString	  *queue_string;
    GError	  *error;
    gchar	  *filename;

    if (this->private->dispose_has_run) return;
    this->private->dispose_has_run = TRUE;

    G_LOCK (GREEN_LIGHT);
    if (this->private->green_light_src_id)
      {
	  source = g_main_context_find_source_by_id (NULL, this->private->green_light_src_id);
	  if (source) g_source_destroy (source);
      }
    G_UNLOCK (GREEN_LIGHT);

    if (this->private->queue_src_id)
      {
	  source = g_main_context_find_source_by_id (NULL, this->private->queue_src_id);
	  if (source) g_source_destroy (source);
      }
  
    queue_string = g_string_new ("");
    g_queue_foreach (this->private->queue, (GFunc) append_to_string, queue_string);

    error = NULL;
    filename = g_build_filename (BMP_PATH_USER_DIR, "lastfm.queue", NULL);
    g_file_set_contents (filename,
			 queue_string->str,
			 (gssize)strlen(queue_string->str),
			 &error);
    g_free (filename);

    if (error)
      {
	g_log (BMPX_SCROBBLER_LOG_DOMAIN, G_LOG_LEVEL_CRITICAL, "%s: Unable to save the queue: %s", G_STRLOC, error->message);
	g_error_free (error);
	error = NULL;
      }
    
    g_string_free (queue_string, TRUE);
}

static void
bmp_scrobbler_finalize (GObject *obj)
{
        BmpScrobbler *this = (BmpScrobbler*)obj;

        g_free (this->private);
}

static void
scrobbler_lastfm_general_enable  (Xcs		    *config,
				  BmpScrobbler	    *this)
{
  if (xcs_key_fetch_boolean (xcs, "lastfm", "general-enable"))
    {  
      this->private->handshaked = FALSE;

      this->private->handler[H_TRACK_CHANGE] =
			g_signal_connect (G_OBJECT(bmp_system_control),
				          "track-change",
					  G_CALLBACK(scrobbler_track_change),
					  this);

      this->private->handler[H_SET_STREAM_POS] =
			g_signal_connect (G_OBJECT(bmp_system_control),
				          "set-stream-pos",
					  G_CALLBACK(scrobbler_set_stream_pos),
					  this);

      this->private->handler[H_SEEK_EVENT] =
			g_signal_connect (G_OBJECT(bmp_system_control),
				          "seek-event",
					  G_CALLBACK(scrobbler_seek_event),
					  this);


      this->private->handler[H_LASTFM_ENABLE] =
			g_signal_connect (G_OBJECT(xcs),	
					  "changed::lastfm-enable",
					  G_CALLBACK(scrobbler_lastfm_enable_changed),
					  this);

      if (xcs_key_fetch_boolean (xcs, "lastfm", "enable"))
	{
	    scrobbler_run_handshake (this);
	}
    }
  else
    {
      GSource *source;

      if (this->private->handler[H_TRACK_CHANGE])
	{
	    g_signal_handler_disconnect (bmp_system_control, this->private->handler[H_TRACK_CHANGE]);
	}

      if (this->private->handler[H_SET_STREAM_POS])
	{
	    g_signal_handler_disconnect (bmp_system_control, this->private->handler[H_SET_STREAM_POS]);
	}

      if (this->private->handler[H_SEEK_EVENT])
	{
	    g_signal_handler_disconnect (bmp_system_control, this->private->handler[H_SEEK_EVENT]);
	}


      if (this->private->handler[H_LASTFM_ENABLE])
	{
	    g_signal_handler_disconnect (xcs, this->private->handler[H_LASTFM_ENABLE]);
	}

    G_LOCK (GREEN_LIGHT);
    if (this->private->green_light_src_id)
      {
	  source = g_main_context_find_source_by_id (NULL, this->private->green_light_src_id);
	  if (source) g_source_destroy (source);
      }
    G_UNLOCK (GREEN_LIGHT);

    if (this->private->queue_src_id)
      {
	  source = g_main_context_find_source_by_id (NULL, this->private->queue_src_id);
	  if (source) g_source_destroy (source);
      }
    this->private->handshaked = FALSE;
  }
}

BmpScrobbler*
bmp_scrobbler_new ()
{
  BmpScrobbler *this;

  this = g_object_new (bmp_scrobbler_get_type (), NULL);

  this->private->message_domain_id =
			 bmp_system_control_message_domain_register (bmp_system_control,
								     "lastfm", 
  								     _("BMP: Last.FM"));

  if (xcs_key_fetch_boolean (xcs, "lastfm", "general-enable"))
    {
	scrobbler_lastfm_general_enable (xcs, this);
    }

  this->private->handler[H_LASTFM_ENABLE_GENERAL] =
			g_signal_connect (G_OBJECT(xcs),	
					  "changed::lastfm-general-enable",
					  G_CALLBACK(scrobbler_lastfm_general_enable),
					  this);
  return this;
}

static void
bmp_scrobbler_class_init (BmpScrobblerClass *g_class)
{
        GObjectClass *gobject_class = G_OBJECT_CLASS (g_class);

        gobject_class->set_property = bmp_scrobbler_set_property;
        gobject_class->get_property = bmp_scrobbler_get_property;
        gobject_class->dispose	    = bmp_scrobbler_dispose;
        gobject_class->finalize	    = bmp_scrobbler_finalize;
        gobject_class->constructor  = bmp_scrobbler_constructor;
}

/* Scrobbler */
static char*
hexify (char *pass, int len)
{
    char *hash;
    char *bp;
    char hexchars[] = "0123456789abcdef";
    int i;

    hash = g_malloc0 (33);
    bp = hash;

    for(i = 0; i < len; i++)
	{
	    *(bp++) = hexchars[(pass[i] >> 4) & 0x0f];
	    *(bp++) = hexchars[pass[i] & 0x0f];
	}
    *bp = 0;

    return hash;
}

static char*
get_timestr (time_t t, int gmt)
{
	struct tm *tm;
	static char buf[30];

	tm = gmt ? gmtime(&t) : localtime(&t);
	snprintf(buf, sizeof(buf), "%d-%.2d-%.2d %.2d:%.2d:%.2d",
			tm->tm_year + 1900,
			tm->tm_mon + 1,
			tm->tm_mday,
			tm->tm_hour,
			tm->tm_min,
			tm->tm_sec);
	return buf;
}

static ScrobblerHandshakeStatus
scrobbler_handshake (BmpScrobbler *this)

{
    SCReadHandle    *rhandle;
    ne_session	    *sess;
    ne_request	    *reqs;
    gchar	    *path;
    gchar	   **resp, **line;
    gchar	    *user;
    ScrobblerHandshakeStatus status = SCROBBLER_HANDSHAKE_STATE_UNDEFINED;

    this->private->info = g_new0 (ScrobblerHandshakeInfo,1);

    user = (gchar*)xcs_key_fetch_string (xcs, "lastfm", "username");

    path = g_strdup_printf (BMPX_SCROBBLER_URI_PATH,
			    BMPX_SCROBBLER_CLIENT_ID,
			    BMPX_SCROBBLER_CLIENT_VERSION,
			    user); 

    // g_free (user);

    sess = ne_session_create (BMPX_SCROBBLER_URI_SCHEME,
			      BMPX_SCROBBLER_URI_HOST,
			      BMPX_SCROBBLER_URI_PORT);

    ne_set_useragent (sess, BMPX_SCROBBLER_USERAGENT);
    ne_set_read_timeout (sess, 15); 
    reqs = ne_request_create (sess, "GET", path);

    rhandle = g_new0 (SCReadHandle, 1);
    rhandle->memory = NULL;
    rhandle->size = 0;
    rhandle->handle_lock = g_mutex_new ();

    g_mutex_lock (rhandle->handle_lock);

    ne_add_response_body_reader (reqs, ne_accept_2xx, read_block, rhandle);

    if (ne_request_dispatch (reqs) != NE_OK)
	{
	    gchar *str;

	    g_mutex_unlock (rhandle->handle_lock);
	    g_free (rhandle);
	    ne_request_destroy (reqs);

	    str = g_strdup_printf ("%s: '%s'", _("A Network error occured while trying to connect to Last.FM"), ne_get_error (sess));
	    ne_session_destroy (sess);

	    bmp_system_control_message_dispatch (bmp_system_control,
					     this->private->message_domain_id, 
					     GTK_MESSAGE_ERROR,
					     str);

	    xcs_key_set_boolean (xcs, "lastfm", "enable", FALSE);
	    g_free (str);

	    status = SCROBBLER_HANDSHAKE_NETWORK_ERROR;
	    return status;
	}

    while (!g_mutex_trylock(rhandle->handle_lock))
      {
        while (gtk_events_pending ())
          gtk_main_iteration ();
      }

    g_mutex_unlock (rhandle->handle_lock);

    ne_request_destroy (reqs);
    ne_session_destroy (sess);

    resp = g_strsplit (rhandle->memory, "\n", -1);
    g_free (rhandle->memory);

    //Parse response 
    if (!resp)
	{
	    status = SCROBBLER_HANDSHAKE_NETWORK_ERROR; //Not 100% true, but also not entirely wrong
	    return status;
	}

    line = g_strsplit (resp[0], " ", -1);
    if (!strcmp(line[0], RESP_UPTODATE)) 
	{
	    this->private->info->uptodate = TRUE;
	    g_strfreev (line);

	    this->private->info->md5 = g_strdup (resp[1]);	    
	    this->private->info->uri = g_strdup (resp[2]);

	    line = g_strsplit (resp[3], " ", -1);

	    if (line[0] && line[1])
		this->private->info->interval = atoi (line[1]);

	    status = SCROBBLER_HANDSHAKE_UPTODATE;
	}
    else
    if (!strcmp(line[0], RESP_UPDATE))
	{
	    this->private->info->uptodate = FALSE;
	    this->private->info->info = g_strdup (resp[0]);

	    this->private->info->md5 = g_strdup (resp[1]);	    
	    this->private->info->uri = g_strdup (resp[2]);

	    line = g_strsplit (resp[3], " ", -1);

	    if (line[0] && line[1])
		this->private->info->interval = atoi (line[1]);

	    status = SCROBBLER_HANDSHAKE_UPDATE;
	}
    else 
    if (!strcmp(line[0], RESP_BADUSER))
	{
	    g_strfreev (line);

	    line = g_strsplit (resp[1], " ", -1);

	    if (line[0] && line[1])
		this->private->info->interval = atoi (line[1]);

	    status = SCROBBLER_HANDSHAKE_BADUSER;
	}
    if (!strcmp(line[0], RESP_FAILED))
	{
	    this->private->info->info = g_strdup (resp[0]);
	    g_strfreev (line);

	    line = g_strsplit (resp[1], " ", -1);

	    if (line[0] && line[1])
		this->private->info->interval = atoi (line[1]);

	    status = SCROBBLER_HANDSHAKE_FAILED;
	}
	
    if ((status == SCROBBLER_HANDSHAKE_BADUSER) || (status == SCROBBLER_HANDSHAKE_FAILED))
      {
        g_strfreev (line);
	line = g_strsplit (resp[1], " ", -1);
	scrobbler_restart_green_light_timeout (this, atoi(line[1])); 
      }
    else
      {
        g_strfreev (line);
	line = g_strsplit (resp[3], " ", -1);
	scrobbler_restart_green_light_timeout (this, atoi(line[1])); 
      }

    g_strfreev (line);
    g_strfreev (resp);
    g_free (rhandle);

    return status;
}

void
scrobbler_run_handshake (BmpScrobbler *this)
{
    ScrobblerHandshakeStatus hs_result;
    gchar *user, *pass;

    if (this->private->handshaked) return;

    user = (gchar*)xcs_key_fetch_string (xcs, "lastfm", "username");
    pass = (gchar*)xcs_key_fetch_string (xcs, "lastfm", "password");

    if (!user || !pass)
	{
	    //FIXME: Error here

	    return;
	}

    hs_result = scrobbler_handshake (this);

    switch (hs_result)
	{
		case SCROBBLER_HANDSHAKE_UPTODATE:
		    {
			md5_state_t  md5state;
			char         md5pword[16];
			char	    *str;
			char	    *pass, *aux;

			g_log (BMPX_SCROBBLER_LOG_DOMAIN, G_LOG_LEVEL_INFO, "%s: UPTODATE. MD5:'%s', URL:'%s', INTERVAL:'%d'", G_STRLOC,
										      this->private->info->md5,
										      this->private->info->uri,
										      this->private->info->interval);

			pass = (gchar*)xcs_key_fetch_string (xcs, "lastfm", "password");

		        md5_init (&md5state);
			md5_append (&md5state, (unsigned const char *)pass, strlen(pass));
			md5_finish (&md5state, (unsigned char *)md5pword);
			aux = hexify (md5pword, sizeof(md5pword));

			str = g_strconcat (aux, this->private->info->md5, NULL);
		        md5_init (&md5state);
			md5_append (&md5state, (unsigned const char *)str, strlen(str));
			md5_finish (&md5state, (unsigned char *)md5pword);

			this->private->md5_response = hexify (md5pword, sizeof(md5pword));

			if ((!this->private->queue_src_id) && xcs_key_fetch_boolean (xcs, "lastfm", "enable")) 
			  {
			      g_log (BMPX_SCROBBLER_LOG_DOMAIN, G_LOG_LEVEL_INFO, "%s: Restarting queue processing", G_STRLOC);
			      this->private->queue_src_id = g_timeout_add (2000, (GSourceFunc) process_queue, this);
			  }

			g_free (aux);
			g_free (str);
		    }
		    break;

		case SCROBBLER_HANDSHAKE_UPDATE:
		    {
			md5_state_t  md5state;
			char         md5pword[16];
			char	    *str;
			char	    *pass, *aux;

			g_log (BMPX_SCROBBLER_LOG_DOMAIN, G_LOG_LEVEL_INFO, "%s: UPDATE. UPDATE_URL:'%s', MD5:'%s', URL:'%s', INTERVAL:'%d'", G_STRLOC,
										      this->private->info->info,
										      this->private->info->md5,
										      this->private->info->uri,
										      this->private->info->interval);

			pass = (gchar*)xcs_key_fetch_string (xcs, "lastfm", "password");

		        md5_init (&md5state);
			md5_append (&md5state, (unsigned const char *)pass, strlen(pass));
			md5_finish (&md5state, (unsigned char *)md5pword);
			aux = hexify (md5pword, sizeof(md5pword));

			str = g_strconcat (aux, this->private->info->md5, NULL);
		        md5_init (&md5state);
			md5_append (&md5state, (unsigned const char *)str, strlen(str));
			md5_finish (&md5state, (unsigned char *)md5pword);

			this->private->md5_response = hexify (md5pword, sizeof(md5pword));

			if ((!this->private->queue_src_id) && xcs_key_fetch_boolean (xcs, "lastfm", "enable")) 
			  {
			      g_log (BMPX_SCROBBLER_LOG_DOMAIN, G_LOG_LEVEL_INFO, "%s: Restarting queue processing", G_STRLOC);
			      this->private->queue_src_id = g_timeout_add (2000, (GSourceFunc) process_queue, this);
			  }

			g_free (aux);
			g_free (str);

		    }
		    break;

		case SCROBBLER_HANDSHAKE_BADUSER:
		    {
			g_log (BMPX_SCROBBLER_LOG_DOMAIN, G_LOG_LEVEL_INFO, "%s: BADUSER. INTERVAL:'%d'", G_STRLOC, 
						  this->private->info->interval);

			bmp_system_control_message_dispatch	    (bmp_system_control,
						    		     this->private->message_domain_id, 
								     GTK_MESSAGE_ERROR,
								     _("Please check your Last.FM username/password!\n\nLast.FM support has to be disabled until proper credentials are provided.")); 
			xcs_key_set_boolean (xcs, "lastfm", "enable", FALSE);
		    }
		    break;

		case SCROBBLER_HANDSHAKE_FAILED:
		    {
			gchar *aux;

			g_log (BMPX_SCROBBLER_LOG_DOMAIN, G_LOG_LEVEL_INFO, "%s: FAILED. REASON:'%s', INTERVAL:'%d'", G_STRLOC, 
						  this->private->info->info,
						  this->private->info->interval);

			aux = g_strdup_printf (_("Connection with Last.FM failed. Reason given is: '%s'"),
						this->private->info->info);

			bmp_system_control_message_dispatch	    (bmp_system_control,
						    		     this->private->message_domain_id, 
								     GTK_MESSAGE_ERROR,
								     aux); 

			g_free (aux);
			xcs_key_set_boolean (xcs, "lastfm", "enable", FALSE);
		    }
		    break;

		case SCROBBLER_HANDSHAKE_NETWORK_ERROR:
		    {
			g_log (BMPX_SCROBBLER_LOG_DOMAIN, G_LOG_LEVEL_INFO, "%s: NETWORK ERROR", G_STRLOC); 

			bmp_system_control_message_dispatch	    (bmp_system_control,
						    		     this->private->message_domain_id, 
								     GTK_MESSAGE_ERROR,
								     _("A network error occured during connection handshake with Last.FM")); 


			xcs_key_set_boolean (xcs, "lastfm", "enable", FALSE);
		    }
		    break;

		case SCROBBLER_HANDSHAKE_STATE_UNDEFINED:
		    {
			g_log (BMPX_SCROBBLER_LOG_DOMAIN, G_LOG_LEVEL_INFO, "%s: STATE UNDEFINED", G_STRLOC); 

			//FIXME: Error message
			xcs_key_set_boolean (xcs, "lastfm", "enable", FALSE);
		    }
		    break;

		default:
			g_log (BMPX_SCROBBLER_LOG_DOMAIN, G_LOG_LEVEL_WARNING, "%s: Unknown Scrobbler handshake status response!", G_STRLOC);
			break;
	}

    this->private->handshaked = TRUE;
}
