#include <glib.h>
#include <glib-object.h>

#include "signal_handler_pool.h"

SignalHandlerPool*
signal_handler_pool_new (const gchar *name)
{
    SignalHandlerPool *pool;

    pool = g_new0 (SignalHandlerPool, 1);
    pool->name = g_strdup (name);
    pool->conns = NULL;

    return pool;
}

void
signal_handler_pool_add_connection (SignalHandlerPool	  *pool,
				    gpointer		   instance,
				    const gchar		  *sigspec,
				    GCallback		   callback,
				    gpointer		   data,
				    GConnectFlags	   flags)
{
    SignalHandlerPoolTuple *stuple;

    stuple = g_new0 (SignalHandlerPoolTuple, 1);

    stuple->instance = instance;

    if (flags & G_CONNECT_SWAPPED)
      {
	stuple->handler = g_signal_connect_swapped (G_OBJECT(instance), 
						    sigspec,
						    G_CALLBACK (callback),
						    data); 
      }
    else
    if (flags & G_CONNECT_AFTER)
      {
	stuple->handler = g_signal_connect_after   (G_OBJECT(instance), 
						    sigspec,
						    G_CALLBACK (callback),
						    data); 
      }
    else
      {
	stuple->handler = g_signal_connect         (G_OBJECT(instance), 
						    sigspec,
						    G_CALLBACK (callback),
						    data); 
      }

#if 0
    g_message ("%s(%d): [%s] Connected object %s:%p with handler id %lu", __FILE__, __LINE__, pool->name,
	    G_OBJECT_TYPE_NAME (G_OBJECT(stuple->instance)), stuple->instance, stuple->handler);
#endif

    pool->conns = g_list_append (pool->conns, stuple);
}

void
signal_handler_pool_free	    (SignalHandlerPool	  *pool)
{
    GList *conns;

    conns = pool->conns; 

    for ( ; conns ; conns = conns->next)
      {
	  SignalHandlerPoolTuple *stuple = SIGNAL_HANDLER_POOL_TUPLE(conns->data);

#if 0
	  if (G_IS_OBJECT(stuple->instance))
	    g_message ("%s(%d): [%s] Disconnecting handler id %lu on object %s:%p", __FILE__, __LINE__, pool->name, stuple->handler, 
	    G_OBJECT_TYPE_NAME (G_OBJECT(stuple->instance)), stuple->instance);
#endif

	  if (stuple->handler)
	    g_signal_handler_disconnect (stuple->instance, stuple->handler);
      }

    g_list_foreach (pool->conns, (GFunc)g_free, NULL);

    g_free (pool->name);
    g_free (pool);
}
