/*  BMPx - The Dumb Music Player
 *  Copyright (C) 2005 BMPx development team.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id$
 *
 * The BMPx project hereby grant permission for non-gpl compatible GStreamer
 * plugins to be used and distributed together with GStreamer and BMPx. This
 * permission are above and beyond the permissions granted by the GPL license
 * BMPx is covered by.
 *  $Id$
 */

/* For sigaction() and SA_RESTART */
#include <config.h>

#ifdef HAVE_GUI
#include <gtk/gtk.h>
#else
#include <glib.h>
#endif

#include <glib/gi18n.h>
#include <stdlib.h>

#include <unistd.h>
#include <sys/types.h>
#include <signal.h>

#include "signals.h"

#include "main.h"


#define SIGNAL_CHECK_INTERVAL 1000


typedef void (*SignalHandler) (gint);


static gboolean terminate = FALSE;


static SignalHandler
signal_install_handler_full (gint           signal_number,
                             SignalHandler  handler,
                             gint          *signals_to_block,
                             gsize          n_signals)
{
  struct sigaction action, old_action;
  gsize i;

  action.sa_handler = handler;
  action.sa_flags = SA_RESTART;

  sigemptyset (&action.sa_mask);

  for (i = 0; i < n_signals; i++)
    sigaddset (&action.sa_mask, signals_to_block[i]);

  if (sigaction (signal_number, &action, &old_action) == -1)
    {
      g_message ("Failed to install handler for signal %d", signal_number);
      return NULL;
    }

    return old_action.sa_handler;
}

/*
 * A version of signal() that works more reliably across different
 * platforms. It:
 * a. restarts interrupted system calls
 * b. does not reset the handler
 * c. blocks the same signal within the handler
 *
 * (adapted from Unix Network Programming Vol. 1)
 */
static SignalHandler
signal_install_handler (gint          signal_number,
                        SignalHandler handler)
{
  return signal_install_handler_full (signal_number, handler, NULL, 0);
}

static void
signal_empty_handler (gint signal_number)
{
  /* empty */
}

static void
sigsegv_handler (gint signal_number)
{
  g_printerr (_("BMPx has crashed. If you suspect a bug, please consult "
                 "http://bmpx.beep-media-player.org/site/Reporting_bugs\n"));

#ifdef HANDLE_SIGSEGV
  /* TODO: backup session, config and playlist */

  exit (EXIT_FAILURE);
#else
  abort ();
#endif
}

static void
sigterm_handler (gint signal_number)
{
  terminate = TRUE;
}

static gboolean
process_signals (gpointer data)
{
  if (terminate)
    {
      g_message (_("SIGTERM received, quitting."));

#ifdef HAVE_GUI
      if (progress_running)
	{
	  exit (0);
	}
      else
	{
	  gtk_main_quit ();
	}
#else
      g_main_loop_quit (mainloop);
#endif
      return FALSE;
    }

  return TRUE;
}

void
signal_handlers_init (void)
{
  signal_install_handler (SIGPIPE, signal_empty_handler);
  signal_install_handler (SIGSEGV, sigsegv_handler);
  signal_install_handler (SIGINT, sigterm_handler);
  signal_install_handler (SIGTERM, sigterm_handler);

  g_timeout_add (SIGNAL_CHECK_INTERVAL, process_signals, NULL);
}
