/*  BMPx - The Dumb Music Player
 *  Copyright (C) 2005 BMPx development team.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * The BMPx project hereby grant permission for non-gpl compatible GStreamer
 * plugins to be used and distributed together with GStreamer and BMPx. This
 * permission are above and beyond the permissions granted by the GPL license
 * BMPx is covered by.
 */


#include <stdlib.h>
#include <glib/gi18n.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <glade/glade.h>
#include <string.h>

#include "system_control.h"
#include "resource_manager.h"

#include "ui.h"
#include "ui_util.h"

#include <bmp/dbus.h>
#include <bmp/util.h>
#include <bmp/uri.h>
#include <bmp/playlist.h>
#include <bmp/metadata.h>

#include <xcs/xml.h>
#include <xcs/xcs.h>
#include <goa/libgoa.h>

#include "bookmarks.h"
#include "dnd.h"
#include "embedded-images/media-pixbufs.h"
#include "glade.h"
#include "loader.h"
#include "signal_handler_pool.h"

#include "main.h"
#include "ui_dialog_streams.h"

#define _ustrcmp(a,b) strcmp((gchar*)a,(gchar*)b)

/* GObject stuff */
G_DEFINE_TYPE (BmpRadioStreams, bmp_streams, G_TYPE_OBJECT)

struct _BmpRadioStreamsPrivate {

    gboolean	       dispose_has_run;

    GladeXML	      *xml;
    SignalHandlerPool *pool;

    gboolean	       clear;
    gboolean	       start_playback;
    gint	       start_playback_track;
};

static void
window_streams_create (BmpRadioStreams *this);

BmpRadioStreams*
bmp_streams_new (void)
{
    BmpRadioStreams *this;

    this = g_object_new (BMP_TYPE_RADIO_STREAMS, NULL);
    return this;
}

static GObject*
bmp_streams_constructor (GType		        type,
	                 guint			n_construct_properties,
         	         GObjectConstructParam *construct_properties)
{
    GObject *obj;

    {
        BmpRadioStreamsClass *klass;
        GObjectClass *parent_class;
        klass = BMP_RADIO_STREAMS_CLASS (g_type_class_peek (BMP_TYPE_RADIO_STREAMS));
        parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
        obj =
            parent_class->constructor (type,
                                       n_construct_properties,
                                       construct_properties);
    }

    return obj;
}

static void
bmp_streams_dispose (GObject *object)
{
    BmpRadioStreams *this = (BmpRadioStreams *)object;
    signal_handler_pool_free (this->private->pool);
    g_object_unref (this->private->xml);
    gtk_widget_destroy (this->window);
}

static void
bmp_streams_class_init (BmpRadioStreamsClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

#if 0
    gobject_class->set_property = bmp_play_set_property;
    gobject_class->get_property = bmp_play_get_property;
    gobject_class->finalize = bmp_play_finalize;
#endif
    gobject_class->dispose	= bmp_streams_dispose;
    gobject_class->constructor	= bmp_streams_constructor;

}

static void
bmp_streams_init (BmpRadioStreams *this)
{
    this->private = g_new (BmpRadioStreamsPrivate, 1);

    this->private->dispose_has_run = FALSE;
    this->private->pool = signal_handler_pool_new ("UiRadioStreams");

    this->private->xml = NULL;
    this->private->clear = FALSE;
    this->private->start_playback = FALSE;
    this->private->start_playback_track = 0;

    window_streams_create (this);
}

static GtkTargetEntry dnd_entries[] = {
    {"application/x-bmpx-db-query", 0, BMP_DROP_QUERY},
};

static DNDQueryTuple *qtuple = NULL;

enum {
    TAB_BOOKMARKS,
    TAB_SHOUTCAST,
    TAB_ICECAST
};

static void
on_button_enqueue_clicked (GtkButton	    *button, 
			   BmpRadioStreams  *this)
{
    GladeXML		*xml;
    GtkWidget		*window;
    GtkNotebook		*streams_notebook;
    GtkWidget		*tree_view;
    GtkTreeModel	*store;
    GtkTreeSelection	*selection;
    GList		*uris = NULL;
    GtkTreeIter		 iter;
    gchar	       **uri_list;
    gchar		*uri;
    gint		 end_position,
			 current_page;

    xml = this->private->xml;
    window = this->window; 

    bmp_window_set_busy (GTK_WIDGET(window));

    streams_notebook = GTK_NOTEBOOK (glade_xml_get_widget_warn (xml, "streams_notebook"));
    current_page = gtk_notebook_get_current_page (streams_notebook);

    switch (current_page)
      {
	case TAB_ICECAST:
	  {
	    tree_view =
		glade_xml_get_widget_warn (xml, "tree_view_icecast");
	    selection =
		gtk_tree_view_get_selection (GTK_TREE_VIEW(tree_view));

	    if (!gtk_tree_selection_get_selected (selection, &store, &iter)) return; 

	    gtk_tree_model_get (store, &iter, 4, &uri, -1);

	    uris = g_list_append (uris, uri);
	    uri_list = glist_to_strv (uris);

	    bmp_system_control_add_uri_list (bmp_system_control,
					    (const gchar**)uri_list,
					    -1,
					    this->private->clear,
					    this->private->start_playback,
					    this->private->start_playback_track,
					    &end_position,
					    NULL);
	  }
	  break;

	case TAB_SHOUTCAST:
	  {
	    tree_view =
		glade_xml_get_widget_warn (xml, "tree_view_shoutcast");
	    selection =
		gtk_tree_view_get_selection (GTK_TREE_VIEW(tree_view));

	    if (!gtk_tree_selection_get_selected (selection, &store, &iter)) return; 

	    gtk_tree_model_get (store, &iter, 5, &uri, -1);

	    uris = g_list_append (uris, uri);
	    uri_list = glist_to_strv (uris);

	    bmp_system_control_add_uri_list (bmp_system_control,
					    (const gchar**)uri_list,
					    -1,
					    this->private->clear,
					    this->private->start_playback,
					    this->private->start_playback_track,
					    &end_position,
					    NULL);
	  }
	  break;

	case TAB_BOOKMARKS:
	  {
	    tree_view =
		glade_xml_get_widget_warn (xml, "tree_view_bookmarks");
	    selection =
		gtk_tree_view_get_selection (GTK_TREE_VIEW(tree_view));

	    if (!gtk_tree_selection_get_selected (selection, &store, &iter)) return; 

	    gtk_tree_model_get (store, &iter, 2, &uri, -1);

	    uris = g_list_append (uris, uri);
	    uri_list = glist_to_strv (uris);

	    bmp_system_control_add_uri_list (bmp_system_control,
					    (const gchar**)uri_list,
					    -1,
					    this->private->clear,
					    this->private->start_playback,
					    this->private->start_playback_track,
					    &end_position,
					    NULL);
	  }
	  break;


    default: break;
  }

  this->private->clear = FALSE;
  this->private->start_playback = FALSE;
  this->private->start_playback_track = 0;

  bmp_window_set_idle (GTK_WIDGET(window));
}

static void
on_button_play_clicked (GtkButton	*button,
			BmpRadioStreams *this)
{
    this->private->clear		 = TRUE;
    this->private->start_playback	 = TRUE;
    this->private->start_playback_track  = 0;

    on_button_enqueue_clicked (GTK_BUTTON (glade_xml_get_widget_warn (this->private->xml, "b_enqueue")), this);
}

static void
bookmark_edit	(BmpRadioStreams    *this,
				 GtkTreeModel	    *model,
				 GtkTreeIter	    *iter)
{
    GladeXML		    *xml;

    enum {
	E_NAME,
	E_COMMENT,
	E_URI,

	N_ENTRIES
    };

    static gchar* entry_names[] = {
	"ab_entry_name",
	"ab_entry_comment",
	"ab_entry_uri"
    };

    GtkWidget		    *ENTRY[N_ENTRIES];
    GtkWidget		    *dialog;

    gchar		    *name,
			    *comment,
			    *uri;
			    
    gint    n,
	    response;

    xml = this->private->xml;

    for (n = 0; n < N_ENTRIES; n++)
	{
	    ENTRY[n] = glade_xml_get_widget_warn (xml, entry_names[n]);
	}

    gtk_tree_model_get (model, iter, 0, &name, 1, &comment, 2, &uri, -1);

    gtk_entry_set_text (GTK_ENTRY(ENTRY[E_NAME]), name ? name : "");
    gtk_entry_set_text (GTK_ENTRY(ENTRY[E_COMMENT]), comment ? comment : "");
    gtk_entry_set_text (GTK_ENTRY(ENTRY[E_URI]), uri ? uri : "");

    dialog = glade_xml_get_widget_warn (xml, "dialog_add_bookmark"); 
    gtk_window_set_title (GTK_WINDOW(dialog), _("Edit Bookmark - BMP"));

    response = gtk_dialog_run (GTK_DIALOG(dialog));

    switch (response)
	{
	    case GTK_RESPONSE_OK:
		{
		    gtk_list_store_set (GTK_LIST_STORE(model), iter,
					0, gtk_entry_get_text (GTK_ENTRY(ENTRY[E_NAME])),
					1, gtk_entry_get_text (GTK_ENTRY(ENTRY[E_COMMENT])),
					2, gtk_entry_get_text (GTK_ENTRY(ENTRY[E_URI])),
				       -1);
		}

	    default: break;
	}

    gtk_widget_hide (dialog);
}

static void
bookmark_add		(BmpRadioStreams    *this,
				 gchar		    *name,
				 gchar		    *comment,
				 gchar		    *uri)
{
    GladeXML		    *xml;

    enum {
	E_NAME,
	E_COMMENT,
	E_URI,

	N_ENTRIES
    };

    static gchar* entry_names[] = {
	"ab_entry_name",
	"ab_entry_comment",
	"ab_entry_uri"
    };

    GtkWidget		    *ENTRY[N_ENTRIES];
    GtkWidget		    *dialog;
			    
    gint    n,
	    response;

    xml = this->private->xml;

    for (n = 0; n < N_ENTRIES; n++)
	{
	    ENTRY[n] = glade_xml_get_widget_warn (xml, entry_names[n]);
	}

    gtk_entry_set_text (GTK_ENTRY(ENTRY[E_NAME]), name ? name : "");
    gtk_entry_set_text (GTK_ENTRY(ENTRY[E_COMMENT]), comment ? comment : "");
    gtk_entry_set_text (GTK_ENTRY(ENTRY[E_URI]), uri ? uri : "");

    dialog = glade_xml_get_widget_warn (xml, "dialog_add_bookmark"); 
    gtk_window_set_title (GTK_WINDOW(dialog), _("Bookmark Stream - BMP"));

    response = gtk_dialog_run (GTK_DIALOG(dialog));

    switch (response)
	{
	    case GTK_RESPONSE_OK:
		{
		    GtkTreeModel *model;
		    GtkTreeIter   iter;

		    model = bookmarks_get_treemodel ();

		    gtk_list_store_append (GTK_LIST_STORE(model), &iter); 
		    gtk_list_store_set (GTK_LIST_STORE(model), &iter,
					0, gtk_entry_get_text (GTK_ENTRY(ENTRY[E_NAME])),
					1, gtk_entry_get_text (GTK_ENTRY(ENTRY[E_COMMENT])),
					2, gtk_entry_get_text (GTK_ENTRY(ENTRY[E_URI])),
				       -1);
		}

	    default: break;
	}

    gtk_widget_hide (dialog);

}

static void
on_bookmark_shoutcast_clicked	(GtkWidget	*widget,
				 gpointer	 data)
{
    BmpRadioStreams	    *this;
    GladeXML		    *xml;
    GtkWidget		    *tree_view;
    GtkTreeSelection	    *selection; 
    GtkTreeModel	    *store;
    GtkTreeIter		     iter;

    gchar		    *name,
			    *uri,
			    *comment;

    this = BMP_RADIO_STREAMS (data);
    xml  = this->private->xml;
    
    tree_view =
	glade_xml_get_widget_warn (xml, "tree_view_shoutcast");
    selection =
	gtk_tree_view_get_selection (GTK_TREE_VIEW(tree_view));
    
    if (!gtk_tree_selection_get_selected (selection, &store, &iter)) return; 

    gtk_tree_model_get (store, &iter, 0, &comment, 2, &name, 5, &uri, -1); 

    bookmark_add (this, name, comment, uri);
}

static void
on_bookmark_icecast_clicked	(GtkWidget	*widget,
				 gpointer	 data)
{
    BmpRadioStreams	    *this;
    GladeXML		    *xml;
    GtkWidget		    *tree_view;
    GtkTreeSelection	    *selection; 
    GtkTreeModel	    *store;
    GtkTreeIter		     iter;

    gchar		    *name,
			    *uri,
			    *comment;

    this = BMP_RADIO_STREAMS (data);
    xml  = this->private->xml;
    
    tree_view =
	glade_xml_get_widget_warn (xml, "tree_view_icecast");
    selection =
	gtk_tree_view_get_selection (GTK_TREE_VIEW(tree_view));
    
    if (!gtk_tree_selection_get_selected (selection, &store, &iter)) return; 

    gtk_tree_model_get (store, &iter, 0, &comment, 2, &name, 4, &uri, -1); 

    bookmark_add (this, name, comment, uri);
}

static void
on_bookmark_add_clicked	    (GtkWidget	*widget,
			     gpointer	 data)
{
    bookmark_add (BMP_RADIO_STREAMS(data), NULL, NULL, NULL);
}

static void
on_bookmark_edit_clicked    (GtkWidget	*widget,
			     gpointer	 data)
{
    BmpRadioStreams	    *this;
    GladeXML		    *xml;
    GtkWidget		    *tree_view;
    GtkTreeSelection	    *selection; 
    GtkTreeModel	    *store;
    GtkTreeIter		     iter;

    this = BMP_RADIO_STREAMS (data);
    xml  = this->private->xml;
    
    tree_view =
	glade_xml_get_widget_warn (xml, "tree_view_bookmarks");
    selection =
	gtk_tree_view_get_selection (GTK_TREE_VIEW(tree_view));
    
    if (!gtk_tree_selection_get_selected (selection, &store, &iter)) return; 

    bookmark_edit (this, store, &iter);
}

static void
on_bookmark_delete_clicked  (GtkWidget	*widget,
			     gpointer	 data)
{
    BmpRadioStreams	    *this;
    GladeXML		    *xml;
    GtkWidget		    *tree_view,
			    *dialog_delete_bookmark,
			    *entry_bookmark_name;
    GtkTreeSelection	    *selection; 
    GtkTreeModel	    *store;
    GtkTreeIter		     iter;
    gint		     response;
    gchar		    *name;

    this = BMP_RADIO_STREAMS (data);
    xml  = this->private->xml;

    dialog_delete_bookmark =
	glade_xml_get_widget_warn (xml, "dialog_delete_bookmark");
    entry_bookmark_name =
	glade_xml_get_widget_warn (xml, "entry_bookmark_name");
    tree_view =
	glade_xml_get_widget_warn (xml, "tree_view_bookmarks");
    selection =
	gtk_tree_view_get_selection (GTK_TREE_VIEW(tree_view));
    
    if (!gtk_tree_selection_get_selected (selection, &store, &iter)) return; 

    gtk_tree_model_get (store, &iter, 0, &name, -1);
    gtk_entry_set_text (GTK_ENTRY(entry_bookmark_name), name);

    response = gtk_dialog_run (GTK_DIALOG(dialog_delete_bookmark));

    if (response == GTK_RESPONSE_YES)
	gtk_list_store_remove (GTK_LIST_STORE(store), &iter);
    
    gtk_widget_hide (dialog_delete_bookmark);
}


//----------- STREAM LISTERS

static gboolean
model_filter_func (GtkTreeModel *model,
		   GtkTreeIter *iter,
                   gpointer data)
{
    GtkEntry *entry = GTK_ENTRY(data);
    gchar    *_name,  *name,
	     *_genre, *genre,
	     *entry_text_lc;

    const gchar *entry_text;

    gtk_tree_model_get (model, iter, 0, &_genre, 2, &_name, -1);
    entry_text = gtk_entry_get_text (entry);

    if ((!entry_text) || (strlen(entry_text)==0))
      return TRUE;

    entry_text_lc = g_utf8_strdown (entry_text, -1);
  
    name = g_utf8_strdown (_name, -1); 
    genre = g_utf8_strdown (_genre, -1); 

    if ((match_keys (name, entry_text_lc)) ||
        (match_keys (genre, entry_text_lc))) 
      {
	g_free (entry_text_lc);
	g_free (_name);
	g_free (name);
	g_free (_genre);
	g_free (genre);
	return TRUE;
      }

  g_free (entry_text_lc);
  g_free (_name);
  g_free (name);
  g_free (_genre);
  g_free (genre);

  return FALSE;  
}

typedef struct _SortDataTuple SortDataTuple;
struct _SortDataTuple {
      GtkTreeView   *tree_view;
      gint	     column_id;
};

void
column_clicked (GtkTreeViewColumn *tree_view_column,
		gpointer	   data)
{
    GtkTreeViewColumn *column;
    GtkTreeModel      *model_filter;
    GtkTreeModel      *model_real;
    GtkTreeView	      *tree_view;
  
    GtkSortType	       sort_type, sort_type_new;
    gint	       sort_id;

    SortDataTuple *sort_data_tuple = (SortDataTuple*)data;    

    tree_view = sort_data_tuple->tree_view; 
    model_filter = gtk_tree_view_get_model (tree_view);
    model_real = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model_filter));

    gtk_tree_sortable_get_sort_column_id (GTK_TREE_SORTABLE(model_real), &sort_id, &sort_type);
    if ((sort_id >= 0) && (sort_id != sort_data_tuple->column_id))
      {
	column = gtk_tree_view_get_column (tree_view, sort_id);
	gtk_tree_view_column_set_sort_indicator (column, FALSE);
      }

    if (sort_id >= 0)
      sort_type_new = (sort_type == GTK_SORT_ASCENDING) ? GTK_SORT_DESCENDING : GTK_SORT_ASCENDING;
    else
      sort_type_new = GTK_SORT_ASCENDING;

    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE(model_real), sort_data_tuple->column_id, sort_type_new);
    gtk_tree_view_column_set_sort_indicator (tree_view_column, TRUE);
    gtk_tree_view_column_set_sort_order (tree_view_column, sort_type_new);
}

static gchar* shoutcast_headers[] = {
      N_("Stream Genre"),
      N_("Bitrate"),
      N_("Stream Name"),
      N_("Now Playing"),
      N_("Listeners")
};

static gchar* icecast_headers[] = {
      N_("Stream Genre"),
      N_("Bitrate"),
      N_("Stream Name"),
      N_("Now Playing")
};

#if 0
static gchar* peercast_headers[] = {
      N_("Stream Genre"),
      N_("Stream Name"),
      N_("Now Playing"),
      N_("Bitrate")
};

static void
on_refresh_peercast_clicked (GtkWidget	  *widget,
			     gpointer	   data)
{
  xmlDocPtr	      doc;
  xmlXPathObjectPtr   xpathobj;
  xmlNodeSetPtr	      nv;

  GtkTreeModelFilter *list_store_peercast_filter;
  GtkListStore	     *list_store_peercast;
  GtkTreeIter	      iter;

  VFSHandle	     *handle;
  gint		      n;

  typedef struct _PeerCastEntry PeerCastEntry;
  struct _PeerCastEntry { 
      gchar   *Name;
      gchar   *Genre;
      gchar   *Nowplaying;

      gint     Bitrate;

      gchar   *URI;
  };

#define PEERCAST_LIST_URI "http://yp.peercast.org/?xml"

#define PN_URI	     "host"
#define PP_URI	     "ip"

#define PN_NP	     "track"
#define PP_NP	     "title"

#define PP_NAME	     "name"
#define PP_GENRE     "genre"
#define PP_BITRATE   "bitrate"
  
  handle = vfs_handle_new ();  
  handle->uri = g_strdup (PEERCAST_LIST_URI);
  if (!vfs_handle_read (handle))
    return;

  doc = xmlParseMemory (handle->buffer, strlen(handle->buffer));
  if (!doc)
    return;

  vfs_handle_free (handle);

  xpathobj = xml_execute_xpath_expression (doc, "//channel", NULL);
  nv = xpathobj->nodesetval;

  list_store_peercast = gtk_list_store_new (5,
					     G_TYPE_STRING,	// Name
					     G_TYPE_STRING,	// Genre
					     G_TYPE_STRING,	// Nowplaying
					     G_TYPE_INT,	// Bitrate
					     G_TYPE_STRING);	// URI

  for (n = 0; n < nv->nodeNr; n++) 
    {
      PeerCastEntry	*entry;
      gchar		*id, *ip, *title, *xpath, *aux;
      xmlXPathObjectPtr  xpathobj_aux;
      xmlNodePtr	 node, node_child;

      node = 
	nv->nodeTab[n];

      entry = g_new0 (PeerCastEntry,1);

      entry->Name    = xmlGetProp (node, PP_NAME);
      entry->Genre   = xmlGetProp (node, PP_GENRE);
      entry->Bitrate = xmlGetProp (node, PP_BITRATE);

      id	     = xmlGetProp (node, "id");

      // Get URI
      xpath	     = g_strdup_printf("//channel[@id='%s']//%s", id, PN_URI);
      xpathobj_aux   = xml_execute_xpath_expression (doc, xpath, NULL);
      g_free (xpath);

      ip	    = xmlGetProp (xpathobj_aux->nodesetval->nodeTab[0], PP_URI); 
      entry->URI    = g_strdup_printf ("http://%s", ip);
      g_free (ip);
      xmlXPathFreeObject (xpathobj_aux);

      // Get Title
      xpath	     = g_strdup_printf("//channel[@id='%s']//%s", id, PN_NP);
      xpathobj_aux   = xml_execute_xpath_expression (doc, xpath, NULL);
      g_free (xpath);
      entry->Nowplaying = xmlGetProp (xpathobj_aux->nodesetval->nodeTab[0], PP_NP); 
      xmlXPathFreeObject (xpathobj_aux);

      gtk_list_store_append (list_store_peercast, &iter);
      gtk_list_store_set (list_store_peercast, &iter,
			  0,
			  entry->Genre,
			  1,
			  entry->Name,
			  2,
			  entry->Nowplaying,
			  3,
			  entry->Bitrate,
			  4,
			  entry->URI,
			  -1);

      {
	g_free (entry->Genre);
	g_free (entry->Name);
	g_free (entry->Nowplaying);
	g_free (entry->URI);
	g_free (entry);
      }

    } 

    xmlXPathFreeObject (xpathobj);

    list_store_peercast_filter = 
	gtk_tree_model_filter_new (GTK_TREE_MODEL (list_store_peercast), NULL); 

    gtk_tree_model_filter_set_visible_func (list_store_peercast_filter,
					    model_filter_func,
					    glade_xml_get_widget_warn (GLADE_XML(data), "peercast_filter"),
					    NULL);

    signal_handler_pool_add_connection (this->private->pool,
					glade_xml_get_widget_warn (GLADE_XML(data), "peercast_filter"),
					"changed",	
					G_CALLBACK (gtk_tree_model_filter_refilter),	
					list_store_peercast_filter, 
					G_CONNECT_SWAPPED);	

    gtk_tree_view_set_model (GTK_TREE_VIEW(glade_xml_get_widget_warn (GLADE_XML(data), "tree_view_peercast")), 
			     GTK_TREE_MODEL(list_store_peercast_filter));
}
#endif

static void
on_refresh_icecast_clicked (GtkWidget		*widget,
			    BmpRadioStreams	*this)
{
  xmlDocPtr	      doc;
  xmlXPathObjectPtr   xpathobj;
  xmlNodeSetPtr	      nv;
  xmlNodePtr	      node;

  GtkWidget	     *dialog_streams;
  GtkWidget          *refresh_shoutcast;
  GtkWidget          *notebook_icecast_inner;
  GtkTreeModelFilter *list_store_icecast_filter;
  GtkListStore	     *list_store_icecast;
  GtkTreeIter	      iter;
  VFSHandle	     *handle;
  GladeXML	     *xml;
  gint		      n;

  typedef struct _IceCastEntry IceCastEntry;
  struct _IceCastEntry { 
      gchar   *Name;
      gchar   *Genre;
      gchar   *Nowplaying;

      gint     Bitrate;

      gchar   *URI;
  };

#define ICECAST_LIST_URI "http://dir.xiph.org/yp.xml"

#define IN_URI      "listen_url"
#define IN_NAME	    "server_name"
#define IN_GENRE    "genre"
#define IN_NP	    "current_song"
#define IN_BITRATE  "bitrate"
 
  xml = this->private->xml;
 
  handle = vfs_handle_new ();  
  handle->uri = g_strdup (ICECAST_LIST_URI);

  refresh_shoutcast = glade_xml_get_widget_warn (xml, "refresh_shoutcast");
  notebook_icecast_inner = glade_xml_get_widget_warn (xml, "notebook_icecast_inner");
  dialog_streams = glade_xml_get_widget_warn (xml, "dialog_streams");

  bmp_window_set_busy (dialog_streams);

  gtk_widget_set_sensitive (refresh_shoutcast, FALSE);
  gtk_notebook_set_current_page (GTK_NOTEBOOK(notebook_icecast_inner), 1);

  while (gtk_events_pending()) gtk_main_iteration ();

  if (!vfs_handle_read (handle, VFS_NEED_CONTAINER_FORCE_NONE))
    {
      gtk_notebook_set_current_page (GTK_NOTEBOOK(notebook_icecast_inner), 0);
      gtk_widget_set_sensitive (refresh_shoutcast, TRUE);
      bmp_window_set_idle (dialog_streams);
      while (gtk_events_pending()) gtk_main_iteration ();
      vfs_handle_free (handle);
      return;
    }

  if (!handle->buffer)
    {
      gtk_notebook_set_current_page (GTK_NOTEBOOK(notebook_icecast_inner), 0);
      gtk_widget_set_sensitive (refresh_shoutcast, TRUE);
      bmp_window_set_idle (dialog_streams);
      while (gtk_events_pending()) gtk_main_iteration ();
      return;
    }

  doc = xmlParseMemory (handle->buffer, handle->buffer_size); 
  if (!doc)
    {
      gtk_notebook_set_current_page (GTK_NOTEBOOK(notebook_icecast_inner), 0);
      gtk_widget_set_sensitive (refresh_shoutcast, FALSE);
      bmp_window_set_idle (dialog_streams);
      vfs_handle_free (handle);
      return;
    } 

  vfs_handle_free (handle);

  xpathobj = xml_execute_xpath_expression (doc, BAD_CAST "//entry", NULL);
  nv = xpathobj->nodesetval;

  list_store_icecast = gtk_list_store_new (5,
					     G_TYPE_STRING,	// Genre
					     G_TYPE_INT,	// Bitrate
					     G_TYPE_STRING,	// Name
					     G_TYPE_STRING,	// Nowplaying
					     G_TYPE_STRING);	// URI

  for (n = 0; n < nv->nodeNr; n++) 
    {
      IceCastEntry  *entry;
      xmlNodePtr     node_sibling;

      node = 
	nv->nodeTab[n];
      node_sibling = 
	node->children;

      entry = g_new0 (IceCastEntry,1);

      while (node_sibling)
	{
	    if (node_sibling->type != XML_TEXT_NODE)
	      {
		const gchar *name = (const gchar*)node_sibling->name;

		if (!_ustrcmp (name, IN_NAME)) 
		  {
		    if (node_sibling->children)
		      {
			gchar *_data = (gchar*)XML_GET_CONTENT (node_sibling->children);
			entry->Name = _data; 
		      }
		    else
			entry->Name = g_strdup ("");
		  }
		else if (!_ustrcmp (name, IN_URI)) 
		  {
		    if (node_sibling->children)
		      {
			gchar *_data = (gchar*)XML_GET_CONTENT (node_sibling->children);
			entry->URI = _data; 
		      }
		    else
			entry->Name = g_strdup ("");
		  }
		else if (!_ustrcmp (name, IN_GENRE))
		  {
		    if (node_sibling->children)
		      {
			gchar *_data = (gchar*)XML_GET_CONTENT (node_sibling->children);
			entry->Genre = _data; 
		      }
		    else
			entry->Genre = g_strdup ("");
		  }
		else if (!_ustrcmp (name, IN_NP))
		  {
		    if (node_sibling->children)
		      {
			gchar *_data = (gchar*)XML_GET_CONTENT (node_sibling->children);
			entry->Nowplaying = _data; 
		      }
		    else
			entry->Nowplaying = g_strdup ("");
		  }
		else if (!_ustrcmp (name, IN_BITRATE))
		  {
		    gchar *_data =  (gchar*)XML_GET_CONTENT (node_sibling->children);
		    entry->Bitrate = strtol (_data, NULL, 10);
		    g_free (_data);
		  }
	      }
	    node_sibling = node_sibling->next;
	    while (gtk_events_pending()) gtk_main_iteration ();
	}

      gtk_list_store_append (list_store_icecast, &iter);
      gtk_list_store_set (list_store_icecast, &iter,
			  0,
			  entry->Genre,
			  1,
			  entry->Bitrate,
			  2,
			  entry->Name,
			  3,
			  entry->Nowplaying,
			  4,
			  entry->URI,
			  -1);

      {
	g_free (entry->Genre);
	g_free (entry->Name);
	g_free (entry->Nowplaying);
	g_free (entry->URI);
	g_free (entry);
      }

    } 
    xmlXPathFreeObject (xpathobj);

    list_store_icecast_filter = GTK_TREE_MODEL_FILTER
	(gtk_tree_model_filter_new (GTK_TREE_MODEL (list_store_icecast), NULL));

    gtk_tree_model_filter_set_visible_func (list_store_icecast_filter,
					    model_filter_func,
					    glade_xml_get_widget_warn (xml, "icecast_filter"),
					    NULL);

    signal_handler_pool_add_connection (this->private->pool,
					glade_xml_get_widget_warn (xml, "icecast_filter"),
					"changed",	
					G_CALLBACK (gtk_tree_model_filter_refilter),	
					list_store_icecast_filter, 
					G_CONNECT_SWAPPED);	

    gtk_tree_view_set_model (GTK_TREE_VIEW(glade_xml_get_widget_warn (xml, "tree_view_icecast")), 
			     GTK_TREE_MODEL(list_store_icecast_filter));

    gtk_notebook_set_current_page (GTK_NOTEBOOK(notebook_icecast_inner), 0);
    gtk_widget_set_sensitive (refresh_shoutcast, TRUE);
    bmp_window_set_idle (dialog_streams);

    while (gtk_events_pending ()) gtk_main_iteration();

    gtk_tree_view_columns_autosize (GTK_TREE_VIEW(glade_xml_get_widget_warn (xml, "tree_view_icecast"))); 
}

static void
on_refresh_shoutcast_clicked (GtkWidget		*widget,
			      BmpRadioStreams   *this)
{
  xmlDocPtr	      doc;
  xmlXPathObjectPtr   xpathobj;
  xmlNodeSetPtr	      nv;
  xmlNodePtr	      node;

  GtkWidget	     *dialog_streams;
  GtkWidget          *refresh_icecast;
  GtkWidget          *notebook_shoutcast_inner;
  GtkTreeModelFilter *list_store_shoutcast_filter;
  GtkListStore	     *list_store_shoutcast;
  GtkTreeIter	      iter;
  VFSHandle	     *handle;
  GladeXML	     *xml;
  gint		      n;

  typedef struct _ShoutCastEntry ShoutCastEntry;
  struct _ShoutCastEntry { 
      gchar   *Name;
      gchar   *Genre;
      gchar   *Nowplaying;

      gint     Listeners;
      gint     Bitrate;

      gchar   *URI;
  };

#define SHOUTCAST_LIST_URI "http://www.shoutcast.com/sbin/xmllister.phtml?service=winamp2&limit=1500&no_compress=1"

#define P_URI	    "Playstring"
#define N_NAME	    "Name"
#define N_GENRE	    "Genre"
#define N_NP	    "Nowplaying"
#define N_LISTENERS "Listeners"
#define N_BITRATE   "Bitrate"

  xml = this->private->xml;
  
  handle = vfs_handle_new ();  
  handle->uri = g_strdup (SHOUTCAST_LIST_URI);
  
  refresh_icecast = glade_xml_get_widget_warn (xml, "refresh_icecast");
  notebook_shoutcast_inner = glade_xml_get_widget_warn (xml, "notebook_shoutcast_inner");
  dialog_streams = glade_xml_get_widget_warn (xml, "dialog_streams");

  bmp_window_set_busy (dialog_streams);

  gtk_widget_set_sensitive (refresh_icecast, FALSE);
  gtk_notebook_set_current_page (GTK_NOTEBOOK(notebook_shoutcast_inner), 1);
  while (gtk_events_pending()) gtk_main_iteration ();

  if (!vfs_handle_read (handle, VFS_NEED_CONTAINER_FORCE_NONE))
    {
      gtk_widget_set_sensitive (refresh_icecast, TRUE);
      bmp_window_set_idle (dialog_streams);
      gtk_notebook_set_current_page (GTK_NOTEBOOK(notebook_shoutcast_inner), 0);
      vfs_handle_free (handle);
      return;
    }

  if (!handle->buffer)
    {
      gtk_widget_set_sensitive (refresh_icecast, TRUE);
      bmp_window_set_idle (dialog_streams);
      gtk_notebook_set_current_page (GTK_NOTEBOOK(notebook_shoutcast_inner), 0);
      return;
    }

  doc = xmlParseMemory (handle->buffer, handle->buffer_size); 
  if (!doc)
    {
      gtk_widget_set_sensitive (refresh_icecast, TRUE);
      bmp_window_set_idle (dialog_streams);
      gtk_notebook_set_current_page (GTK_NOTEBOOK(notebook_shoutcast_inner), 0);
      vfs_handle_free (handle);
      return;
    }

  vfs_handle_free (handle);

  xpathobj = xml_execute_xpath_expression (doc, BAD_CAST "//entry", NULL);
  nv = xpathobj->nodesetval;

  list_store_shoutcast = gtk_list_store_new (6,
					     G_TYPE_STRING,	// Genre
					     G_TYPE_INT,	// Bitrate
					     G_TYPE_STRING,	// Name
					     G_TYPE_STRING,	// Nowplaying
					     G_TYPE_INT,	// Listeners
					     G_TYPE_STRING);	// URI

  for (n = 0; n < nv->nodeNr; n++) 
    {
      ShoutCastEntry  *entry;
      xmlNodePtr       node_sibling;

      node = 
	nv->nodeTab[n];
      node_sibling = 
	node->children;

      entry = g_new0 (ShoutCastEntry,1);
      entry->URI = (gchar*) xmlGetProp (node, BAD_CAST P_URI); 

      while (node_sibling)
	{
	    if (node_sibling->type != XML_TEXT_NODE)
	      {
		const unsigned char *name = node_sibling->name;

		if (!_ustrcmp (name, N_NAME)) 
		  {
		    if (node_sibling->children)
		      {
			gchar *_data = (gchar*)XML_GET_CONTENT (node_sibling->children);
			entry->Name = _data; 
		      }
		    else
			entry->Name = g_strdup ("");
		  }
		else if (!_ustrcmp (name, N_GENRE))
		  {
		    if (node_sibling->children)
		      {
			gchar *_data = (gchar*)XML_GET_CONTENT (node_sibling->children);
			entry->Genre = _data; 
		      }
		    else
			entry->Genre = g_strdup ("");
		  }
		else if (!_ustrcmp (name, N_NP))
		  {
		    if (node_sibling->children)
		      {
			gchar *_data = (gchar*)XML_GET_CONTENT (node_sibling->children);
			entry->Nowplaying = _data; 
		      }
		    else
			entry->Nowplaying = g_strdup ("");
		  }
		else if (!_ustrcmp (name, N_LISTENERS))
		  {
		    gchar *_data =  (gchar*)XML_GET_CONTENT (node_sibling->children);
		    entry->Listeners = strtol (_data, NULL, 10);
		    g_free (_data);
		  }
		else if (!_ustrcmp (name, N_BITRATE))
		  {
		    gchar *_data = (gchar*)XML_GET_CONTENT (node_sibling->children);
		    entry->Bitrate = strtol (_data, NULL, 10);
		    g_free (_data);
		  }
	      }
	    node_sibling = node_sibling->next;
	    while (gtk_events_pending()) gtk_main_iteration ();
	}

      gtk_list_store_append (list_store_shoutcast, &iter);
      gtk_list_store_set (list_store_shoutcast, &iter,
			  0,
			  entry->Genre,
			  1,
			  entry->Bitrate,
			  2,
			  entry->Name,
			  3,
			  entry->Nowplaying,
			  4,
			  entry->Listeners,
			  5,
			  entry->URI,
			  -1);

      {
	g_free (entry->Genre);
	g_free (entry->Name);
	g_free (entry->Nowplaying);
	g_free (entry->URI);
	g_free (entry);
      }

    } 
    xmlXPathFreeObject (xpathobj);

    list_store_shoutcast_filter = GTK_TREE_MODEL_FILTER
	(gtk_tree_model_filter_new (GTK_TREE_MODEL (list_store_shoutcast), NULL));

    gtk_tree_model_filter_set_visible_func (list_store_shoutcast_filter,
					    model_filter_func,
					    glade_xml_get_widget_warn (xml, "shoutcast_filter"),
					    NULL);

    signal_handler_pool_add_connection (this->private->pool,
					glade_xml_get_widget_warn (xml, "shoutcast_filter"),
					"changed",	
					G_CALLBACK (gtk_tree_model_filter_refilter),	
					list_store_shoutcast_filter, 
					G_CONNECT_SWAPPED);	

    gtk_tree_view_set_model (GTK_TREE_VIEW(glade_xml_get_widget_warn (xml, "tree_view_shoutcast")), 
			     GTK_TREE_MODEL(list_store_shoutcast_filter));

    gtk_notebook_set_current_page (GTK_NOTEBOOK(notebook_shoutcast_inner), 0);
    gtk_widget_set_sensitive (refresh_icecast, TRUE);
    bmp_window_set_idle (dialog_streams);

    while (gtk_events_pending ()) gtk_main_iteration();

    gtk_tree_view_columns_autosize (GTK_TREE_VIEW(glade_xml_get_widget_warn (xml, "tree_view_icecast"))); 
}

static gboolean
on_dialog_streams_delete (GtkWidget *widget,
			  GdkEvent *event,
                          gpointer user_data)
{
  gtk_widget_hide (widget);
  return TRUE;
}


static void
on_streams_notebook_switch_page (GtkNotebook	  *notebook,
				 GtkNotebookPage  *notebook_page,
				 guint		   page,
				 BmpRadioStreams  *this)
{
    GtkTreeView	      *tree_view;
    GtkTreeSelection  *tree_selection;
    GladeXML	      *xml;
    GtkWidget	      
		      *b_play, 
		      *b_enqueue;

    static gchar* page_names[] = {
	"tree_view_bookmarks",
	"tree_view_shoutcast",
	"tree_view_icecast",  
    };
		
    g_return_if_fail (GTK_IS_NOTEBOOK(notebook));

    xml = this->private->xml; 
    tree_view = GTK_TREE_VIEW(glade_xml_get_widget_warn (xml, page_names[page]));
    tree_selection = gtk_tree_view_get_selection (tree_view);

    b_play = 
	glade_xml_get_widget_warn (xml, "b_play");
    b_enqueue = 
	glade_xml_get_widget_warn (xml, "b_enqueue");

    gtk_widget_set_sensitive (GTK_WIDGET(b_play),
			     (!gtk_tree_selection_count_selected_rows (tree_selection)) ? FALSE : TRUE);

    gtk_widget_set_sensitive (GTK_WIDGET(b_enqueue),
			     (!gtk_tree_selection_count_selected_rows (tree_selection)) ? FALSE : TRUE);
}

static void
on_tree_selection_bookmarks_changed (GtkTreeSelection	*tree_selection,
				     BmpRadioStreams	*this)
{
    GtkWidget	      *widget; 

    static gchar *buttons[] = {
	"b_play",
	"b_enqueue",
	"b_bookmark_delete",
	"b_bookmark_edit"
    };

    gboolean	       sensitive;
    gint	       n;

    sensitive =
	(!gtk_tree_selection_count_selected_rows (tree_selection)) ? FALSE : TRUE;

    for (n = 0; n < G_N_ELEMENTS(buttons); n++)
	{
	    widget = glade_xml_get_widget_warn (this->private->xml, buttons[n]);
	    gtk_widget_set_sensitive (GTK_WIDGET(widget), sensitive);
	}
}


static void
on_tree_selection_icecast_changed (GtkTreeSelection	*tree_selection,
				   BmpRadioStreams	*this)
{
    GtkWidget	      *widget; 

    static gchar *buttons[] = {
	"b_play",
	"b_enqueue",
	"bookmark_icecast",
    };

    gboolean	       sensitive;
    gint	       n;

    sensitive =
	(!gtk_tree_selection_count_selected_rows (tree_selection)) ? FALSE : TRUE;

    for (n = 0; n < G_N_ELEMENTS(buttons); n++)
	{
	    widget = glade_xml_get_widget_warn (this->private->xml, buttons[n]);
	    gtk_widget_set_sensitive (GTK_WIDGET(widget), sensitive);
	}
}


static void
on_tree_selection_shoutcast_changed (GtkTreeSelection	*tree_selection,
				     BmpRadioStreams	*this)
{
    GtkWidget	      *widget; 

    static gchar *buttons[] = {
	"b_play",
	"b_enqueue",
	"bookmark_shoutcast",
    };

    gboolean	       sensitive;
    gint	       n;

    sensitive =
	(!gtk_tree_selection_count_selected_rows (tree_selection)) ? FALSE : TRUE;

    for (n = 0; n < G_N_ELEMENTS(buttons); n++)
	{
	    widget = glade_xml_get_widget_warn (this->private->xml, buttons[n]);
	    gtk_widget_set_sensitive (GTK_WIDGET(widget), sensitive);
	}
}

static GdkPixmap *
tv_shoutcast_create_row_drag_icon (GtkWidget	      *tree_view,
				   BmpRadioStreams    *this)
{
    GtkTreeModel      *tree_model;
    GtkTreeSelection  *tree_selection;
    GtkTreeIter	       iter;
    PangoLayout	      *layout;
    GdkPixbuf	      *pb;
    GdkDrawable	      *drawable;
    cairo_t	      *cr;
    gint	       width;
    gint	       height;
    gint	       depth;
    gchar	      *name,
		      *uri;
  
    GladeXML	      *xml; 

    xml = this->private->xml;

    depth = gdk_visual_get_best_depth ();
    tree_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(tree_view));
    if (!gtk_tree_selection_get_selected (tree_selection, &tree_model, &iter))
	return NULL;

    gtk_tree_model_get (tree_model, &iter, 1, &name, 5, &uri, -1);
 
    qtuple = g_new0 (DNDQueryTuple,1);
    qtuple->name = g_strdup(name);
    qtuple->uri  = g_strdup(uri);

    layout = gtk_widget_create_pango_layout (GTK_WIDGET(tree_view), name);
    pango_layout_get_pixel_size (layout, &width, &height);
    
    height = 28;
    width += 16 + 16 + 8;
    drawable = gdk_pixmap_new (NULL, width, height, depth);

    gtk_paint_box                   (GTK_WIDGET(tree_view)->style,
                                             drawable, 
                                             GTK_STATE_NORMAL, 
                                             GTK_SHADOW_OUT, 
                                             NULL, 
                                             GTK_WIDGET(tree_view), 
                                             "", 
                                             0, 
                                             0, 
                                             width,
                                             height);


    gtk_paint_layout                (GTK_WIDGET(tree_view)->style,
                                             drawable, 
                                             GTK_STATE_NORMAL, 
                                             TRUE, 
                                             NULL, 
                                             GTK_WIDGET(tree_view), 
                                             "", 
                                             32, 
                                             6, 
                                             layout); 

    cr = gdk_cairo_create (drawable);
    pb = gdk_pixbuf_new_from_file (DATA_DIR "/images/shoutcast.png", NULL);
    gdk_cairo_set_source_pixbuf     (cr,
				     pb,   
                                     10, 
				     6); 

    cairo_paint (cr);
    cairo_destroy (cr);

    g_object_unref (pb);

    return drawable;
}

static GdkPixmap *
tv_icecast_create_row_drag_icon (GtkWidget	    *tree_view,
			         BmpRadioStreams    *this)
{
    GtkTreeModel      *tree_model;
    GtkTreeSelection  *tree_selection;
    GtkTreeIter	       iter;
    PangoLayout	      *layout;
    GdkPixbuf	      *pb;
    GdkDrawable	      *drawable;
    cairo_t	      *cr;
    gint	       width;
    gint	       height;
    gint	       depth;
    gchar	      *name,
		      *uri;
    GladeXML	      *xml;

    xml = this->private->xml;

    depth = gdk_visual_get_best_depth ();
    tree_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(tree_view));
    if (!gtk_tree_selection_get_selected (tree_selection, &tree_model, &iter))
	return NULL;

    gtk_tree_model_get (tree_model, &iter, 1, &name, 4, &uri, -1);
 
    qtuple = g_new0 (DNDQueryTuple,1);
    qtuple->name = g_strdup(name);
    qtuple->uri  = g_strdup(uri);

    layout = gtk_widget_create_pango_layout (GTK_WIDGET(tree_view), name);
    pango_layout_get_pixel_size (layout, &width, &height);
    
    height = 28;
    width += 16 + 16 + 8;
    drawable = gdk_pixmap_new (NULL, width, height, depth);

    gtk_paint_box                   (GTK_WIDGET(tree_view)->style,
                                             drawable, 
                                             GTK_STATE_NORMAL, 
                                             GTK_SHADOW_OUT, 
                                             NULL, 
                                             GTK_WIDGET(tree_view), 
                                             "", 
                                             0, 
                                             0, 
                                             width,
                                             height);


    gtk_paint_layout                (GTK_WIDGET(tree_view)->style,
                                             drawable, 
                                             GTK_STATE_NORMAL, 
                                             TRUE, 
                                             NULL, 
                                             GTK_WIDGET(tree_view), 
                                             "", 
                                             32, 
                                             6, 
                                             layout); 

    cr = gdk_cairo_create (drawable);
    pb = gdk_pixbuf_new_from_file (DATA_DIR "/images/xiph.png", NULL);
    gdk_cairo_set_source_pixbuf     (cr,
				     pb,   
                                     10, 
				     6); 

    cairo_paint (cr);
    cairo_destroy (cr);

    g_object_unref (pb);

    return drawable;
}


static void
tv_shoutcast_drag_begin (GtkWidget	      *widget,
			 GdkDragContext	      *ctx,
			 BmpRadioStreams      *this) 
{
    GdkPixmap *pixmap;
    gint       w, 
	       h;

    pixmap = tv_shoutcast_create_row_drag_icon (widget, this); 
    gdk_drawable_get_size (pixmap, &w, &h);
    gtk_drag_set_icon_pixmap (ctx,
                              gdk_drawable_get_colormap (pixmap),
                              pixmap, 
			      NULL, 
			      0, 0); 
}

static void
tv_icecast_drag_begin (GtkWidget	      *widget,
		       GdkDragContext	      *ctx,
		       BmpRadioStreams        *this) 
{
    GdkPixmap *pixmap;
    gint       w, 
	       h;

    pixmap = tv_icecast_create_row_drag_icon (widget, this); 
    gdk_drawable_get_size (pixmap, &w, &h);
    gtk_drag_set_icon_pixmap (ctx,
                              gdk_drawable_get_colormap (pixmap),
                              pixmap, 
			      NULL, 
			      0, 0); 
}


static void
drag_end (GtkWidget	    *widget,
          GdkDragContext    *drag_context,
          BmpRadioStreams   *this) 
{
    if (qtuple)
      {
	  g_free (qtuple->name);
	  g_free (qtuple->uri);
	  g_free (qtuple);
      }
}

static void
tv_shoutcast_drag_data_get (GtkWidget        *widget,
			    GdkDragContext   *drag_context,
			    GtkSelectionData *data,
			    guint             info,
			    guint             time,
                            gpointer          user_data)
{
    gtk_selection_data_set          (data,
                                     gdk_atom_intern("application/x-bmpx-db-query", FALSE), 
                                     8, 
                                     (gpointer) &qtuple, 
                                     sizeof(gpointer)); 
}

static void
tv_icecast_drag_data_get (GtkWidget        *widget,
			  GdkDragContext   *drag_context,
			  GtkSelectionData *data,
			  guint             info,
			  guint             time,
                          gpointer          user_data)
{
    gtk_selection_data_set          (data,
                                     gdk_atom_intern("application/x-bmpx-db-query", FALSE), 
                                     8, 
                                     (gpointer) &qtuple, 
                                     sizeof(gpointer)); 
}

static gboolean
on_dialog_streams_key_press (GtkWidget	    *widget,
			     GdkEventKey    *event,
			     gpointer	     data)
{
    g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);

    if (event->keyval == GDK_Escape)
	gtk_widget_hide (widget);

    return FALSE;
}

static void
window_streams_create (BmpRadioStreams *this)
{
    GtkWidget		  *widget;
    static const gchar	  *glade_file = DATA_DIR "/glade/dialog_streams.glade";
    gint		   n;

    this->private->xml = glade_xml_new_or_die ("Streams Dialog", glade_file, NULL, NULL);
    if (!this->private->xml) return;

    glade_xml_signal_autoconnect (this->private->xml);

    this->private->pool = signal_handler_pool_new ("UiStreams");
    this->window = glade_xml_get_widget_warn (this->private->xml, "dialog_streams");
    gtk_widget_realize (this->window);

    bmp_window_set_icon_list (GTK_WIDGET(this->window), "player");
    gtk_window_set_position (GTK_WINDOW(this->window), GTK_WIN_POS_CENTER);
    RM_ITEM_NEW("windows", "window-dialog-streams", this->window, NULL);

    signal_handler_pool_add_connection (this->private->pool,
					this->window,	
					"delete-event",	
					G_CALLBACK (on_dialog_streams_delete),	
					this,
					0);	

    signal_handler_pool_add_connection (this->private->pool,
					this->window,	
					"key-press-event",	
					G_CALLBACK (on_dialog_streams_key_press),	
					NULL,
					0);	


    /* Various */
    widget = glade_xml_get_widget_warn (this->private->xml, "streams_notebook");
    signal_handler_pool_add_connection (this->private->pool,
					widget,	
					"switch-page",	
					G_CALLBACK (on_streams_notebook_switch_page),	
					this,
					0);	

    gtk_notebook_set_current_page (GTK_NOTEBOOK(widget), 0);

    widget = glade_xml_get_widget_warn (this->private->xml, "image_streams");
    gtk_image_set_from_file (GTK_IMAGE (widget), DATA_DIR "/images/header-streams.png");

    widget = glade_xml_get_widget_warn (this->private->xml, "image_wait1");
    gtk_image_set_from_file (GTK_IMAGE(widget), DATA_DIR "/images/wait.gif");
    widget = glade_xml_get_widget_warn (this->private->xml, "image_wait2");
    gtk_image_set_from_file (GTK_IMAGE(widget), DATA_DIR "/images/wait.gif");

    /* Setup SHOUTCAST Tab */
    widget = glade_xml_get_widget_warn (this->private->xml, "image_shoutcast");
    gtk_image_set_from_file (GTK_IMAGE(widget), DATA_DIR "/images/shoutcast.png");

    widget = glade_xml_get_widget_warn (this->private->xml, "tree_view_shoutcast");
    gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW(widget)), GTK_SELECTION_SINGLE);

    signal_handler_pool_add_connection (this->private->pool,
					widget,	
					"row-activated",	
					G_CALLBACK (gtk_window_activate_default),	
					GTK_WINDOW(this->window),
					G_CONNECT_SWAPPED);	

    signal_handler_pool_add_connection (this->private->pool,
					gtk_tree_view_get_selection (GTK_TREE_VIEW(widget)),    
					"changed",	
					G_CALLBACK (on_tree_selection_shoutcast_changed),	
					this, 
					0);	

    gtk_drag_source_set (GTK_WIDGET(widget),
			     GDK_MODIFIER_MASK, 
			     dnd_entries, 
			     G_N_ELEMENTS(dnd_entries), 
                             GDK_ACTION_COPY); 

    signal_handler_pool_add_connection (this->private->pool,
					    widget, 
					    "drag-begin",	
					    G_CALLBACK (tv_shoutcast_drag_begin),	
					    this, 
					    0);	

    signal_handler_pool_add_connection (this->private->pool,
					    widget, 
					    "drag-end",	
					    G_CALLBACK (drag_end),	
					    this, 
					    0);	

    signal_handler_pool_add_connection (this->private->pool,
					    widget, 
					    "drag-data-get",	
					    G_CALLBACK (tv_shoutcast_drag_data_get),	
					    this, 
					    0);	

    for (n = 0; n < G_N_ELEMENTS (shoutcast_headers); n++)
      {
	    GtkTreeViewColumn *column;
	    GtkCellRenderer   *cell;
	    SortDataTuple     *sort_data_tuple;

	    sort_data_tuple = g_new0 (SortDataTuple,1);

	    sort_data_tuple->tree_view = GTK_TREE_VIEW (widget);
	    sort_data_tuple->column_id = n;

	    column = 
	      gtk_tree_view_column_new ();
	    cell =
	      gtk_cell_renderer_text_new ();

	    gtk_tree_view_column_set_resizable (column, TRUE);
	    gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_GROW_ONLY);

	    gtk_tree_view_column_pack_start
		(column,
		 cell,
		 TRUE);  

	    gtk_tree_view_column_set_attributes
		(column,
		 cell, 
		 "text",
		  n,
		  NULL);

	    gtk_tree_view_column_set_clickable (column, TRUE);
	    signal_handler_pool_add_connection (this->private->pool,
						column, 
						"clicked",	
						G_CALLBACK (column_clicked),	
						sort_data_tuple, 
						0);	

	    gtk_tree_view_column_set_title (column, _(shoutcast_headers[n]));
	    gtk_tree_view_append_column (GTK_TREE_VIEW(widget), column);
      }

    signal_handler_pool_add_connection (this->private->pool,
					    glade_xml_get_widget_warn (this->private->xml, "refresh_shoutcast"),    
					    "clicked",	
					    G_CALLBACK (on_refresh_shoutcast_clicked),	
					    this, 
					    0);	

    signal_handler_pool_add_connection (this->private->pool,
					    glade_xml_get_widget_warn (this->private->xml, "bookmark_shoutcast"),    
					    "clicked",	
					    G_CALLBACK (on_bookmark_shoutcast_clicked),	
					    this, 
					    0);	


    /* Setup ICECAST Tab */
    widget = glade_xml_get_widget_warn (this->private->xml, "image_icecast");
    gtk_image_set_from_file (GTK_IMAGE(widget), DATA_DIR "/images/xiph.png");

    widget = glade_xml_get_widget_warn (this->private->xml, "tree_view_icecast");
    gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW(widget)), GTK_SELECTION_SINGLE);

    signal_handler_pool_add_connection (this->private->pool,
					    widget, 
					    "row-activated",	
					    G_CALLBACK (gtk_window_activate_default),	
					    this->window, 
					    G_CONNECT_SWAPPED);	

    signal_handler_pool_add_connection (this->private->pool,
					    gtk_tree_view_get_selection (GTK_TREE_VIEW(widget)),    
					    "changed",	
					    G_CALLBACK (on_tree_selection_icecast_changed),	
					    this, 
					    0);	

    gtk_drag_source_set (GTK_WIDGET(widget),
			     GDK_MODIFIER_MASK, 
			     dnd_entries, 
			     G_N_ELEMENTS(dnd_entries), 
                             GDK_ACTION_COPY); 

    signal_handler_pool_add_connection (this->private->pool,
					    widget, 
					    "drag-begin",	
					    G_CALLBACK (tv_icecast_drag_begin),	
					    this, 
					    0);	

    signal_handler_pool_add_connection (this->private->pool,
					    widget, 
					    "drag-end",	
					    G_CALLBACK (drag_end),	
					    this, 
					    0);	

    signal_handler_pool_add_connection (this->private->pool,
					    widget, 
					    "drag-data-get",	
					    G_CALLBACK (tv_icecast_drag_data_get),	
					    this, 
					    0);	

    for (n = 0; n < G_N_ELEMENTS (icecast_headers); n++)
      {
	    GtkTreeViewColumn *column;
	    GtkCellRenderer   *cell;
	    SortDataTuple     *sort_data_tuple;

	    sort_data_tuple = g_new0 (SortDataTuple,1);

	    sort_data_tuple->tree_view = GTK_TREE_VIEW (widget);
	    sort_data_tuple->column_id = n;

	    column = 
	      gtk_tree_view_column_new ();
	    cell =
	      gtk_cell_renderer_text_new ();

	    gtk_tree_view_column_set_resizable (column, TRUE);
	    gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_GROW_ONLY);

	    gtk_tree_view_column_pack_start
		(column,
		 cell,
		 TRUE);  

	    gtk_tree_view_column_set_attributes
		(column,
		 cell, 
		 "text",
		  n,
		  NULL);

	    gtk_tree_view_column_set_clickable (column, TRUE);
	    signal_handler_pool_add_connection (this->private->pool,
						column, 
						"clicked",	
						G_CALLBACK (column_clicked),	
						sort_data_tuple, 
						0);	


	    gtk_tree_view_column_set_title
		(column,
		 _(icecast_headers[n]));

	    gtk_tree_view_append_column (GTK_TREE_VIEW(widget), column);
      }

    signal_handler_pool_add_connection (this->private->pool,
					    glade_xml_get_widget_warn (this->private->xml, "refresh_icecast"),    
					    "clicked",	
					    G_CALLBACK (on_refresh_icecast_clicked),	
					    this, 
					    0);	

    signal_handler_pool_add_connection (this->private->pool,
					    glade_xml_get_widget_warn (this->private->xml, "bookmark_icecast"),    
					    "clicked",	
					    G_CALLBACK (on_bookmark_icecast_clicked),	
					    this, 
					    0);	


#if 0
	/* Setup PEERCAST Tab */
	widget = glade_xml_get_widget_warn (xml, "tree_view_peercast");
	g_object_connect(G_OBJECT(widget), 
			 "swapped-signal::row-activated", 
			  G_CALLBACK(gtk_window_activate_default), 
			  GTK_WINDOW(dialog_streams), 
			  NULL);

	for (n = 0; n < G_N_ELEMENTS (peercast_headers); n++)
	  {
	    GtkTreeViewColumn *column;
	    GtkCellRenderer   *cell;
	    SortDataTuple     *sort_data_tuple;

	    sort_data_tuple = g_new0 (SortDataTuple,1);

	    sort_data_tuple->tree_view = widget;
	    sort_data_tuple->column_id = n;

	    column = 
	      gtk_tree_view_column_new ();
	    cell =
	      gtk_cell_renderer_text_new ();

	    gtk_tree_view_column_set_resizable (column, TRUE);
	    gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);

	    gtk_tree_view_column_pack_start
		(column,
		 cell,
		 TRUE);  

	    gtk_tree_view_column_set_attributes
		(column,
		 cell, 
		 "text",
		  n,
		  NULL);

	    gtk_tree_view_column_set_clickable (column, TRUE);
	    g_object_connect (G_OBJECT(column),
			      "signal::clicked",
			      G_CALLBACK (column_clicked),
			      sort_data_tuple, 
			      NULL);

	    gtk_tree_view_column_set_title
		(column,
		 _(peercast_headers[n]));

	    gtk_tree_view_append_column (GTK_TREE_VIEW(widget), column);
	  }

	g_object_connect(G_OBJECT(glade_xml_get_widget_warn (xml, "refresh_peercast")),
		         "signal::clicked", 
			 G_CALLBACK(on_refresh_peercast_clicked), 
			 xml, 
		         NULL); 
#endif

	/* Bookmarks tab */
	widget = glade_xml_get_widget_warn (this->private->xml, "tree_view_bookmarks");

	signal_handler_pool_add_connection (this->private->pool,
					    gtk_tree_view_get_selection (GTK_TREE_VIEW(widget)),    
					    "changed",	
					    G_CALLBACK (on_tree_selection_bookmarks_changed),	
					    this, 
					    0);	

	g_object_connect(G_OBJECT(widget), 
			 "swapped-signal::row-activated", 
			  G_CALLBACK(gtk_window_activate_default), 
			  GTK_WINDOW(this->window), 
			  NULL);

	{
	    GtkTreeViewColumn *column;
	    GtkCellRenderer   *cell;

	    column = 
	      gtk_tree_view_column_new ();
	    cell =
	      gtk_cell_renderer_text_new ();
	    gtk_tree_view_column_pack_start
		(column,
		 cell,
		 TRUE);  
	    gtk_tree_view_column_set_attributes
		(column,
		 cell, 
		 "text",
		  0,
		  NULL);
	    gtk_tree_view_column_set_title
		(column,
		 _("Name")); 

	    gtk_tree_view_column_set_sort_column_id (column, 0);
	    gtk_tree_view_column_set_resizable (column, TRUE);
	    gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_GROW_ONLY);

	    gtk_tree_view_append_column (GTK_TREE_VIEW(widget), column);


	    column = 
	      gtk_tree_view_column_new ();
	    cell =
	      gtk_cell_renderer_text_new ();
	    gtk_tree_view_column_pack_start
		(column,
		 cell,
		 TRUE);  
	    gtk_tree_view_column_set_attributes
		(column,
		 cell, 
		 "text",
		  1,
		  NULL);
	    gtk_tree_view_column_set_title
		(column,
		 _("Comment")); 

	    gtk_tree_view_column_set_sort_column_id (column, 1);
	    gtk_tree_view_column_set_resizable (column, TRUE);
	    gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_GROW_ONLY);

	    gtk_tree_view_append_column (GTK_TREE_VIEW(widget), column);

	  }

	gtk_tree_view_set_model (GTK_TREE_VIEW(widget), bookmarks_get_treemodel() );

	/* Common widgets */
	widget = glade_xml_get_widget_warn (this->private->xml, "b_bookmark_edit");
	signal_handler_pool_add_connection (this->private->pool,
					    widget, 
					    "clicked",	
					    G_CALLBACK (on_bookmark_edit_clicked),	
					    this, 
					    0);	

	widget = glade_xml_get_widget_warn (this->private->xml, "b_bookmark_delete");
	signal_handler_pool_add_connection (this->private->pool,
					    widget, 
					    "clicked",	
					    G_CALLBACK (on_bookmark_delete_clicked),	
					    this, 
					    0);	

	widget = glade_xml_get_widget_warn (this->private->xml, "b_bookmark_add");
	signal_handler_pool_add_connection (this->private->pool,
					    widget, 
					    "clicked",	
					    G_CALLBACK (on_bookmark_add_clicked),	
					    this, 
					    0);	

	widget = glade_xml_get_widget_warn (this->private->xml, "b_enqueue");
	signal_handler_pool_add_connection (this->private->pool,
					    widget, 
					    "clicked",	
					    G_CALLBACK (on_button_enqueue_clicked),	
					    this, 
					    0);	

	widget = glade_xml_get_widget_warn (this->private->xml, "b_play");
	signal_handler_pool_add_connection (this->private->pool,
					    widget, 
					    "clicked",	
					    G_CALLBACK (on_button_play_clicked),	
					    this, 
					    0);	
}

void
bmp_streams_show (BmpRadioStreams *this)
{
    gtk_window_present (GTK_WINDOW(this->window));
}
