/*  BMPx - The Dumb Music Player
 *  Copyright (C) 2005 BMPx development team.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id$
 *
 * The BMPx project hereby grant permission for non-gpl compatible GStreamer
 * plugins to be used and distributed together with GStreamer and BMPx. This
 * permission are above and beyond the permissions granted by the GPL license
 * BMPx is covered by.
 *
 */

#include <config.h>

#include <glib.h>
#include <glib/gi18n.h>
#include <glib/gstdio.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <string.h>
#include <limits.h>
#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>

#include <bmp/file_utils.h>
#include <bmp/util.h>
#include <bmp/uri.h>

#include "main.h"

#include <libhrel/types.h>
#include <libhrel/tuple.h>

#include "glade.h"
#include "ui_util.h"

#include "ui_dialog_trackinfo.h"
#include "signal_handler_pool.h"

/* GObject stuff */
G_DEFINE_TYPE (BmpTrackInfo, bmp_trackinfo, G_TYPE_OBJECT)

struct _BmpTrackInfoPrivate {

    gboolean	       dispose_has_run;

    GladeXML	      *xml;
    SignalHandlerPool *pool;
};

void
window_trackinfo_create (BmpTrackInfo *self);

BmpTrackInfo*
bmp_trackinfo_new (void)
{
    BmpTrackInfo *self;

    self = g_object_new (BMP_TYPE_TRACKINFO, NULL);
    return self;
}

static GObject *
bmp_trackinfo_constructor (GType type,
	                   guint n_construct_properties,
         	           GObjectConstructParam * construct_properties)
{
    GObject *obj;

    {
        /*
         * Invoke parent constructor.
         */
        BmpTrackInfoClass *klass;
        GObjectClass *parent_class;
        klass = BMP_TRACKINFO_CLASS (g_type_class_peek (BMP_TYPE_TRACKINFO));
        parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
        obj =
            parent_class->constructor (type,
                                       n_construct_properties,
                                       construct_properties);
    }

    /*
     * do stuff.
     */
    return obj;
}

static void
bmp_trackinfo_dispose (GObject *object)
{
    BmpTrackInfo *self = (BmpTrackInfo *)object;
    signal_handler_pool_free (self->private->pool);
    g_object_unref (self->private->xml);
    gtk_widget_destroy (self->window);
}

static void
bmp_trackinfo_class_init (BmpTrackInfoClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

#if 0
    gobject_class->set_property = bmp_play_set_property;
    gobject_class->get_property = bmp_play_get_property;
    gobject_class->finalize = bmp_play_finalize;
#endif
    gobject_class->dispose	= bmp_trackinfo_dispose;
    gobject_class->constructor	= bmp_trackinfo_constructor;

}

static void
bmp_trackinfo_init (BmpTrackInfo *self)
{
    self->private = g_new (BmpTrackInfoPrivate, 1);

    self->private->dispose_has_run = FALSE;
    self->private->pool = signal_handler_pool_new ("UiTrackinfo");

    window_trackinfo_create (self);
}

static void
_entry_clear (GladeXML *xml, const gchar *widget)
{
    GtkWidget *entry; 
	
    entry = glade_xml_get_widget (xml, widget);
    gtk_entry_set_text (GTK_ENTRY (entry), "");
}

static void
_label_clear (GladeXML *xml, const gchar *widget)
{
    GtkWidget *label; 
	
    label = glade_xml_get_widget (xml, widget);
    gtk_label_set_text (GTK_LABEL(label), "");
}
 
static void
_entry_set_text (HTuple		  *tuple, 
		 GladeXML	  *xml, 
		 const gchar	  *widget, 
		 BmpMetadatumId    metadatum_id)
{
    HTupleElem	*elem;
    GtkWidget	*entry; 
    gchar	*text;
	
    entry = 
      glade_xml_get_widget (xml, widget);
    elem  = 
      h_tuple_get (tuple, bmp_metadata_get_id_static (metadatum_id));
    
    if (elem->value && (elem->type == G_TYPE_INT)) 
      {
	text = g_strdup_printf("%d", GPOINTER_TO_INT(elem->value));
      }
    else if (elem->value && (elem->type == G_TYPE_STRING) && (strlen(elem->value)>0)) 
      {
	text = g_strdup (elem->value); 
      }
    else
	text = NULL;

    if (text) 
      {
	gtk_widget_set_sensitive (entry, TRUE); 
      }
    else
      { 
	gtk_widget_set_sensitive (entry, FALSE); 
	text = g_strdup("");
      }

    if (metadatum_id == BMP_DATUM_LOCATION) 
      { 
	BMPURI *uri; 
	gchar *text_uri;

	uri = bmp_uri_new (text); 
	bmp_uri_unescape (uri);

	text_uri = bmp_uri_get_string(uri); 

	gtk_entry_set_text (GTK_ENTRY(entry), text_uri);

	g_free (text_uri); 
	bmp_uri_delete (uri);
      } 
    else 
      {
	gtk_entry_set_text (GTK_ENTRY(entry), text); 
      } 

  g_free (text);
}

static void
_label_set_text (HTuple		  *tuple, 
		 GladeXML	  *xml, 
		 const gchar	  *widget, 
		 BmpMetadatumId    metadatum_id, 
		 const gchar	  *append)
{
    HTupleElem	    *elem;
    GtkWidget	    *label; 
    gchar	    *text, 
		    *text_fq;
	
    label = glade_xml_get_widget (xml, widget);
    elem  = h_tuple_get (tuple, bmp_metadata_get_id_static (metadatum_id));
    
    if (elem->value && (elem->type == G_TYPE_INT)) 
      {
	text = g_strdup_printf("%d", GPOINTER_TO_INT(elem->value));
      }
    else if (elem->value && (elem->type == G_TYPE_STRING)) 
      {
	text = g_strdup (elem->value); 
      }
    else
	text = NULL;

    if (text) 
      {
	gtk_widget_set_sensitive (label, TRUE); 
      }
    else
      { 
	gtk_widget_set_sensitive (label, FALSE); 
	text = g_strdup("");
      }

  text_fq = g_strdup_printf("%s %s", text, append);
  gtk_label_set_text (GTK_LABEL(label), text_fq);

  g_free (text);
  g_free (text_fq); 
}

/* DnD/XDS */
static const GtkTargetEntry xds_source[] = {
    {"XdndDirectSave0",		    0, 0},
};

static void
image_drag_begin      (GtkWidget      *widget,
		       GdkDragContext *drag_context,
		       gpointer	       data)
{
    BmpTrackInfo  *self = BMP_TRACKINFO (data);
    GtkWidget	  *image;
    GdkPixbuf	  *drag_pixbuf;
    gchar	  *basename, *artist, *album;
    GdkAtom	   property, type;

    //Set the drag icon
    image = glade_xml_get_widget_warn (self->private->xml, "fi_image");
    drag_pixbuf = gdk_pixbuf_new_from_file (g_object_get_data (G_OBJECT(image), "image-filename"), NULL);

    gtk_drag_set_icon_pixbuf (drag_context,
			      drag_pixbuf,
			      0, 0); 
    g_object_ref (drag_pixbuf);

    //XDS: Set the filename 
    artist = g_object_get_data (G_OBJECT (image), "_artist");
    album  = g_object_get_data (G_OBJECT (image), "_album");
    basename = g_strdup_printf ("%s - %s", artist, album);

    //XDS: Set up atoms
    property = gdk_atom_intern ("XdndDirectSave0", FALSE);
    type = gdk_atom_intern ("text/plain", FALSE); 

    //XDS: Set property on the window
    gdk_property_change (drag_context->source_window,
                          property, type, 8, GDK_PROP_MODE_REPLACE,
                          (const guchar *) basename,
                          basename ? strlen (basename) : 0);
}

static void
image_drag_end	      (GtkWidget      *widget,
		       GdkDragContext *drag_context,
		       gpointer	       data)
{
    BmpTrackInfo  *self = BMP_TRACKINFO (data);
    GtkWidget	  *image;
    gchar	  *artist = NULL, 
		  *album = NULL, 
		  *image_filename = NULL;
    GdkAtom	   property;

    image = glade_xml_get_widget_warn (self->private->xml, "fi_image");

    //XDS: Free GObject data associated with XDS
    artist = g_object_get_data (G_OBJECT (image), "_artist");
    album  = g_object_get_data (G_OBJECT (image), "_album");
    image_filename  = g_object_get_data (G_OBJECT (image), "image-filename");

    g_free (artist);
    g_free (album);
    g_free (image_filename);

    //XDS: Set up atoms
    property = gdk_atom_intern ("XdndDirectSave0", FALSE);

    //XDS: Clear property on the window
    gdk_property_delete (drag_context->source_window, property);
}

#define MAX_URI_LEN 4096

static void
collect_uris  (HTuple	  *tuple,
	       GList	 **uris)
{
  HTupleElem  *tuple_elem;

  tuple_elem = h_tuple_get (tuple, bmp_metadata_get_id_static (BMP_DATUM_LOCATION));

  *uris = g_list_append (*uris, g_strdup (tuple_elem->value));
}

static void
image_drag_data_get   (GtkWidget	*widget,
                       GdkDragContext	*drag_context,
                       GtkSelectionData *selection_data,
                       guint		 info,
                       guint		 time,
                       gpointer		 data)
{
  BmpTrackInfo  *self = BMP_TRACKINFO (data);
  GdkAtom        property = gdk_atom_intern ("XdndDirectSave0", FALSE);
  GdkAtom        type     = gdk_atom_intern ("text/plain", FALSE);
  guchar        *dnddata;
  gchar         *uri, 
		*pathname, 
		*host;
  GError        *error = NULL;
  gint           length;

  g_return_if_fail (GDK_IS_DRAG_CONTEXT (drag_context));

  if (! gdk_property_get (drag_context->source_window, property, type,
                          0, MAX_URI_LEN, FALSE,
                          NULL, NULL, &length, &dnddata))
    return;


  uri = g_strndup ((const gchar *) dnddata, length);
  pathname = g_filename_from_uri (uri, &host, &error);  
  g_free (dnddata);

  if (!host && (!g_mkdir (pathname, 0700)))
    {
	GtkWidget *image,
		  *label,
		  *vbox,
		  *pbar,
		  *window;

	HRelation *result;
	Query	  *query;
	gchar	  *artist,
		  *album;
	GList	  *uris = NULL;
	GList	  *files = NULL;
	gint	   n = 1, length;
	gdouble	   percent;

	gtk_selection_data_set (selection_data, selection_data->target, 8,
                                      (const guchar *) "S", 1);

	window = glade_xml_get_widget_warn (self->private->xml, "dialog_file_information");
	image = glade_xml_get_widget_warn (self->private->xml, "fi_image");

	//Now start the copying action
        artist = g_object_get_data (G_OBJECT (image), "_artist");
	album  = g_object_get_data (G_OBJECT (image), "_album");

	query = bmp_metadata_cache_query_initialize (bmp_metadata_cache);

	bmp_metadata_cache_query_append_string (bmp_metadata_cache,
						query,
					        BMP_DATUM_ARTIST,
						TRUE,
						artist);			      
  
	bmp_metadata_cache_query_append_string (bmp_metadata_cache,
						query,
					        BMP_DATUM_ALBUM,
						TRUE,
						album);			      

	result = bmp_metadata_cache_query_run  (bmp_metadata_cache,
						query);

	h_relation_for_each_tuple (result,
				   (HTupleFunc) collect_uris,
				   &uris);

	for ( ; uris ; uris = uris->next)	
	  {
	    BMPURI *_uri;

	    _uri = bmp_uri_new (uris->data);
	    bmp_uri_unescape (_uri);
	    files = g_list_append (files, g_strdup(_uri->path));   	    
	    bmp_uri_delete (_uri);
	  }
	g_list_foreach (g_list_first(uris), (GFunc)g_free, NULL);

	label = glade_xml_get_widget_warn (self->private->xml, "copying_label");
	vbox  = glade_xml_get_widget_warn (self->private->xml, "copying_vbox");
	pbar  = glade_xml_get_widget_warn (self->private->xml, "copying_pb");

	gtk_widget_show_all (vbox);

	length = g_list_length (files);
	percent = 1.0/(gdouble)length;

	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (pbar), 0.0); 

	for ( ; files ; files = files->next)	
	  {
	    gchar *basename, *destination, *text;
  
	    text = g_strdup_printf ("Copying Track %d/%d", n, length); 
	    gtk_label_set_text (GTK_LABEL(label), text);
	    g_free (text);

	    basename = g_path_get_basename (files->data);
	    destination = g_build_filename (pathname, basename, NULL);

	    copy_file (files->data, destination);

	    gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (pbar), ((percent * n)>1.0) ? 1.0 : (percent * n));
	    n++;

	    g_free (basename);
	    g_free (destination);
	  }
	g_list_foreach (g_list_first(files), (GFunc)g_free, NULL);

	//This will give the user some feedback that the operation succeeded
	gtk_label_set_text (GTK_LABEL(label), "Done");
	g_usleep (2000000);

	h_relation_unref (result);
	gtk_widget_hide (vbox);
	gtk_window_reshow_with_initial_size (GTK_WINDOW(window));
    }
  else
    {
	gtk_selection_data_set (selection_data, selection_data->target, 8,
                                      (const guchar *) "E", 1);
    }

   g_free (uri);
}

/* END DnD/XDS */

void
window_trackinfo_create (BmpTrackInfo *self)
{
    GtkWidget *widget;

    const gchar	*glade_file = DATA_DIR "/glade/dialog_file_info.glade";

    self->private->xml = glade_xml_new_or_die("File Info Dialog", glade_file, NULL, NULL);
    glade_xml_signal_autoconnect (self->private->xml);

    self->window = glade_xml_get_widget(self->private->xml, "dialog_file_information");
    gtk_window_set_position (GTK_WINDOW(self->window), GTK_WIN_POS_CENTER_ALWAYS);

    /* Set icon to window */
    bmp_window_set_icon_list (GTK_WIDGET(self->window), "player");
  
    signal_handler_pool_add_connection (self->private->pool,
				        glade_xml_get_widget(self->private->xml,"fi_ok"), 	
					"clicked", 
					G_CALLBACK(gtk_widget_hide), 
				        self->window,
					G_CONNECT_SWAPPED);	

    widget = glade_xml_get_widget (self->private->xml, "fi_image");
    gtk_image_set_from_file (GTK_IMAGE (widget), DATA_DIR "/images/audio.png"); 
    g_object_set_data (G_OBJECT (widget), "image-filename", NULL);

    widget = glade_xml_get_widget (self->private->xml, "fi_image_eventbox");
    signal_handler_pool_add_connection (self->private->pool,
				        widget, 
					"drag-begin", 
					G_CALLBACK(image_drag_begin), 
				        self,
					0);	

    signal_handler_pool_add_connection (self->private->pool,
				        widget, 
					"drag-end", 
					G_CALLBACK(image_drag_end), 
				        self,
					0);	

    signal_handler_pool_add_connection (self->private->pool,
				        widget, 
					"drag-data-get", 
					G_CALLBACK(image_drag_data_get), 
				        self,
					0);	
}

void
bmp_trackinfo_show (BmpTrackInfo	*self,
		    BmpFileItem		*item) 
{
    GdkPixbuf	    *cover, *scale;
    GtkWidget	    *image;

    gchar	    *artist_lc,
		    *album_lc;

    gchar	    *thumb_filename,
		    *thumb_path; 

    HTuple	    *tuple;

    HTupleElem	    *elem_artist,
		    *elem_album,
		    *elem_location;

    bmp_trackinfo_clear (self);

    tuple = bmp_metadata_cache_metadata_get (bmp_metadata_cache, item);
    image = glade_xml_get_widget_warn (self->private->xml, "fi_image");
    gtk_widget_set_size_request (GTK_WIDGET(self->window), 600, -1);
    gtk_widget_realize (GTK_WIDGET(self->window));
    gtk_widget_show (self->window);

    if (!tuple)
      {
	return;
      }

    gtk_widget_set_sensitive (GTK_WIDGET(self->window), FALSE);
    bmp_window_set_busy (GTK_WIDGET(self->window));

    _entry_set_text(tuple, self->private->xml, "entry_file",	BMP_DATUM_LOCATION);
    _entry_set_text(tuple, self->private->xml, "entry_artist",  BMP_DATUM_ARTIST);
    _entry_set_text(tuple, self->private->xml, "entry_album",   BMP_DATUM_ALBUM);
    _entry_set_text(tuple, self->private->xml, "entry_title",   BMP_DATUM_TITLE);
    _entry_set_text(tuple, self->private->xml, "entry_genre",   BMP_DATUM_GENRE);
    _entry_set_text(tuple, self->private->xml, "entry_date",    BMP_DATUM_DATE);
    _entry_set_text(tuple, self->private->xml, "entry_comment", BMP_DATUM_COMMENT);
    _entry_set_text(tuple, self->private->xml, "entry_track",   BMP_DATUM_TRACK);

    _label_set_text(tuple, self->private->xml, "label_bitrate", BMP_DATUM_BITRATE, "kbit/s");

    elem_artist = h_tuple_get (tuple, bmp_metadata_get_id_static (BMP_DATUM_ARTIST));
    elem_album = h_tuple_get (tuple, bmp_metadata_get_id_static (BMP_DATUM_ALBUM));
    elem_location = h_tuple_get (tuple, bmp_metadata_get_id_static (BMP_DATUM_LOCATION));

    g_object_set_data (G_OBJECT (image), "_artist", g_strdup(elem_artist->value));
    g_object_set_data (G_OBJECT (image), "_album", g_strdup(elem_album->value));

    if (((elem_album->value) && (strlen(elem_album->value)>0)) && !g_uri_test(elem_location->value, BMP_URI_TEST_PROTOCOL, PROTOCOL_HTTP))
      {
        gtk_drag_source_set (glade_xml_get_widget_warn (self->private->xml, "fi_image_eventbox"),
			 GDK_MODIFIER_MASK,
			 xds_source,
			 1,
			 GDK_ACTION_COPY);

#ifdef USE_AMAZON
	artist_lc = g_utf8_strdown (elem_artist->value, strlen (elem_artist->value));
	album_lc = g_utf8_strdown (elem_album->value, strlen (elem_album->value));

	thumb_filename = g_strconcat (artist_lc, "_", album_lc, ".png", NULL);
	thumb_path = g_build_filename (BMP_PATH_COVER_THUMB_DIR, thumb_filename, NULL);
	g_free (thumb_filename);

	if (g_file_test (thumb_path, G_FILE_TEST_EXISTS))
	  {
	     gtk_image_set_from_file (GTK_IMAGE (image), thumb_path);
	  }
	else if (strcmp(elem_album->value, bmp_metadata_get_default_char (BMP_DATUM_ALBUM)))
	  {
	    cover = bmp_metadata_cache_get_cover_from_tuple (bmp_metadata_cache, tuple);
	    if (cover)
	      { 
		scale = gdk_pixbuf_scale_simple (cover, 128, 128, GDK_INTERP_HYPER);  
		gtk_image_set_from_pixbuf (GTK_IMAGE (image), scale);
		g_object_unref (cover);

		gdk_pixbuf_save (scale,
			       thumb_path,
			       "png",
			       NULL,
			       NULL);
		g_object_unref (scale);
	      }
	    else
	      {
		gtk_image_set_from_file (GTK_IMAGE (image), DATA_DIR "/images/audio.png"); 
	      }
	  }
	g_object_set_data (G_OBJECT (image), "image-filename", thumb_path);
#endif
      }
    else
      {
        gtk_drag_source_unset (glade_xml_get_widget_warn (self->private->xml, "fi_image_eventbox"));
	gtk_image_set_from_file (GTK_IMAGE (image), DATA_DIR "/images/audio.png"); 
      }

    gtk_widget_set_sensitive (self->window, TRUE);
    bmp_window_set_idle (GTK_WIDGET(self->window));
    g_boxed_free (BMP_TYPE_FILE_ITEM, item); 
}

void
bmp_trackinfo_clear (BmpTrackInfo *self)
{
    GtkWidget *image;

    _entry_clear(self->private->xml, "entry_file");
    _entry_clear(self->private->xml, "entry_artist");
    _entry_clear(self->private->xml, "entry_album");
    _entry_clear(self->private->xml, "entry_title");
    _entry_clear(self->private->xml, "entry_genre");
    _entry_clear(self->private->xml, "entry_date");
    _entry_clear(self->private->xml, "entry_comment");
    _entry_clear(self->private->xml, "entry_track");

    _label_clear(self->private->xml, "label_bitrate");

    image = glade_xml_get_widget(self->private->xml, "fi_image");
    gtk_image_set_from_file (GTK_IMAGE (image), DATA_DIR "/images/audio.png"); 
}


