#include <config.h>

#include <glib.h>
#include <glib/gi18n.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include <bmp/metadata.h>

#include "main.h"

#include "ui_util.h"
#include "ui_splash.h"

static GtkWidget *splash_window = NULL;
static GdkPixbuf *splash_pixbuf = NULL;

static gint splash_width = 0;
static gint splash_height = 0;

static gboolean has_alpha = FALSE;

#define BAR_X		(10)
#define BAR_Y		(splash_height - 38)
#define BAR_WIDTH	(splash_width - 20)
#define BAR_HEIGHT	(15)

static gboolean
on_splash_window_expose (GtkWidget	  *widget,
			 GdkEventExpose	  *event,
			 gpointer	   data)
{
  cairo_t *cr;

  cr = gdk_cairo_create (widget->window);

  cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);

  if (has_alpha)
    {
      cairo_set_source_rgba (cr, .0, .0, .0, .0);
      cairo_paint (cr);
    }

  gdk_cairo_set_source_pixbuf (cr, splash_pixbuf, 0, 0);
  cairo_paint (cr);
  cairo_destroy (cr);


  return FALSE;
}

void
bmp_splash_window_show(void)
{
  GdkPixmap	*mask_pixmap_window1 = NULL,
		*mask_pixmap_window2 = NULL;
  GdkBitmap	*mask_bitmap_window1 = NULL,
		*mask_bitmap_window2 = NULL;
  gchar		*filename;
  GdkScreen	*screen;
  GdkColormap	*colormap;
  GdkVisual	*visual;

  if (splash_window) return;

  screen = gdk_screen_get_default (); 
  visual = gdk_screen_get_rgba_visual (screen);

  if (!visual)
    {
      colormap = NULL;
      has_alpha = FALSE;
    }
  else
    {
      colormap = gdk_screen_get_rgba_colormap (screen);
      has_alpha = TRUE;
    }

  splash_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_type_hint(GTK_WINDOW(splash_window),
			   GDK_WINDOW_TYPE_HINT_SPLASHSCREEN);

  filename = DATA_DIR G_DIR_SEPARATOR_S "images" G_DIR_SEPARATOR_S "splash.png";
  splash_pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
  splash_width = gdk_pixbuf_get_width (splash_pixbuf);
  splash_height = gdk_pixbuf_get_height (splash_pixbuf);

  gtk_widget_set_size_request (GTK_WIDGET (splash_window),
			       splash_width,
			       splash_height);

  gtk_widget_add_events (splash_window, GDK_ALL_EVENTS_MASK);
  g_signal_connect (splash_window,
		    "expose_event",
		    G_CALLBACK(on_splash_window_expose),
		    NULL);
  
  if (has_alpha)
    {
      gtk_widget_set_colormap (splash_window, colormap);
    }
  else
    {
      gdk_pixbuf_render_pixmap_and_mask (splash_pixbuf,
				     &mask_pixmap_window1,
				     &mask_bitmap_window1,
				     0);

      gdk_pixbuf_render_pixmap_and_mask (splash_pixbuf,
				     &mask_pixmap_window2,
				     &mask_bitmap_window2,
				     128);
      gtk_widget_shape_combine_mask (GTK_WIDGET (splash_window), mask_bitmap_window2, 0, 0);
    }

  gtk_widget_set_app_paintable (splash_window, TRUE);
  gtk_window_set_decorated (GTK_WINDOW (splash_window), FALSE);
  bmp_window_set_icon_list (GTK_WIDGET(splash_window), "player");
  gtk_window_set_title (GTK_WINDOW (splash_window), "BMPx Starting...");
  gtk_window_set_position (GTK_WINDOW (splash_window), GTK_WIN_POS_CENTER);
  gtk_window_set_resizable (GTK_WINDOW (splash_window), FALSE);

  gtk_widget_show_all(splash_window);
  gdk_window_raise (splash_window->window);
}

void
bmp_splash_window_hide (void)
{
  if (splash_window)
    {
      gtk_widget_hide (splash_window);
      gtk_widget_destroy (splash_window);
      g_object_unref (splash_pixbuf);
      splash_window = NULL;
    }
}
  
