#ifndef UI_UTIL_H
#define UI_UTIL_H

#include <config.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include "ui.h"

typedef enum {
	BMP_SKINCOLOR_PL_BG_NORMAL,
	BMP_SKINCOLOR_PL_FG_NORMAL,
	BMP_SKINCOLOR_PL_BG_SELECTED,
   BMP_SKINCOLOR_PL_FG_CURRENT,
	BMP_N_SKINCOLORS
} BmpSkinColor;

typedef enum {
    CURSOR_NORMAL,
    CURSOR_POSBAR,
    CURSOR_VOLBAR,
    CURSOR_PLAYLIST,
    CURSOR_PLAYLIST_SIZE,
    CURSOR_MAINMENU,
    CURSOR_N_CURSORS
} CursorId;

typedef enum {
    SKIN_PLEDIT_NORMAL = 0,
    SKIN_PLEDIT_CURRENT,
    SKIN_PLEDIT_NORMALBG,
    SKIN_PLEDIT_SELECTEDBG,
    SKIN_TEXTBG,
    SKIN_TEXTFG,
    SKIN_N_COLORS
} SkinColorId;


GdkPixbuf*
bmp_pixbuf_new_simple(guint width, guint height);

void
bmp_pixbuf_copy_area_simple(GdkPixbuf *dest, guint x, guint y, guint width, guint height);

void
bmp_draw_pixbuf_simple(cairo_t *cr, GdkPixbuf *buf, guint x, guint y);

/* Cursors */
void
bmp_cursors_init    (void);
 
void
bmp_cursors_free    (void);

void
bmp_cursors_set	    (const gchar *archive_dirname);

void
bmp_cursor_add	    (const gchar *archive_dirname, gint cursor_enum);

void
bmp_window_set_idle (GtkWidget *widget);

void
bmp_window_set_busy (GtkWidget *widget);
 
GdkCursor*
bmp_cursor_get (gint cursor_enum);

/* XShape Masks */
GdkBitmap*
bmp_get_mask_window_main(const gchar *path);

/* Colors */
gchar*
bmp_skin_get_color(const gchar *path, const gchar *key);

GdkColor*
bmp_skin_convert_color(const gchar *value_arg);

void
bmp_colors_set_color(BmpSkinColor color, const gchar *text);

void
bmp_colors_init(const gchar *archive_dirname);

void
bmp_colors_free(void);

void
bmp_color_allocate(BmpSkinColor color, GtkWidget *widget);

GdkColor*
bmp_colors_get_color(BmpSkinColor color);

GdkColor*
bmp_colors_change_brightness(GdkColor *original, GtkWidget *widget, gfloat delta);

void
gdk_color_to_rgba (GdkColor *color, double *r, double *g, double *b, double *a);

void
draw_polygon (cairo_t *cr, GdkColor *color, double alpha, BmpPoint points[], gint n_points, gboolean fill);

void
draw_rounded_rectangle_pattern (cairo_t *cr, double xc, double yc, double _w, double _h, cairo_pattern_t *cr_pattern, double line_width, gboolean fill, gdouble rounding_offset);

void
draw_rounded_rectangle (cairo_t *cr, double xc, double yc, double _w, double _h, GdkColor *color, double alpha, double line_width, gboolean fill, gdouble rounding_offset);

#if 0
BmpFontInformation*
bmp_get_font_information_pfd (const PangoFontDescription *font_desc, GtkWidget *widget);

BmpFontInformation*
bmp_get_font_information (const gchar *fontname, GtkWidget *widget);
#endif

void
bmp_window_set_icon_list (GtkWidget *window, const gchar *icon_name);

/* UI functions */

gdouble screen_get_resolution   (GdkScreen *screen);
gdouble screen_get_x_resolution (GdkScreen *screen);
gdouble screen_get_y_resolution (GdkScreen *screen);

#if ENABLE_NLS
gchar *bmp_menu_translate (const gchar * path, gpointer func_data);
#else
#  define bmp_menu_translate NULL
#endif

GtkWidget*
ui_manager_get_popup (GtkUIManager *self,
                      const gchar  *path);

void
bmp_menu_popup (GtkMenu *menu,
		     gint     x,
		     gint     y,
		     guint    button,
		     guint    time);

#if 0
void
draw_polygon (cairo_t *cr, GdkColor *color, double alpha, BmpPoint points[], gint n_points, gboolean fill);
#endif

#endif
