/*  BMPx - The Dumb Music Player
 *  Copyright (C) 2005 BMPx development team.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * The BMPx project hereby grant permission for non-gpl compatible GStreamer
 * plugins to be used and distributed together with GStreamer and BMPx. This
 * permission are above and beyond the permissions granted by the GPL license
 * BMPx is covered by.
 */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <strings.h>
#include <glib/gstdio.h>

#ifdef ENABLE_NLS
# include <libintl.h>
#endif

#include <bmp/util.h>
#include <bmp/uri.h>
#include <bmp/vfs.h>
#include <bmp/plugin.h>
#include <bmp/plugin_interfaces.h>
#include <bmp/metadata.h>

#include <xcs/xcs.h>

#include "main.h"

#include "resource_manager.h"

#include "loader.h"

#ifdef HAVE_FTS_H
#  include <fts.h>
#endif

#ifdef HAVE_GUI
#  include <ui.h>
#  include <ui_playlist.h>
#endif

typedef struct {

    GList *local_uris; 
    gchar *uri_original;

} CleanupTuple;

static GList *cleanups = NULL;
#if 0
static guint  guid = -1;
#endif

CleanupTuple*
cleanup_new (const gchar *uri_original)
{
    CleanupTuple *tuple;

    tuple = 
	g_new0 (CleanupTuple,1); 
    tuple->local_uris = NULL;
    tuple->uri_original = 
	g_strdup (uri_original);
    cleanups = g_list_append (cleanups, tuple);
    
    return tuple;
}

void
cleanup_add_uri (CleanupTuple *tuple, const gchar *uri)
{
    tuple->local_uris = g_list_append (tuple->local_uris, g_strdup(uri)); 
}

void
cleanup_clean_up (CleanupTuple *tuple)
{
    for ( ; tuple->local_uris; tuple->local_uris = tuple->local_uris->next)
      {
	gchar *filename;

	filename = g_filename_from_uri (tuple->local_uris->data, NULL, NULL);
	g_unlink (filename);
      }
    g_free (tuple->uri_original);
    g_free (tuple);
}

/* VFS begin */
void
vfs_init(void)
{}

void
vfs_stop(void)
{
    g_list_foreach (cleanups, (GFunc)cleanup_clean_up, NULL);
}

VFSHandle*
vfs_handle_new (void)
{
    VFSHandle *handle;

    handle = g_new0 (VFSHandle,1);
    handle->uri_list = NULL;
    handle->uri = NULL; 
    handle->filename = NULL;
    handle->valid = TRUE;
    handle->need_container = TRUE;
    handle->buffer = NULL;
    handle->buffer_size = 0;

    return handle;
}


static gboolean
find_transport (gpointer key,
                gpointer value,
                gpointer user_data)
{
    VFSHandle *handle = (VFSHandle*)user_data;

    if (bmp_plugin_transport_can_process (value, handle->uri))
	return TRUE;

    return FALSE;
}

static gboolean
find_container (gpointer key,
                gpointer value,
                gpointer user_data)
{
    VFSHandle *handle = (VFSHandle*)user_data;

    if (bmp_plugin_container_can_process (value, handle->uri))	return TRUE;
    return FALSE;
}

gboolean
vfs_handle_write (VFSHandle *handle, const gchar *container)
{
    GObject *plugin_container = NULL;

    plugin_container = g_hash_table_lookup (plugins_table[BMP_PLUGIN_TYPE_CONTAINER], container);

    if (!plugin_container)
	return FALSE;

    bmp_plugin_container_handle_set ((BmpPluginContainerInterface*)plugin_container, handle);

    return bmp_plugin_container_handle_write ((BmpPluginContainerInterface*)plugin_container);
}

gboolean
vfs_handle_read (VFSHandle *handle, gint need_container)
{
    GObject *plugin_transport = NULL;
    GObject *plugin_container = NULL;

    g_return_val_if_fail (handle != NULL, FALSE);
    g_return_val_if_fail (VFS_HANDLE_VALID(handle), FALSE);

    plugin_transport = g_hash_table_find (plugins_table[BMP_PLUGIN_TYPE_TRANSPORT], (GHRFunc) find_transport, handle);
    if (!plugin_transport)
      {
	return FALSE;
      }
    else
      {
	if (!bmp_plugin_transport_handle_set  ((BmpPluginTransportInterface*)plugin_transport, handle))
	  {
	    g_log (G_LOG_DOMAIN, G_LOG_LEVEL_CRITICAL, "%s: Unable to set handle (URI: %s) to transport plugin", G_STRLOC, handle->uri);
	    return FALSE;
	  }
	if (!bmp_plugin_transport_handle_read ((BmpPluginTransportInterface*)plugin_transport))
	  {
	    g_log (G_LOG_DOMAIN, G_LOG_LEVEL_CRITICAL, "%s: Unable to process handle (URI: %s) with transport plugin", G_STRLOC, handle->uri);
	    return FALSE;
	  }
      }    

    if ((!handle->need_container) || (need_container == VFS_NEED_CONTAINER_FORCE_NONE))	return TRUE;

    plugin_container = g_hash_table_find (plugins_table[BMP_PLUGIN_TYPE_CONTAINER], (GHRFunc) find_container, handle);
    if (!plugin_container)
      {
        g_log (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING, "%s: No container found for URI '%s'", G_STRLOC, handle->uri);
	return FALSE;
      }
    else
      {
	if (!bmp_plugin_container_handle_set  ((BmpPluginContainerInterface*)plugin_container, handle))
	  {
	    g_log (G_LOG_DOMAIN, G_LOG_LEVEL_CRITICAL, "%s: Unable to set handle (URI: %s) to container plugin", G_STRLOC, handle->uri);
	    return FALSE;
	  }

	if (!bmp_plugin_container_handle_read ((BmpPluginContainerInterface*)plugin_container))
	  {
	    g_log (G_LOG_DOMAIN, G_LOG_LEVEL_CRITICAL, "%s: Unable to process handle (URI: %s) with container plugin", G_STRLOC, handle->uri);
	    return FALSE;
	  }
      }    

    return TRUE;
}

void
vfs_handle_free (VFSHandle *handle)
{
    g_free (handle->uri);
    g_free (handle->filename);
    g_list_foreach (g_list_first(handle->uri_list), (GFunc)g_free, NULL);
    g_free (handle->buffer);
    g_free (handle);
}

#if 0
/* URI access functions */
VFSFileHandle*
vfs_file_handle_new (const gchar *uri);

#define vfs_file_handle_get_location(handle) (const(handle->handle_path))
#endif
