/*  BMPx - The Dumb Music Player
 *  Copyright (C) 2005 BMPx development team.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * The BMPx project hereby grant permission for non-gpl compatible GStreamer
 * plugins to be used and distributed together with GStreamer and BMPx. This
 * permission are above and beyond the permissions granted by the GPL license
 * BMPx is covered by.
 */

#ifndef _XCS_H_
#define _XCS_H_

#ifdef HAVE_GUI
#  include <gtk/gtk.h>
#endif

#include <glib-object.h>
#include <config.h>

G_BEGIN_DECLS

#define XCS_TYPE_CONFIGURATION		  (xcs_get_type ())
#define XCS_CONFIGURATION(obj)		  (G_TYPE_CHECK_INSTANCE_CAST ((obj), XCS_TYPE_CONFIGURATION, Xcs))
#define XCS_CONFIGURATION_CLASS(klass)	  (G_TYPE_CHECK_CLASS_CAST ((klass), XCS_TYPE_CONFIGURATION, XcsClass))
#define XCS_IS_CONFIGURATION(obj)	  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), XCS_TYPE_CONFIGURATION))
#define XCS_IS_CONFIGURATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), XCS_TYPE_CONFIGURATION))
#define XCS_CONFIGURATION_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), XCS_TYPE_CONFIGURATION, XcsClass))

typedef struct _Xcs Xcs;
typedef struct _XcsClass XcsClass;
typedef struct _XcsPrivate XcsPrivate;

struct _Xcs {
    GObject parent;

    XcsPrivate *priv;
};

struct _XcsClass {
    GObjectClass parent;

    void (*removed) (Xcs *self);
    void (*changed) (Xcs *self);
    void (*added) (Xcs *self);
};

GType
xcs_get_type (void);

Xcs*
xcs_new (gboolean *new_configuration);

gboolean
xcs_save (Xcs *self);

#ifndef USE_GCONF
gboolean
xcs_domain_new	  (Xcs *configuration, const gchar * domain_id);
gboolean
xcs_domain_exists (Xcs *configuration, const gchar * domain_id);
#endif

gboolean
xcs_key_set (Xcs	    *self,
             const gchar    *domain_id,
             const gchar    *key_id,
             GValue	    *key_value);

gchar*
xcs_key_fetch_string (Xcs	   *self,
		      const gchar  *domain_id,
		      const gchar  *key_id);
#define xcs_key_fetch(a,b,c) xcs_key_fetch_string (a,b,c)

gint
xcs_key_fetch_int (Xcs		*self,
                   const gchar	*domain_id,
                   const gchar	*key_id);
gboolean
xcs_key_fetch_boolean (Xcs	    *self,
                       const gchar  *domain_id,
                       const gchar  *key_id);

gdouble
xcs_key_fetch_float   (Xcs	    *self,
                       const gchar  *domain_id,
                       const gchar  *key_id);



void
xcs_key_set_string (Xcs		  *self,
                    const gchar	  *domain_id,
                    const gchar	  *key_id,
                    const gchar	  *value_arg);
void
xcs_key_set_int (Xcs		*self,
                 const gchar	*domain_id,
                 const gchar	*key_id,
                 gint		 value_arg);

void
xcs_key_set_boolean (Xcs	    *self,
                     const gchar    *domain_id,
                     const gchar    *key_id,
                     gboolean	     value_arg);

void
xcs_key_set_float (Xcs		  *self,
                   const gchar    *domain_id,
                   const gchar    *key_id,
                   gdouble	   value_arg);


typedef void (*BindingDataFunc) (GObject *object, GValue *value);

void
xcs_bind_object (Xcs		  *self, 
		 GObject	  *object, 
		 BindingDataFunc   func, 
		 const gchar	  *signal_spec, 
		 const gchar	  *domain, 
		 const gchar	  *key);

#ifdef HAVE_GUI
/* Convenience functions since we deal a lot with ToggleActions and ToggleButtons */
void
xcs_bind_toggle_action (Xcs *self, 
				      GtkToggleAction *action, 
				      const gchar *domain, 
				      const gchar *key);

void
xcs_bind_toggle_button (Xcs *self, 
				      GtkToggleButton *button, 
				      const gchar *domain, 
				      const gchar *key);

void
xcs_bind_entry	      (Xcs		*configuration,
		       GtkEntry		*entry,
		       const gchar	*domain,
		       const gchar	*key);

void
xcs_bind_spin_button  (Xcs		*configuration,
		       GtkSpinButton	*sbutton,
		       const gchar	*domain,
		       const gchar	*key);
#endif

G_END_DECLS
 
#endif /* _XCS_H_ */
